/* Generated by Cython 0.25.1 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "define_macros": [
            [
                "FASTIMAGE_IPP_ARCH", 
                "\"intel64\""
            ], 
            [
                "FASTIMAGE_STATICIPP", 
                "1"
            ]
        ], 
        "depends": [
            "src/FastImage_macros.h"
        ], 
        "extra_objects": [
            "/opt/intel//compilers_and_libraries_2017.0.102/mac/ipp/lib/libippi.a", 
            "/opt/intel//compilers_and_libraries_2017.0.102/mac/ipp/lib/libipps.a", 
            "/opt/intel//compilers_and_libraries_2017.0.102/mac/ipp/lib/libippcv.a", 
            "/opt/intel//compilers_and_libraries_2017.0.102/mac/ipp/lib/libippcc.a", 
            "/opt/intel//compilers_and_libraries_2017.0.102/mac/ipp/lib/libippcore.a"
        ], 
        "include_dirs": [
            "/opt/intel//compilers_and_libraries_2017.0.102/mac/ipp/include"
        ]
    }, 
    "module_name": "motmot.FastImage.FastImage"
}
END: Cython Metadata */

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03020000)
    #error Cython requires Python 2.6+ or Python 3.2+.
#else
#define CYTHON_ABI "0_25_1"
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x03030000 || (PY_MAJOR_VERSION == 2 && PY_VERSION_HEX >= 0x02070000)
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
#endif
#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_FASTCALL
  #define METH_FASTCALL 0x80
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject **args,
                                              Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && METH_FASTCALL == PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST)))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None)) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? PyMethod_New(func, self) : PyInstanceMethod_New(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)

#ifndef CYTHON_INLINE
  #if defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif


#define __PYX_ERR(f_index, lineno, Ln_error) \
{ \
  __pyx_filename = __pyx_f[f_index]; __pyx_lineno = lineno; __pyx_clineno = __LINE__; goto Ln_error; \
}

#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__FastImage
#define __PYX_HAVE_API__FastImage
#include "stdlib.h"
#include "math.h"
#include "ads_wrap_system.h"
#include "stdio.h"
#include "fi_ipp.h"
#include "fic.h"
#include "unistd.h"
#include "limits.h"
#include "FastImage_macros.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#ifdef PYREX_WITHOUT_ASSERTIONS
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER) && defined (_M_X64)
    #define __Pyx_sst_abs(value) _abs64(value)
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyObject_AsSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#if PY_MAJOR_VERSION < 3
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u)
{
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#else
#define __Pyx_Py_UNICODE_strlen Py_UNICODE_strlen
#endif
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
#define __Pyx_PyBool_FromLong(b) ((b) ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False))
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c));
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */

static PyObject *__pyx_m;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "src/FastImage.pyx",
};

/*--- Type declarations ---*/
struct __pyx_obj_9FastImage_Size;
struct __pyx_obj_9FastImage_Point;
struct __pyx_obj_9FastImage_FastImageBase;
struct __pyx_obj_9FastImage_FastImage8u;
struct __pyx_obj_9FastImage_FastImage32f;
struct __pyx_t_9FastImage_PyArrayInterface;
typedef struct __pyx_t_9FastImage_PyArrayInterface __pyx_t_9FastImage_PyArrayInterface;

/* "FastImage.pxd":7
 * cimport fic
 * 
 * ctypedef void* fiptr # pointer to FastImage base type             # <<<<<<<<<<<<<<
 * 
 * cdef class Size:
 */
typedef void *__pyx_t_9FastImage_fiptr;

/* "FastImage.pyx":97
 *         raise FicError('error %d: %s'%(errval,errstr))
 * 
 * ctypedef struct PyArrayInterface:             # <<<<<<<<<<<<<<
 *     int two                       # contains the integer 2 as a sanity check
 *     int nd                        # number of dimensions
 */
struct __pyx_t_9FastImage_PyArrayInterface {
  int two;
  int nd;
  char typekind;
  int itemsize;
  int flags;
  Py_intptr_t *shape;
  Py_intptr_t *strides;
  void *data;
};

/* "FastImage.pxd":9
 * ctypedef void* fiptr # pointer to FastImage base type
 * 
 * cdef class Size:             # <<<<<<<<<<<<<<
 *     cdef ipp.IppiSize sz
 * 
 */
struct __pyx_obj_9FastImage_Size {
  PyObject_HEAD
  IppiSize sz;
};


/* "FastImage.pxd":12
 *     cdef ipp.IppiSize sz
 * 
 * cdef class Point:             # <<<<<<<<<<<<<<
 *     cdef ipp.IppiPoint pt
 * 
 */
struct __pyx_obj_9FastImage_Point {
  PyObject_HEAD
  IppiPoint pt;
};


/* "FastImage.pxd":15
 *     cdef ipp.IppiPoint pt
 * 
 * cdef class FastImageBase:             # <<<<<<<<<<<<<<
 *     cdef fiptr im
 *     cdef c_python.Py_intptr_t shape[2] # don't use, purely for making __array_struct__ without extra malloc
 */
struct __pyx_obj_9FastImage_FastImageBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_9FastImage_FastImageBase *__pyx_vtab;
  __pyx_t_9FastImage_fiptr im;
  Py_intptr_t shape[2];
  Py_intptr_t strides[2];
  int step;
  int view;
  PyObject *basetype;
  struct __pyx_obj_9FastImage_Size *imsiz;
  PyObject *source_data;
};


/* "FastImage.pxd":29
 * cdef class FastImage32f(FastImageBase) # forward definition
 * 
 * cdef class FastImage8u(FastImageBase):             # <<<<<<<<<<<<<<
 *     cdef void fast_get_absdiff_put(self, FastImage8u other, FastImage8u result, Size size)
 *     cdef void fast_get_sub_put(self, FastImage8u other, FastImage8u result, Size size)
 */
struct __pyx_obj_9FastImage_FastImage8u {
  struct __pyx_obj_9FastImage_FastImageBase __pyx_base;
};


/* "FastImage.pxd":27
 *     cdef FastImageBase c_roi( self, int left, int bottom, Size size)
 * 
 * cdef class FastImage32f(FastImageBase) # forward definition             # <<<<<<<<<<<<<<
 * 
 * cdef class FastImage8u(FastImageBase):
 */
struct __pyx_obj_9FastImage_FastImage32f {
  struct __pyx_obj_9FastImage_FastImageBase __pyx_base;
};



/* "FastImage.pyx":185
 *             return self.pt.y
 * 
 * cdef class FastImageBase:             # <<<<<<<<<<<<<<
 *     def __cinit__(self,*args,**kw):
 *         self.view = 1
 */

struct __pyx_vtabstruct_9FastImage_FastImageBase {
  struct __pyx_obj_9FastImage_FastImageBase *(*c_roi)(struct __pyx_obj_9FastImage_FastImageBase *, int, int, struct __pyx_obj_9FastImage_Size *);
};
static struct __pyx_vtabstruct_9FastImage_FastImageBase *__pyx_vtabptr_9FastImage_FastImageBase;


/* "FastImage.pyx":545
 *         return dest
 * 
 * cdef class FastImage32f(FastImageBase):             # <<<<<<<<<<<<<<
 * 
 *     def __cinit__(self,*args,**kw):
 */

struct __pyx_vtabstruct_9FastImage_FastImage32f {
  struct __pyx_vtabstruct_9FastImage_FastImageBase __pyx_base;
  void (*fast_toself_add_weighted_8u)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_Size *, float);
  void (*fast_toself_add_weighted_32f)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_Size *, float);
  void (*fast_get_8u_copy_put)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_Size *);
  void (*fast_toself_square)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_Size *);
  void (*fast_get_square_put)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_Size *);
  void (*fast_get_subtracted_put)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_Size *);
  void (*fast_toself_multiply)(struct __pyx_obj_9FastImage_FastImage32f *, float, struct __pyx_obj_9FastImage_Size *);
  struct __pyx_obj_9FastImage_FastImage32f *(*create_equal_shape_empty)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_Size *);
};
static struct __pyx_vtabstruct_9FastImage_FastImage32f *__pyx_vtabptr_9FastImage_FastImage32f;


/* "FastImage.pyx":338
 * 
 * cdef class FastImage32f(FastImageBase) # forward declaration
 * cdef class FastImage8u(FastImageBase):             # <<<<<<<<<<<<<<
 *     """a fast 2D array of 8u"""
 * 
 */

struct __pyx_vtabstruct_9FastImage_FastImage8u {
  struct __pyx_vtabstruct_9FastImage_FastImageBase __pyx_base;
  void (*fast_get_absdiff_put)(struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_Size *);
  void (*fast_get_sub_put)(struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_Size *);
  void (*fast_set_val_masked)(struct __pyx_obj_9FastImage_FastImage8u *, Ipp8u, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_Size *);
  void (*fast_get_32f_copy_put)(struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_Size *);
  void (*fast_get_compare_int_put_greater)(struct __pyx_obj_9FastImage_FastImage8u *, int, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_Size *);
};
static struct __pyx_vtabstruct_9FastImage_FastImage8u *__pyx_vtabptr_9FastImage_FastImage8u;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* GetModuleGlobalName.proto */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = PyThreadState_GET();
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* KeywordStringCheck.proto */
static CYTHON_INLINE int __Pyx_CheckKeywordStrings(PyObject *kwdict, const char* function_name, int kw_allowed);

/* ArgTypeTest.proto */
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        Py_SIZE(list) = len+1;
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* append.proto */
static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x);

/* StringJoin.proto */
#if PY_MAJOR_VERSION < 3
#define __Pyx_PyString_Join __Pyx_PyBytes_Join
#define __Pyx_PyBaseString_Join(s, v) (PyUnicode_CheckExact(s) ? PyUnicode_Join(s, v) : __Pyx_PyBytes_Join(s, v))
#else
#define __Pyx_PyString_Join PyUnicode_Join
#define __Pyx_PyBaseString_Join PyUnicode_Join
#endif
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION < 3
    #define __Pyx_PyBytes_Join _PyString_Join
    #else
    #define __Pyx_PyBytes_Join _PyBytes_Join
    #endif
#else
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values);
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyIntBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, int inplace);
#else
#define __Pyx_PyInt_EqObjC(op1, op2, intval, inplace)\
    PyObject_RichCompare(op1, op2, Py_EQ)
    #endif

/* PyFloatBinop.proto */
#if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_EqObjC(PyObject *op1, PyObject *op2, double floatval, int inplace);
#else
#define __Pyx_PyFloat_EqObjC(op1, op2, floatval, inplace)\
    PyObject_RichCompare(op1, op2, Py_EQ)
    #endif

/* None.proto */
static CYTHON_INLINE long __Pyx_div_long(long, long);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunction.proto */
#define __Pyx_CyFunction_USED 1
#include <structmember.h>
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_NewEx(ml, flags, qualname, self, module, globals, code)\
    __Pyx_CyFunction_New(__pyx_CyFunctionType, ml, flags, qualname, self, module, globals, code)
static PyObject *__Pyx_CyFunction_New(PyTypeObject *, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_IppCmpOp(IppCmpOp value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_IppStatus(IppStatus value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_FicStatus(FicStatus value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_intptr_t(intptr_t value);

/* Print.proto */
static int __Pyx_Print(PyObject*, PyObject *, int);
#if CYTHON_COMPILING_IN_PYPY || PY_MAJOR_VERSION >= 3
static PyObject* __pyx_print = 0;
static PyObject* __pyx_print_kwargs = 0;
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Ipp8u(Ipp8u value);

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_char(char value);

/* CIntFromPy.proto */
static CYTHON_INLINE Ipp32s __Pyx_PyInt_As_Ipp32s(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* PrintOne.proto */
static int __Pyx_PrintOne(PyObject* stream, PyObject *o);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static struct __pyx_obj_9FastImage_FastImageBase *__pyx_f_9FastImage_13FastImageBase_c_roi(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self, int __pyx_v_left, int __pyx_v_bottom, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto*/
static void __pyx_f_9FastImage_11FastImage8u_fast_set_val_masked(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, Ipp8u __pyx_v_val, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_mask, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto*/
static void __pyx_f_9FastImage_11FastImage8u_fast_get_32f_copy_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto*/
static void __pyx_f_9FastImage_11FastImage8u_fast_get_absdiff_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto*/
static void __pyx_f_9FastImage_11FastImage8u_fast_get_sub_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto*/
static void __pyx_f_9FastImage_11FastImage8u_fast_get_compare_int_put_greater(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, int __pyx_v_other_int, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_dest, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto*/
static void __pyx_f_9FastImage_12FastImage32f_fast_toself_add_weighted_8u(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size, float __pyx_v_alpha); /* proto*/
static void __pyx_f_9FastImage_12FastImage32f_fast_toself_add_weighted_32f(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size, float __pyx_v_alpha); /* proto*/
static void __pyx_f_9FastImage_12FastImage32f_fast_get_square_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto*/
static void __pyx_f_9FastImage_12FastImage32f_fast_toself_square(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto*/
static void __pyx_f_9FastImage_12FastImage32f_fast_get_subtracted_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto*/
static void __pyx_f_9FastImage_12FastImage32f_fast_get_8u_copy_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto*/
static void __pyx_f_9FastImage_12FastImage32f_fast_toself_multiply(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, float __pyx_v_val, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto*/
static struct __pyx_obj_9FastImage_FastImage32f *__pyx_f_9FastImage_12FastImage32f_create_equal_shape_empty(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto*/

/* Module declarations from 'c_lib' */

/* Module declarations from 'c_python' */

/* Module declarations from 'ipp' */

/* Module declarations from 'fic' */

/* Module declarations from 'FastImage' */
static PyTypeObject *__pyx_ptype_9FastImage_Size = 0;
static PyTypeObject *__pyx_ptype_9FastImage_Point = 0;
static PyTypeObject *__pyx_ptype_9FastImage_FastImageBase = 0;
static PyTypeObject *__pyx_ptype_9FastImage_FastImage32f = 0;
static PyTypeObject *__pyx_ptype_9FastImage_FastImage8u = 0;
static int __pyx_v_9FastImage_FASTIMAGEDEBUG;
static char __pyx_v_9FastImage_CHAR_u;
static char __pyx_v_9FastImage_CHAR_f;
static int __pyx_v_9FastImage_n_hist_levels;
static Ipp32s __pyx_v_9FastImage_hist_levels[0x101];
static void __pyx_f_9FastImage_CHK_HAVEGIL(IppStatus); /*proto*/
static void __pyx_f_9FastImage_CHK_FIC_HAVEGIL(FicStatus); /*proto*/
static void __pyx_f_9FastImage_free_array_struct(void *, void *); /*proto*/
#define __Pyx_MODULE_NAME "FastImage"
int __pyx_module_is_main_FastImage = 0;

/* Implementation of 'FastImage' */
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_TypeError;
static PyObject *__pyx_builtin_ValueError;
static PyObject *__pyx_builtin_map;
static PyObject *__pyx_builtin_NotImplementedError;
static PyObject *__pyx_builtin_MemoryError;
static PyObject *__pyx_builtin_chr;
static const char __pyx_k_i[] = "i";
static const char __pyx_k_x[] = "x";
static const char __pyx_k_y[] = "y";
static const char __pyx_k_8u[] = "8u";
static const char __pyx_k__7[] = " ";
static const char __pyx_k__8[] = "\n";
static const char __pyx_k_op[] = "op";
static const char __pyx_k_u1[] = "|u1";
static const char __pyx_k_32f[] = "32f";
static const char __pyx_k__10[] = "";
static const char __pyx_k__12[] = "-=";
static const char __pyx_k__13[] = "\n\n";
static const char __pyx_k_arr[] = "arr";
static const char __pyx_k_chr[] = "chr";
static const char __pyx_k_doc[] = "__doc__";
static const char __pyx_k_end[] = "end";
static const char __pyx_k_map[] = "map";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_sys[] = "sys";
static const char __pyx_k_val[] = "val";
static const char __pyx_k_attr[] = "attr";
static const char __pyx_k_cmsg[] = "cmsg";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_data[] = "data";
static const char __pyx_k_dest[] = "dest";
static const char __pyx_k_file[] = "file";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_join[] = "join";
static const char __pyx_k_left[] = "left";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mask[] = "mask";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_size[] = "size";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_CmpEq[] = "CmpEq";
static const char __pyx_k_alpha[] = "alpha";
static const char __pyx_k_dtype[] = "dtype";
static const char __pyx_k_empty[] = "empty";
static const char __pyx_k_flush[] = "flush";
static const char __pyx_k_inter[] = "inter";
static const char __pyx_k_numpy[] = "numpy";
static const char __pyx_k_other[] = "other";
static const char __pyx_k_print[] = "print";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_shape[] = "shape";
static const char __pyx_k_width[] = "width";
static const char __pyx_k_write[] = "write";
static const char __pyx_k_append[] = "append";
static const char __pyx_k_bottom[] = "bottom";
static const char __pyx_k_errval[] = "errval";
static const char __pyx_k_height[] = "height";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_module[] = "__module__";
static const char __pyx_k_result[] = "result";
static const char __pyx_k_stderr[] = "stderr";
static const char __pyx_k_CmpLess[] = "CmpLess";
static const char __pyx_k_float32[] = "float32";
static const char __pyx_k_prepare[] = "__prepare__";
static const char __pyx_k_strides[] = "strides";
static const char __pyx_k_typestr[] = "typestr";
static const char __pyx_k_FicError[] = "FicError";
static const char __pyx_k_IppError[] = "IppError";
static const char __pyx_k_arch_str[] = "arch_str";
static const char __pyx_k_qualname[] = "__qualname__";
static const char __pyx_k_result_2[] = "  result";
static const char __pyx_k_CmpLessEq[] = "CmpLessEq";
static const char __pyx_k_FastImage[] = "FastImage";
static const char __pyx_k_TypeError[] = "TypeError";
static const char __pyx_k_error_d_s[] = "error %d: %s";
static const char __pyx_k_metaclass[] = "__metaclass__";
static const char __pyx_k_new_value[] = "new_value";
static const char __pyx_k_ones_like[] = "ones_like";
static const char __pyx_k_set_debug[] = "set_debug";
static const char __pyx_k_traceback[] = "traceback";
static const char __pyx_k_CmpGreater[] = "CmpGreater";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_MemoryError[] = "MemoryError";
static const char __pyx_k_asfastimage[] = "asfastimage";
static const char __pyx_k_get_8u_copy[] = "get_8u_copy";
static const char __pyx_k_print_stack[] = "print_stack";
static const char __pyx_k_result_view[] = "    result.view";
static const char __pyx_k_toself_sqrt[] = "toself_sqrt";
static const char __pyx_k_CmpGreaterEq[] = "CmpGreaterEq";
static const char __pyx_k_array_struct[] = "__array_struct__";
static const char __pyx_k_get_IPP_arch[] = "get_IPP_arch";
static const char __pyx_k_compare_value[] = "compare_value";
static const char __pyx_k_toself_square[] = "toself_square";
static const char __pyx_k_get_IPP_static[] = "get_IPP_static";
static const char __pyx_k_IppError___init[] = "IppError.__init__";
static const char __pyx_k_get_IPP_version[] = "get_IPP_version";
static const char __pyx_k_get_compare_put[] = "get_compare_put";
static const char __pyx_k_freeing_memory_x[] = "freeing memory %#x";
static const char __pyx_k_got_mem_x_step_d[] = "got mem %#x step %d";
static const char __pyx_k_need_file_object[] = "need file object";
static const char __pyx_k_show_mem_and_step[] = "show_mem_and_step";
static const char __pyx_k_FastImage_Size_d_d[] = "FastImage.Size(%d,%d)";
static const char __pyx_k_type_not_supported[] = "type not supported";
static const char __pyx_k_unknown_basetype_r[] = "unknown basetype: %r";
static const char __pyx_k_FastImage_Point_d_d[] = "FastImage.Point(%d,%d)";
static const char __pyx_k_NotImplementedError[] = "NotImplementedError";
static const char __pyx_k_creating_ROI_viewer[] = "creating ROI viewer";
static const char __pyx_k_invalid___array_struct[] = "invalid __array_struct__";
static const char __pyx_k_Error_allocating_memory[] = "Error allocating memory";
static const char __pyx_k_asfastimage_calling___new[] = "asfastimage() calling __new__";
static const char __pyx_k_cannot_handle_such_large_data[] = "cannot handle such large data";
static const char __pyx_k_requesting_memory_size_W_d_H_d[] = "requesting memory (size W:%d H:%d)";
static const char __pyx_k_Users_straw_Documents_src_motmo[] = "/Users/straw/Documents/src/motmot/motmot-base/FastImage/src/FastImage.pyx";
static const char __pyx_k_assigning_ROI_from_self_im_x_to[] = "assigning ROI from self (im: %#x)\n    to (%s: 0x%0x) (L:%d,B:%d)";
static const char __pyx_k_cannot_handle_arrays_with_descr[] = "cannot handle arrays with descr";
static const char __pyx_k_cannot_handle_misaligned_arrays[] = "cannot handle misaligned arrays.";
static const char __pyx_k_cannot_handle_misaligned_or_not[] = "cannot handle misaligned or not writeable arrays.";
static const char __pyx_k_comparison_between_Size_objects[] = "comparison between Size objects only";
static const char __pyx_k_only_2D_arrays_may_be_copied_to[] = "only 2D arrays may be copied to a FastImage";
static const char __pyx_k_only_2D_arrays_may_be_viewed_as[] = "only 2D arrays may be viewed as a FastImage";
static const char __pyx_k_only_2_and_0_5_powers_supported[] = "only 2 and 0.5 powers supported";
static const char __pyx_k_self_step_is_d_width_d_height_d[] = "self.step is %d (width=%d, height=%d)";
static const char __pyx_k_can_only_compare_against_ints_an[] = "can only compare against ints and FastImage8u types";
static const char __pyx_k_cannot_perform_comparison_equali[] = "cannot perform comparison (equality testing only)";
static const char __pyx_k_comparison_between_Point_objects[] = "comparison between Point objects only";
static const char __pyx_k_requesting_32f_memory_size_W_d_H[] = "requesting 32f memory (size W:%d H:%d)";
static const char __pyx_k_typekind_s_itemsize_d_not_suppor[] = "typekind \"%s\", itemsize %d not supported";
static PyObject *__pyx_kp_s_32f;
static PyObject *__pyx_kp_s_8u;
static PyObject *__pyx_n_s_CmpEq;
static PyObject *__pyx_n_s_CmpGreater;
static PyObject *__pyx_n_s_CmpGreaterEq;
static PyObject *__pyx_n_s_CmpLess;
static PyObject *__pyx_n_s_CmpLessEq;
static PyObject *__pyx_kp_s_Error_allocating_memory;
static PyObject *__pyx_n_s_FastImage;
static PyObject *__pyx_kp_s_FastImage_Point_d_d;
static PyObject *__pyx_kp_s_FastImage_Size_d_d;
static PyObject *__pyx_n_s_FicError;
static PyObject *__pyx_n_s_IppError;
static PyObject *__pyx_n_s_IppError___init;
static PyObject *__pyx_n_s_MemoryError;
static PyObject *__pyx_n_s_NotImplementedError;
static PyObject *__pyx_n_s_TypeError;
static PyObject *__pyx_kp_s_Users_straw_Documents_src_motmo;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_kp_s__10;
static PyObject *__pyx_kp_s__12;
static PyObject *__pyx_kp_s__13;
static PyObject *__pyx_kp_s__7;
static PyObject *__pyx_kp_s__8;
static PyObject *__pyx_n_s_alpha;
static PyObject *__pyx_n_s_append;
static PyObject *__pyx_n_s_arch_str;
static PyObject *__pyx_n_s_arr;
static PyObject *__pyx_n_s_array_struct;
static PyObject *__pyx_n_s_asfastimage;
static PyObject *__pyx_kp_s_asfastimage_calling___new;
static PyObject *__pyx_kp_s_assigning_ROI_from_self_im_x_to;
static PyObject *__pyx_n_s_attr;
static PyObject *__pyx_n_s_bottom;
static PyObject *__pyx_kp_s_can_only_compare_against_ints_an;
static PyObject *__pyx_kp_s_cannot_handle_arrays_with_descr;
static PyObject *__pyx_kp_s_cannot_handle_misaligned_arrays;
static PyObject *__pyx_kp_s_cannot_handle_misaligned_or_not;
static PyObject *__pyx_kp_s_cannot_handle_such_large_data;
static PyObject *__pyx_kp_s_cannot_perform_comparison_equali;
static PyObject *__pyx_n_s_chr;
static PyObject *__pyx_n_s_cmsg;
static PyObject *__pyx_n_s_compare_value;
static PyObject *__pyx_kp_s_comparison_between_Point_objects;
static PyObject *__pyx_kp_s_comparison_between_Size_objects;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_kp_s_creating_ROI_viewer;
static PyObject *__pyx_n_s_data;
static PyObject *__pyx_n_s_dest;
static PyObject *__pyx_n_s_doc;
static PyObject *__pyx_n_s_dtype;
static PyObject *__pyx_n_s_empty;
static PyObject *__pyx_n_s_end;
static PyObject *__pyx_kp_s_error_d_s;
static PyObject *__pyx_n_s_errval;
static PyObject *__pyx_n_s_file;
static PyObject *__pyx_n_s_float32;
static PyObject *__pyx_n_s_flush;
static PyObject *__pyx_kp_s_freeing_memory_x;
static PyObject *__pyx_n_s_get_8u_copy;
static PyObject *__pyx_n_s_get_IPP_arch;
static PyObject *__pyx_n_s_get_IPP_static;
static PyObject *__pyx_n_s_get_IPP_version;
static PyObject *__pyx_n_s_get_compare_put;
static PyObject *__pyx_kp_s_got_mem_x_step_d;
static PyObject *__pyx_n_s_height;
static PyObject *__pyx_n_s_i;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_inter;
static PyObject *__pyx_kp_s_invalid___array_struct;
static PyObject *__pyx_n_s_join;
static PyObject *__pyx_n_s_left;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_map;
static PyObject *__pyx_n_s_mask;
static PyObject *__pyx_n_s_metaclass;
static PyObject *__pyx_n_s_module;
static PyObject *__pyx_kp_s_need_file_object;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_new_value;
static PyObject *__pyx_n_s_numpy;
static PyObject *__pyx_n_s_ones_like;
static PyObject *__pyx_kp_s_only_2D_arrays_may_be_copied_to;
static PyObject *__pyx_kp_s_only_2D_arrays_may_be_viewed_as;
static PyObject *__pyx_kp_s_only_2_and_0_5_powers_supported;
static PyObject *__pyx_n_s_op;
static PyObject *__pyx_n_s_other;
static PyObject *__pyx_n_s_prepare;
static PyObject *__pyx_n_s_print;
static PyObject *__pyx_n_s_print_stack;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_qualname;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_kp_s_requesting_32f_memory_size_W_d_H;
static PyObject *__pyx_kp_s_requesting_memory_size_W_d_H_d;
static PyObject *__pyx_n_s_result;
static PyObject *__pyx_kp_s_result_2;
static PyObject *__pyx_kp_s_result_view;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_kp_s_self_step_is_d_width_d_height_d;
static PyObject *__pyx_n_s_set_debug;
static PyObject *__pyx_n_s_shape;
static PyObject *__pyx_n_s_show_mem_and_step;
static PyObject *__pyx_n_s_size;
static PyObject *__pyx_n_s_stderr;
static PyObject *__pyx_n_s_strides;
static PyObject *__pyx_n_s_sys;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_toself_sqrt;
static PyObject *__pyx_n_s_toself_square;
static PyObject *__pyx_n_s_traceback;
static PyObject *__pyx_kp_s_type_not_supported;
static PyObject *__pyx_kp_s_typekind_s_itemsize_d_not_suppor;
static PyObject *__pyx_n_s_typestr;
static PyObject *__pyx_kp_s_u1;
static PyObject *__pyx_kp_s_unknown_basetype_r;
static PyObject *__pyx_n_s_val;
static PyObject *__pyx_n_s_width;
static PyObject *__pyx_n_s_write;
static PyObject *__pyx_n_s_x;
static PyObject *__pyx_n_s_y;
static PyObject *__pyx_pf_9FastImage_set_debug(CYTHON_UNUSED PyObject *__pyx_self, int __pyx_v_val); /* proto */
static PyObject *__pyx_pf_9FastImage_2get_IPP_version(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_9FastImage_4get_IPP_arch(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_9FastImage_6get_IPP_static(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_9FastImage_8IppError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, int __pyx_v_errval); /* proto */
static int __pyx_pf_9FastImage_4Size___init__(struct __pyx_obj_9FastImage_Size *__pyx_v_self, int __pyx_v_width, int __pyx_v_height); /* proto */
static PyObject *__pyx_pf_9FastImage_4Size_2__repr__(struct __pyx_obj_9FastImage_Size *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9FastImage_4Size_4__richcmp__(PyObject *__pyx_v_x, PyObject *__pyx_v_y, int __pyx_v_op); /* proto */
static PyObject *__pyx_pf_9FastImage_4Size_1h___get__(struct __pyx_obj_9FastImage_Size *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9FastImage_4Size_1w___get__(struct __pyx_obj_9FastImage_Size *__pyx_v_self); /* proto */
static int __pyx_pf_9FastImage_5Point___init__(struct __pyx_obj_9FastImage_Point *__pyx_v_self, int __pyx_v_x, int __pyx_v_y); /* proto */
static PyObject *__pyx_pf_9FastImage_5Point_2__repr__(struct __pyx_obj_9FastImage_Point *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9FastImage_5Point_4__richcmp__(PyObject *__pyx_v_a, PyObject *__pyx_v_b, int __pyx_v_op); /* proto */
static PyObject *__pyx_pf_9FastImage_5Point_1x___get__(struct __pyx_obj_9FastImage_Point *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9FastImage_5Point_1y___get__(struct __pyx_obj_9FastImage_Point *__pyx_v_self); /* proto */
static int __pyx_pf_9FastImage_13FastImageBase___cinit__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kw); /* proto */
static int __pyx_pf_9FastImage_13FastImageBase_2__init__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static void __pyx_pf_9FastImage_13FastImageBase_4__dealloc__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9FastImage_13FastImageBase_6show_mem_and_step(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9FastImage_13FastImageBase_8stringview(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9FastImage_13FastImageBase_10dump_to_file(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self, PyObject *__pyx_v_fobj); /* proto */
static PyObject *__pyx_pf_9FastImage_13FastImageBase_5_step___get__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9FastImage_13FastImageBase_16__array_struct_____get__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9FastImage_13FastImageBase_19__array_interface_____get__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9FastImage_13FastImageBase_4size___get__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9FastImage_13FastImageBase_12roi(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self, int __pyx_v_left, int __pyx_v_bottom, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_13FastImageBase_14__getitem__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self, PyObject *__pyx_v_a); /* proto */
static int __pyx_pf_9FastImage_11FastImage8u___cinit__(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kw); /* proto */
static int __pyx_pf_9FastImage_11FastImage8u_2__init__(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_4set_val(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, int __pyx_v_val, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_6set_val_masked(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, int __pyx_v_val, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_mask, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_8get_32f_copy(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_10get_32f_copy_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_12get_8u_copy(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_14get_8u_copy_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_16get_absdiff_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_18get_sub_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_20toself_threshold(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, int __pyx_v_compare_value, int __pyx_v_new_value, struct __pyx_obj_9FastImage_Size *__pyx_v_size, int __pyx_v_op); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_22get_compare(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, PyObject *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size, int __pyx_v_op); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_24get_compare_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, PyObject *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_dest, struct __pyx_obj_9FastImage_Size *__pyx_v_size, int __pyx_v_op); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_26min_index(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_28max_index(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_30to_yuv422(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_32dot(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_34sobel_horiz(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_dest); /* proto */
static PyObject *__pyx_pf_9FastImage_11FastImage8u_36sobel_vert(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_dest); /* proto */
static int __pyx_pf_9FastImage_12FastImage32f___cinit__(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kw); /* proto */
static int __pyx_pf_9FastImage_12FastImage32f_2__init__(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_4set_val(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, float __pyx_v_val, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_6toself_add(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_8toself_add_weighted(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size, float __pyx_v_alpha); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_10toself_add_square(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_12get_square_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_14get_square(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_16toself_square(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_18toself_sqrt(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_20get_subtracted(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_22get_subtracted_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_24toself_subtract(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_26get_8u_copy(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_28get_8u_copy_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_30toself_multiply(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, float __pyx_v_val, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_32get_multiply_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, float __pyx_v_val, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_34pow(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, PyObject *__pyx_v_y); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_36max_index(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_38dot(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_40sobel_horiz(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_dest); /* proto */
static PyObject *__pyx_pf_9FastImage_12FastImage32f_42sobel_vert(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_dest); /* proto */
static PyObject *__pyx_pf_9FastImage_8asfastimage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_arr); /* proto */
static PyObject *__pyx_pf_9FastImage_10copy(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_arr); /* proto */
static PyObject *__pyx_tp_new_9FastImage_Size(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9FastImage_Point(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9FastImage_FastImageBase(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9FastImage_FastImage32f(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_9FastImage_FastImage8u(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_float_0_5;
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_20;
static PyObject *__pyx_int_128;
static PyObject *__pyx_int_257;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_slice__20;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__14;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__16;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__18;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_tuple__21;
static PyObject *__pyx_tuple__22;
static PyObject *__pyx_tuple__23;
static PyObject *__pyx_tuple__24;
static PyObject *__pyx_tuple__25;
static PyObject *__pyx_tuple__26;
static PyObject *__pyx_tuple__27;
static PyObject *__pyx_tuple__28;
static PyObject *__pyx_tuple__29;
static PyObject *__pyx_tuple__30;
static PyObject *__pyx_tuple__31;
static PyObject *__pyx_tuple__32;
static PyObject *__pyx_tuple__33;
static PyObject *__pyx_tuple__34;
static PyObject *__pyx_tuple__36;
static PyObject *__pyx_tuple__38;
static PyObject *__pyx_tuple__41;
static PyObject *__pyx_tuple__43;
static PyObject *__pyx_tuple__45;
static PyObject *__pyx_codeobj__35;
static PyObject *__pyx_codeobj__37;
static PyObject *__pyx_codeobj__39;
static PyObject *__pyx_codeobj__40;
static PyObject *__pyx_codeobj__42;
static PyObject *__pyx_codeobj__44;
static PyObject *__pyx_codeobj__46;

/* "FastImage.pyx":25
 *     hist_levels[i]=i
 * 
 * def set_debug(int val):             # <<<<<<<<<<<<<<
 *     global FASTIMAGEDEBUG
 *     FASTIMAGEDEBUG = val
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_1set_debug(PyObject *__pyx_self, PyObject *__pyx_arg_val); /*proto*/
static PyMethodDef __pyx_mdef_9FastImage_1set_debug = {"set_debug", (PyCFunction)__pyx_pw_9FastImage_1set_debug, METH_O, 0};
static PyObject *__pyx_pw_9FastImage_1set_debug(PyObject *__pyx_self, PyObject *__pyx_arg_val) {
  int __pyx_v_val;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_debug (wrapper)", 0);
  assert(__pyx_arg_val); {
    __pyx_v_val = __Pyx_PyInt_As_int(__pyx_arg_val); if (unlikely((__pyx_v_val == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 25, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.set_debug", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9FastImage_set_debug(__pyx_self, ((int)__pyx_v_val));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_set_debug(CYTHON_UNUSED PyObject *__pyx_self, int __pyx_v_val) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_debug", 0);

  /* "FastImage.pyx":27
 * def set_debug(int val):
 *     global FASTIMAGEDEBUG
 *     FASTIMAGEDEBUG = val             # <<<<<<<<<<<<<<
 * 
 * cdef extern from "unistd.h":
 */
  __pyx_v_9FastImage_FASTIMAGEDEBUG = __pyx_v_val;

  /* "FastImage.pyx":25
 *     hist_levels[i]=i
 * 
 * def set_debug(int val):             # <<<<<<<<<<<<<<
 *     global FASTIMAGEDEBUG
 *     FASTIMAGEDEBUG = val
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":65
 * InitIPP()
 * 
 * def get_IPP_version():             # <<<<<<<<<<<<<<
 *     cdef ipp_version_struct_t val
 *     val = GetIPPVersion()
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_3get_IPP_version(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_9FastImage_3get_IPP_version = {"get_IPP_version", (PyCFunction)__pyx_pw_9FastImage_3get_IPP_version, METH_NOARGS, 0};
static PyObject *__pyx_pw_9FastImage_3get_IPP_version(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_IPP_version (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_2get_IPP_version(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_2get_IPP_version(CYTHON_UNUSED PyObject *__pyx_self) {
  ipp_version_struct_t __pyx_v_val;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("get_IPP_version", 0);

  /* "FastImage.pyx":67
 * def get_IPP_version():
 *     cdef ipp_version_struct_t val
 *     val = GetIPPVersion()             # <<<<<<<<<<<<<<
 *     return val.major, val.minor, val.build
 * 
 */
  __pyx_v_val = GetIPPVersion();

  /* "FastImage.pyx":68
 *     cdef ipp_version_struct_t val
 *     val = GetIPPVersion()
 *     return val.major, val.minor, val.build             # <<<<<<<<<<<<<<
 * 
 * def get_IPP_arch():
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_val.major); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_val.minor); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_val.build); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 68, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":65
 * InitIPP()
 * 
 * def get_IPP_version():             # <<<<<<<<<<<<<<
 *     cdef ipp_version_struct_t val
 *     val = GetIPPVersion()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("FastImage.get_IPP_version", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":70
 *     return val.major, val.minor, val.build
 * 
 * def get_IPP_arch():             # <<<<<<<<<<<<<<
 *     cdef const char* arch_str
 *     arch_str = GetIPPArch()
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_5get_IPP_arch(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_9FastImage_5get_IPP_arch = {"get_IPP_arch", (PyCFunction)__pyx_pw_9FastImage_5get_IPP_arch, METH_NOARGS, 0};
static PyObject *__pyx_pw_9FastImage_5get_IPP_arch(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_IPP_arch (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_4get_IPP_arch(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_4get_IPP_arch(CYTHON_UNUSED PyObject *__pyx_self) {
  char const *__pyx_v_arch_str;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_IPP_arch", 0);

  /* "FastImage.pyx":72
 * def get_IPP_arch():
 *     cdef const char* arch_str
 *     arch_str = GetIPPArch()             # <<<<<<<<<<<<<<
 *     return arch_str
 * 
 */
  __pyx_v_arch_str = GetIPPArch();

  /* "FastImage.pyx":73
 *     cdef const char* arch_str
 *     arch_str = GetIPPArch()
 *     return arch_str             # <<<<<<<<<<<<<<
 * 
 * def get_IPP_static():
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBytes_FromString(__pyx_v_arch_str); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":70
 *     return val.major, val.minor, val.build
 * 
 * def get_IPP_arch():             # <<<<<<<<<<<<<<
 *     cdef const char* arch_str
 *     arch_str = GetIPPArch()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("FastImage.get_IPP_arch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":75
 *     return arch_str
 * 
 * def get_IPP_static():             # <<<<<<<<<<<<<<
 *     return IsIPPStatic()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_7get_IPP_static(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyMethodDef __pyx_mdef_9FastImage_7get_IPP_static = {"get_IPP_static", (PyCFunction)__pyx_pw_9FastImage_7get_IPP_static, METH_NOARGS, 0};
static PyObject *__pyx_pw_9FastImage_7get_IPP_static(PyObject *__pyx_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_IPP_static (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_6get_IPP_static(__pyx_self);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_6get_IPP_static(CYTHON_UNUSED PyObject *__pyx_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("get_IPP_static", 0);

  /* "FastImage.pyx":76
 * 
 * def get_IPP_static():
 *     return IsIPPStatic()             # <<<<<<<<<<<<<<
 * 
 * class IppError(Exception):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(IsIPPStatic()); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":75
 *     return arch_str
 * 
 * def get_IPP_static():             # <<<<<<<<<<<<<<
 *     return IsIPPStatic()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("FastImage.get_IPP_static", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":79
 * 
 * class IppError(Exception):
 *     def __init__(self, int errval):             # <<<<<<<<<<<<<<
 *         cdef const char* cmsg
 *         cmsg = ippGetStatusString(<ipp.IppStatus>errval)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_8IppError_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyMethodDef __pyx_mdef_9FastImage_8IppError_1__init__ = {"__init__", (PyCFunction)__pyx_pw_9FastImage_8IppError_1__init__, METH_VARARGS|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_9FastImage_8IppError_1__init__(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_self = 0;
  int __pyx_v_errval;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_errval,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_errval)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 79, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 79, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_self = values[0];
    __pyx_v_errval = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_errval == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 79, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 79, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.IppError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9FastImage_8IppError___init__(__pyx_self, __pyx_v_self, __pyx_v_errval);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_8IppError___init__(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_self, int __pyx_v_errval) {
  char const *__pyx_v_cmsg;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "FastImage.pyx":81
 *     def __init__(self, int errval):
 *         cdef const char* cmsg
 *         cmsg = ippGetStatusString(<ipp.IppStatus>errval)             # <<<<<<<<<<<<<<
 *         #Exception.__init__(self,"Error %d: %s"%(errval,cmsg))
 *         Exception.__init__(self,cmsg)
 */
  __pyx_v_cmsg = ippGetStatusString(((IppStatus)__pyx_v_errval));

  /* "FastImage.pyx":83
 *         cmsg = ippGetStatusString(<ipp.IppStatus>errval)
 *         #Exception.__init__(self,"Error %d: %s"%(errval,cmsg))
 *         Exception.__init__(self,cmsg)             # <<<<<<<<<<<<<<
 * 
 * class FicError(Exception):
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBytes_FromString(__pyx_v_cmsg); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 83, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_self, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_self, __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_self);
    __Pyx_GIVEREF(__pyx_v_self);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_self);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 83, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "FastImage.pyx":79
 * 
 * class IppError(Exception):
 *     def __init__(self, int errval):             # <<<<<<<<<<<<<<
 *         cdef const char* cmsg
 *         cmsg = ippGetStatusString(<ipp.IppStatus>errval)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("FastImage.IppError.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":88
 *     pass
 * 
 * cdef void CHK_HAVEGIL( ipp.IppStatus errval ) except *:             # <<<<<<<<<<<<<<
 *     if (errval!=0):
 *         raise IppError(errval)
 */

static void __pyx_f_9FastImage_CHK_HAVEGIL(IppStatus __pyx_v_errval) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("CHK_HAVEGIL", 0);

  /* "FastImage.pyx":89
 * 
 * cdef void CHK_HAVEGIL( ipp.IppStatus errval ) except *:
 *     if (errval!=0):             # <<<<<<<<<<<<<<
 *         raise IppError(errval)
 * 
 */
  __pyx_t_1 = ((__pyx_v_errval != 0) != 0);
  if (__pyx_t_1) {

    /* "FastImage.pyx":90
 * cdef void CHK_HAVEGIL( ipp.IppStatus errval ) except *:
 *     if (errval!=0):
 *         raise IppError(errval)             # <<<<<<<<<<<<<<
 * 
 * cdef void CHK_FIC_HAVEGIL( fic.FicStatus errval ) except *:
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_IppError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 90, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_IppStatus(__pyx_v_errval); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 90, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_5) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_5, __pyx_t_4};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else
      #endif
      {
        __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 90, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_5); __pyx_t_5 = NULL;
        __Pyx_GIVEREF(__pyx_t_4);
        PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_4);
        __pyx_t_4 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 90, __pyx_L1_error)

    /* "FastImage.pyx":89
 * 
 * cdef void CHK_HAVEGIL( ipp.IppStatus errval ) except *:
 *     if (errval!=0):             # <<<<<<<<<<<<<<
 *         raise IppError(errval)
 * 
 */
  }

  /* "FastImage.pyx":88
 *     pass
 * 
 * cdef void CHK_HAVEGIL( ipp.IppStatus errval ) except *:             # <<<<<<<<<<<<<<
 *     if (errval!=0):
 *         raise IppError(errval)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("FastImage.CHK_HAVEGIL", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":92
 *         raise IppError(errval)
 * 
 * cdef void CHK_FIC_HAVEGIL( fic.FicStatus errval ) except *:             # <<<<<<<<<<<<<<
 *     if (errval!=0):
 *         errstr = fic.ficGetErrorString(errval)
 */

static void __pyx_f_9FastImage_CHK_FIC_HAVEGIL(FicStatus __pyx_v_errval) {
  char const *__pyx_v_errstr;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("CHK_FIC_HAVEGIL", 0);

  /* "FastImage.pyx":93
 * 
 * cdef void CHK_FIC_HAVEGIL( fic.FicStatus errval ) except *:
 *     if (errval!=0):             # <<<<<<<<<<<<<<
 *         errstr = fic.ficGetErrorString(errval)
 *         raise FicError('error %d: %s'%(errval,errstr))
 */
  __pyx_t_1 = ((__pyx_v_errval != 0) != 0);
  if (__pyx_t_1) {

    /* "FastImage.pyx":94
 * cdef void CHK_FIC_HAVEGIL( fic.FicStatus errval ) except *:
 *     if (errval!=0):
 *         errstr = fic.ficGetErrorString(errval)             # <<<<<<<<<<<<<<
 *         raise FicError('error %d: %s'%(errval,errstr))
 * 
 */
    __pyx_v_errstr = ficGetErrorString(__pyx_v_errval);

    /* "FastImage.pyx":95
 *     if (errval!=0):
 *         errstr = fic.ficGetErrorString(errval)
 *         raise FicError('error %d: %s'%(errval,errstr))             # <<<<<<<<<<<<<<
 * 
 * ctypedef struct PyArrayInterface:
 */
    __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_FicError); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 95, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyInt_From_FicStatus(__pyx_v_errval); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 95, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyBytes_FromString(__pyx_v_errstr); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 95, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = PyTuple_New(2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 95, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_5);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_error_d_s, __pyx_t_6); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 95, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (!__pyx_t_6) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
        PyObject *__pyx_temp[2] = {__pyx_t_6, __pyx_t_5};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      } else
      #endif
      {
        __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 95, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_6); __pyx_t_6 = NULL;
        __Pyx_GIVEREF(__pyx_t_5);
        PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_t_5);
        __pyx_t_5 = 0;
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 95, __pyx_L1_error)

    /* "FastImage.pyx":93
 * 
 * cdef void CHK_FIC_HAVEGIL( fic.FicStatus errval ) except *:
 *     if (errval!=0):             # <<<<<<<<<<<<<<
 *         errstr = fic.ficGetErrorString(errval)
 *         raise FicError('error %d: %s'%(errval,errstr))
 */
  }

  /* "FastImage.pyx":92
 *         raise IppError(errval)
 * 
 * cdef void CHK_FIC_HAVEGIL( fic.FicStatus errval ) except *:             # <<<<<<<<<<<<<<
 *     if (errval!=0):
 *         errstr = fic.ficGetErrorString(errval)
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("FastImage.CHK_FIC_HAVEGIL", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":107
 *     void *data                    # A pointer to the first element of the array
 * 
 * cdef void free_array_struct( void* ptr, void *arr ):             # <<<<<<<<<<<<<<
 *     arrpy = <object>arr
 *     c_python.Py_DECREF( arrpy )
 */

static void __pyx_f_9FastImage_free_array_struct(void *__pyx_v_ptr, void *__pyx_v_arr) {
  PyObject *__pyx_v_arrpy = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("free_array_struct", 0);

  /* "FastImage.pyx":108
 * 
 * cdef void free_array_struct( void* ptr, void *arr ):
 *     arrpy = <object>arr             # <<<<<<<<<<<<<<
 *     c_python.Py_DECREF( arrpy )
 *     c_lib.free(ptr)
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_arr);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_arrpy = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "FastImage.pyx":109
 * cdef void free_array_struct( void* ptr, void *arr ):
 *     arrpy = <object>arr
 *     c_python.Py_DECREF( arrpy )             # <<<<<<<<<<<<<<
 *     c_lib.free(ptr)
 * 
 */
  Py_DECREF(__pyx_v_arrpy);

  /* "FastImage.pyx":110
 *     arrpy = <object>arr
 *     c_python.Py_DECREF( arrpy )
 *     c_lib.free(ptr)             # <<<<<<<<<<<<<<
 * 
 * # comparision operators
 */
  free(__pyx_v_ptr);

  /* "FastImage.pyx":107
 *     void *data                    # A pointer to the first element of the array
 * 
 * cdef void free_array_struct( void* ptr, void *arr ):             # <<<<<<<<<<<<<<
 *     arrpy = <object>arr
 *     c_python.Py_DECREF( arrpy )
 */

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_arrpy);
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":125
 *     """
 *     # keep this read-only to prevent bizarre problems if multiple image share same size
 *     def __init__(self,int width,int height):             # <<<<<<<<<<<<<<
 *         self.sz.width = width
 *         self.sz.height = height
 */

/* Python wrapper */
static int __pyx_pw_9FastImage_4Size_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9FastImage_4Size_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_width;
  int __pyx_v_height;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_width,&__pyx_n_s_height,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_width)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_height)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 125, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 125, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_width = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_width == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 125, __pyx_L3_error)
    __pyx_v_height = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_height == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 125, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 125, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.Size.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9FastImage_4Size___init__(((struct __pyx_obj_9FastImage_Size *)__pyx_v_self), __pyx_v_width, __pyx_v_height);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9FastImage_4Size___init__(struct __pyx_obj_9FastImage_Size *__pyx_v_self, int __pyx_v_width, int __pyx_v_height) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "FastImage.pyx":126
 *     # keep this read-only to prevent bizarre problems if multiple image share same size
 *     def __init__(self,int width,int height):
 *         self.sz.width = width             # <<<<<<<<<<<<<<
 *         self.sz.height = height
 *     def __repr__(self):
 */
  __pyx_v_self->sz.width = __pyx_v_width;

  /* "FastImage.pyx":127
 *     def __init__(self,int width,int height):
 *         self.sz.width = width
 *         self.sz.height = height             # <<<<<<<<<<<<<<
 *     def __repr__(self):
 *         return 'FastImage.Size(%d,%d)'%(self.sz.width,self.sz.height)
 */
  __pyx_v_self->sz.height = __pyx_v_height;

  /* "FastImage.pyx":125
 *     """
 *     # keep this read-only to prevent bizarre problems if multiple image share same size
 *     def __init__(self,int width,int height):             # <<<<<<<<<<<<<<
 *         self.sz.width = width
 *         self.sz.height = height
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":128
 *         self.sz.width = width
 *         self.sz.height = height
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return 'FastImage.Size(%d,%d)'%(self.sz.width,self.sz.height)
 *     def __richcmp__(x, y, int op):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_4Size_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9FastImage_4Size_3__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_4Size_2__repr__(((struct __pyx_obj_9FastImage_Size *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_4Size_2__repr__(struct __pyx_obj_9FastImage_Size *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "FastImage.pyx":129
 *         self.sz.height = height
 *     def __repr__(self):
 *         return 'FastImage.Size(%d,%d)'%(self.sz.width,self.sz.height)             # <<<<<<<<<<<<<<
 *     def __richcmp__(x, y, int op):
 *         cdef Size xs
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->sz.width); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->sz.height); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_FastImage_Size_d_d, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 129, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":128
 *         self.sz.width = width
 *         self.sz.height = height
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return 'FastImage.Size(%d,%d)'%(self.sz.width,self.sz.height)
 *     def __richcmp__(x, y, int op):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("FastImage.Size.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":130
 *     def __repr__(self):
 *         return 'FastImage.Size(%d,%d)'%(self.sz.width,self.sz.height)
 *     def __richcmp__(x, y, int op):             # <<<<<<<<<<<<<<
 *         cdef Size xs
 *         cdef Size ys
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_4Size_5__richcmp__(PyObject *__pyx_v_x, PyObject *__pyx_v_y, int __pyx_v_op); /*proto*/
static PyObject *__pyx_pw_9FastImage_4Size_5__richcmp__(PyObject *__pyx_v_x, PyObject *__pyx_v_y, int __pyx_v_op) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_4Size_4__richcmp__(((PyObject *)__pyx_v_x), ((PyObject *)__pyx_v_y), ((int)__pyx_v_op));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_4Size_4__richcmp__(PyObject *__pyx_v_x, PyObject *__pyx_v_y, int __pyx_v_op) {
  struct __pyx_obj_9FastImage_Size *__pyx_v_xs = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_ys = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__richcmp__", 0);

  /* "FastImage.pyx":133
 *         cdef Size xs
 *         cdef Size ys
 *         if type(x) != Size:             # <<<<<<<<<<<<<<
 *             raise TypeError("comparison between Size objects only")
 *         if type(y) != Size:
 */
  __pyx_t_1 = PyObject_RichCompare(((PyObject *)Py_TYPE(__pyx_v_x)), ((PyObject *)__pyx_ptype_9FastImage_Size), Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 133, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "FastImage.pyx":134
 *         cdef Size ys
 *         if type(x) != Size:
 *             raise TypeError("comparison between Size objects only")             # <<<<<<<<<<<<<<
 *         if type(y) != Size:
 *             raise TypeError("comparison between Size objects only")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple_, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 134, __pyx_L1_error)

    /* "FastImage.pyx":133
 *         cdef Size xs
 *         cdef Size ys
 *         if type(x) != Size:             # <<<<<<<<<<<<<<
 *             raise TypeError("comparison between Size objects only")
 *         if type(y) != Size:
 */
  }

  /* "FastImage.pyx":135
 *         if type(x) != Size:
 *             raise TypeError("comparison between Size objects only")
 *         if type(y) != Size:             # <<<<<<<<<<<<<<
 *             raise TypeError("comparison between Size objects only")
 *         xs = x
 */
  __pyx_t_1 = PyObject_RichCompare(((PyObject *)Py_TYPE(__pyx_v_y)), ((PyObject *)__pyx_ptype_9FastImage_Size), Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 135, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 135, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "FastImage.pyx":136
 *             raise TypeError("comparison between Size objects only")
 *         if type(y) != Size:
 *             raise TypeError("comparison between Size objects only")             # <<<<<<<<<<<<<<
 *         xs = x
 *         ys = y
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 136, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 136, __pyx_L1_error)

    /* "FastImage.pyx":135
 *         if type(x) != Size:
 *             raise TypeError("comparison between Size objects only")
 *         if type(y) != Size:             # <<<<<<<<<<<<<<
 *             raise TypeError("comparison between Size objects only")
 *         xs = x
 */
  }

  /* "FastImage.pyx":137
 *         if type(y) != Size:
 *             raise TypeError("comparison between Size objects only")
 *         xs = x             # <<<<<<<<<<<<<<
 *         ys = y
 *         if op==2:
 */
  if (!(likely(((__pyx_v_x) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_x, __pyx_ptype_9FastImage_Size))))) __PYX_ERR(0, 137, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_x;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_xs = ((struct __pyx_obj_9FastImage_Size *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "FastImage.pyx":138
 *             raise TypeError("comparison between Size objects only")
 *         xs = x
 *         ys = y             # <<<<<<<<<<<<<<
 *         if op==2:
 *             return xs.sz.width == ys.sz.width and xs.sz.height == ys.sz.height
 */
  if (!(likely(((__pyx_v_y) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_y, __pyx_ptype_9FastImage_Size))))) __PYX_ERR(0, 138, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_y;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_ys = ((struct __pyx_obj_9FastImage_Size *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "FastImage.pyx":139
 *         xs = x
 *         ys = y
 *         if op==2:             # <<<<<<<<<<<<<<
 *             return xs.sz.width == ys.sz.width and xs.sz.height == ys.sz.height
 *         elif op==3:
 */
  switch (__pyx_v_op) {
    case 2:

    /* "FastImage.pyx":140
 *         ys = y
 *         if op==2:
 *             return xs.sz.width == ys.sz.width and xs.sz.height == ys.sz.height             # <<<<<<<<<<<<<<
 *         elif op==3:
 *             return not (xs.sz.width == ys.sz.width and xs.sz.height == ys.sz.height)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = (__pyx_v_xs->sz.width == __pyx_v_ys->sz.width);
    if (__pyx_t_2) {
    } else {
      __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 140, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = __pyx_t_3;
      __pyx_t_3 = 0;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = (__pyx_v_xs->sz.height == __pyx_v_ys->sz.height);
    __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
    __pyx_L5_bool_binop_done:;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "FastImage.pyx":139
 *         xs = x
 *         ys = y
 *         if op==2:             # <<<<<<<<<<<<<<
 *             return xs.sz.width == ys.sz.width and xs.sz.height == ys.sz.height
 *         elif op==3:
 */
    break;

    /* "FastImage.pyx":141
 *         if op==2:
 *             return xs.sz.width == ys.sz.width and xs.sz.height == ys.sz.height
 *         elif op==3:             # <<<<<<<<<<<<<<
 *             return not (xs.sz.width == ys.sz.width and xs.sz.height == ys.sz.height)
 *         else:
 */
    case 3:

    /* "FastImage.pyx":142
 *             return xs.sz.width == ys.sz.width and xs.sz.height == ys.sz.height
 *         elif op==3:
 *             return not (xs.sz.width == ys.sz.width and xs.sz.height == ys.sz.height)             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError("cannot perform comparison (equality testing only)")
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = ((__pyx_v_xs->sz.width == __pyx_v_ys->sz.width) != 0);
    if (__pyx_t_4) {
    } else {
      __pyx_t_2 = __pyx_t_4;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_4 = ((__pyx_v_xs->sz.height == __pyx_v_ys->sz.height) != 0);
    __pyx_t_2 = __pyx_t_4;
    __pyx_L7_bool_binop_done:;
    __pyx_t_1 = __Pyx_PyBool_FromLong((!__pyx_t_2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 142, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "FastImage.pyx":141
 *         if op==2:
 *             return xs.sz.width == ys.sz.width and xs.sz.height == ys.sz.height
 *         elif op==3:             # <<<<<<<<<<<<<<
 *             return not (xs.sz.width == ys.sz.width and xs.sz.height == ys.sz.height)
 *         else:
 */
    break;
    default:

    /* "FastImage.pyx":144
 *             return not (xs.sz.width == ys.sz.width and xs.sz.height == ys.sz.height)
 *         else:
 *             raise ValueError("cannot perform comparison (equality testing only)")             # <<<<<<<<<<<<<<
 * 
 *     # leave as short names to make life easy when looking for .sz.width (fast C) vs. .w (slow Python)
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 144, __pyx_L1_error)
    break;
  }

  /* "FastImage.pyx":130
 *     def __repr__(self):
 *         return 'FastImage.Size(%d,%d)'%(self.sz.width,self.sz.height)
 *     def __richcmp__(x, y, int op):             # <<<<<<<<<<<<<<
 *         cdef Size xs
 *         cdef Size ys
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("FastImage.Size.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_xs);
  __Pyx_XDECREF((PyObject *)__pyx_v_ys);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":148
 *     # leave as short names to make life easy when looking for .sz.width (fast C) vs. .w (slow Python)
 *     property h:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.sz.height
 *     property w:
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_4Size_1h_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9FastImage_4Size_1h_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_4Size_1h___get__(((struct __pyx_obj_9FastImage_Size *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_4Size_1h___get__(struct __pyx_obj_9FastImage_Size *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "FastImage.pyx":149
 *     property h:
 *         def __get__(self):
 *             return self.sz.height             # <<<<<<<<<<<<<<
 *     property w:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->sz.height); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":148
 *     # leave as short names to make life easy when looking for .sz.width (fast C) vs. .w (slow Python)
 *     property h:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.sz.height
 *     property w:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("FastImage.Size.h.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":151
 *             return self.sz.height
 *     property w:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.sz.width
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_4Size_1w_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9FastImage_4Size_1w_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_4Size_1w___get__(((struct __pyx_obj_9FastImage_Size *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_4Size_1w___get__(struct __pyx_obj_9FastImage_Size *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "FastImage.pyx":152
 *     property w:
 *         def __get__(self):
 *             return self.sz.width             # <<<<<<<<<<<<<<
 * 
 * cdef class Point:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->sz.width); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 152, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":151
 *             return self.sz.height
 *     property w:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.sz.width
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("FastImage.Size.w.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":156
 * cdef class Point:
 *     # keep this read-only to prevent bizarre problems if multiple image share same origin
 *     def __init__(self,int x,int y):             # <<<<<<<<<<<<<<
 *         self.pt.x = x
 *         self.pt.y = y
 */

/* Python wrapper */
static int __pyx_pw_9FastImage_5Point_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9FastImage_5Point_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_x;
  int __pyx_v_y;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_x,&__pyx_n_s_y,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_x)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_y)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, 1); __PYX_ERR(0, 156, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 156, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_x = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_x == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 156, __pyx_L3_error)
    __pyx_v_y = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_y == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 156, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 156, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.Point.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_9FastImage_5Point___init__(((struct __pyx_obj_9FastImage_Point *)__pyx_v_self), __pyx_v_x, __pyx_v_y);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9FastImage_5Point___init__(struct __pyx_obj_9FastImage_Point *__pyx_v_self, int __pyx_v_x, int __pyx_v_y) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "FastImage.pyx":157
 *     # keep this read-only to prevent bizarre problems if multiple image share same origin
 *     def __init__(self,int x,int y):
 *         self.pt.x = x             # <<<<<<<<<<<<<<
 *         self.pt.y = y
 *     def __repr__(self):
 */
  __pyx_v_self->pt.x = __pyx_v_x;

  /* "FastImage.pyx":158
 *     def __init__(self,int x,int y):
 *         self.pt.x = x
 *         self.pt.y = y             # <<<<<<<<<<<<<<
 *     def __repr__(self):
 *         return 'FastImage.Point(%d,%d)'%(self.pt.x,self.pt.y)
 */
  __pyx_v_self->pt.y = __pyx_v_y;

  /* "FastImage.pyx":156
 * cdef class Point:
 *     # keep this read-only to prevent bizarre problems if multiple image share same origin
 *     def __init__(self,int x,int y):             # <<<<<<<<<<<<<<
 *         self.pt.x = x
 *         self.pt.y = y
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":159
 *         self.pt.x = x
 *         self.pt.y = y
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return 'FastImage.Point(%d,%d)'%(self.pt.x,self.pt.y)
 *     def __richcmp__(a, b, int op):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_5Point_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9FastImage_5Point_3__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_5Point_2__repr__(((struct __pyx_obj_9FastImage_Point *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_5Point_2__repr__(struct __pyx_obj_9FastImage_Point *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "FastImage.pyx":160
 *         self.pt.y = y
 *     def __repr__(self):
 *         return 'FastImage.Point(%d,%d)'%(self.pt.x,self.pt.y)             # <<<<<<<<<<<<<<
 *     def __richcmp__(a, b, int op):
 *         cdef Point ap
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->pt.x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->pt.y); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_FastImage_Point_d_d, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 160, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":159
 *         self.pt.x = x
 *         self.pt.y = y
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return 'FastImage.Point(%d,%d)'%(self.pt.x,self.pt.y)
 *     def __richcmp__(a, b, int op):
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("FastImage.Point.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":161
 *     def __repr__(self):
 *         return 'FastImage.Point(%d,%d)'%(self.pt.x,self.pt.y)
 *     def __richcmp__(a, b, int op):             # <<<<<<<<<<<<<<
 *         cdef Point ap
 *         cdef Point bp
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_5Point_5__richcmp__(PyObject *__pyx_v_a, PyObject *__pyx_v_b, int __pyx_v_op); /*proto*/
static PyObject *__pyx_pw_9FastImage_5Point_5__richcmp__(PyObject *__pyx_v_a, PyObject *__pyx_v_b, int __pyx_v_op) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__richcmp__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_5Point_4__richcmp__(((PyObject *)__pyx_v_a), ((PyObject *)__pyx_v_b), ((int)__pyx_v_op));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_5Point_4__richcmp__(PyObject *__pyx_v_a, PyObject *__pyx_v_b, int __pyx_v_op) {
  struct __pyx_obj_9FastImage_Point *__pyx_v_ap = 0;
  struct __pyx_obj_9FastImage_Point *__pyx_v_bp = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__richcmp__", 0);

  /* "FastImage.pyx":164
 *         cdef Point ap
 *         cdef Point bp
 *         if type(a) != Point:             # <<<<<<<<<<<<<<
 *             raise TypeError("comparison between Point objects only")
 *         if type(b) != Point:
 */
  __pyx_t_1 = PyObject_RichCompare(((PyObject *)Py_TYPE(__pyx_v_a)), ((PyObject *)__pyx_ptype_9FastImage_Point), Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 164, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 164, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "FastImage.pyx":165
 *         cdef Point bp
 *         if type(a) != Point:
 *             raise TypeError("comparison between Point objects only")             # <<<<<<<<<<<<<<
 *         if type(b) != Point:
 *             raise TypeError("comparison between Point objects only")
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 165, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 165, __pyx_L1_error)

    /* "FastImage.pyx":164
 *         cdef Point ap
 *         cdef Point bp
 *         if type(a) != Point:             # <<<<<<<<<<<<<<
 *             raise TypeError("comparison between Point objects only")
 *         if type(b) != Point:
 */
  }

  /* "FastImage.pyx":166
 *         if type(a) != Point:
 *             raise TypeError("comparison between Point objects only")
 *         if type(b) != Point:             # <<<<<<<<<<<<<<
 *             raise TypeError("comparison between Point objects only")
 *         ap = a
 */
  __pyx_t_1 = PyObject_RichCompare(((PyObject *)Py_TYPE(__pyx_v_b)), ((PyObject *)__pyx_ptype_9FastImage_Point), Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 166, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 166, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "FastImage.pyx":167
 *             raise TypeError("comparison between Point objects only")
 *         if type(b) != Point:
 *             raise TypeError("comparison between Point objects only")             # <<<<<<<<<<<<<<
 *         ap = a
 *         bp = b
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 167, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 167, __pyx_L1_error)

    /* "FastImage.pyx":166
 *         if type(a) != Point:
 *             raise TypeError("comparison between Point objects only")
 *         if type(b) != Point:             # <<<<<<<<<<<<<<
 *             raise TypeError("comparison between Point objects only")
 *         ap = a
 */
  }

  /* "FastImage.pyx":168
 *         if type(b) != Point:
 *             raise TypeError("comparison between Point objects only")
 *         ap = a             # <<<<<<<<<<<<<<
 *         bp = b
 *         if op==2:
 */
  if (!(likely(((__pyx_v_a) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_a, __pyx_ptype_9FastImage_Point))))) __PYX_ERR(0, 168, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_a;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_ap = ((struct __pyx_obj_9FastImage_Point *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "FastImage.pyx":169
 *             raise TypeError("comparison between Point objects only")
 *         ap = a
 *         bp = b             # <<<<<<<<<<<<<<
 *         if op==2:
 *             return ap.pt.x == bp.pt.x and ap.pt.y == bp.pt.y
 */
  if (!(likely(((__pyx_v_b) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_b, __pyx_ptype_9FastImage_Point))))) __PYX_ERR(0, 169, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_b;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_bp = ((struct __pyx_obj_9FastImage_Point *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "FastImage.pyx":170
 *         ap = a
 *         bp = b
 *         if op==2:             # <<<<<<<<<<<<<<
 *             return ap.pt.x == bp.pt.x and ap.pt.y == bp.pt.y
 *         elif op==3:
 */
  switch (__pyx_v_op) {
    case 2:

    /* "FastImage.pyx":171
 *         bp = b
 *         if op==2:
 *             return ap.pt.x == bp.pt.x and ap.pt.y == bp.pt.y             # <<<<<<<<<<<<<<
 *         elif op==3:
 *             return not (ap.pt.x == bp.pt.x and ap.pt.y == bp.pt.y)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = (__pyx_v_ap->pt.x == __pyx_v_bp->pt.x);
    if (__pyx_t_2) {
    } else {
      __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 171, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __pyx_t_1 = __pyx_t_3;
      __pyx_t_3 = 0;
      goto __pyx_L5_bool_binop_done;
    }
    __pyx_t_2 = (__pyx_v_ap->pt.y == __pyx_v_bp->pt.y);
    __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 171, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __pyx_t_3 = 0;
    __pyx_L5_bool_binop_done:;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "FastImage.pyx":170
 *         ap = a
 *         bp = b
 *         if op==2:             # <<<<<<<<<<<<<<
 *             return ap.pt.x == bp.pt.x and ap.pt.y == bp.pt.y
 *         elif op==3:
 */
    break;

    /* "FastImage.pyx":172
 *         if op==2:
 *             return ap.pt.x == bp.pt.x and ap.pt.y == bp.pt.y
 *         elif op==3:             # <<<<<<<<<<<<<<
 *             return not (ap.pt.x == bp.pt.x and ap.pt.y == bp.pt.y)
 *         else:
 */
    case 3:

    /* "FastImage.pyx":173
 *             return ap.pt.x == bp.pt.x and ap.pt.y == bp.pt.y
 *         elif op==3:
 *             return not (ap.pt.x == bp.pt.x and ap.pt.y == bp.pt.y)             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError("cannot perform comparison (equality testing only)")
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = ((__pyx_v_ap->pt.x == __pyx_v_bp->pt.x) != 0);
    if (__pyx_t_4) {
    } else {
      __pyx_t_2 = __pyx_t_4;
      goto __pyx_L7_bool_binop_done;
    }
    __pyx_t_4 = ((__pyx_v_ap->pt.y == __pyx_v_bp->pt.y) != 0);
    __pyx_t_2 = __pyx_t_4;
    __pyx_L7_bool_binop_done:;
    __pyx_t_1 = __Pyx_PyBool_FromLong((!__pyx_t_2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 173, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "FastImage.pyx":172
 *         if op==2:
 *             return ap.pt.x == bp.pt.x and ap.pt.y == bp.pt.y
 *         elif op==3:             # <<<<<<<<<<<<<<
 *             return not (ap.pt.x == bp.pt.x and ap.pt.y == bp.pt.y)
 *         else:
 */
    break;
    default:

    /* "FastImage.pyx":175
 *             return not (ap.pt.x == bp.pt.x and ap.pt.y == bp.pt.y)
 *         else:
 *             raise ValueError("cannot perform comparison (equality testing only)")             # <<<<<<<<<<<<<<
 * 
 *     # leave as short names to make life easy when looking for .sz.width (fast C) vs. .w (slow Python)
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 175, __pyx_L1_error)
    break;
  }

  /* "FastImage.pyx":161
 *     def __repr__(self):
 *         return 'FastImage.Point(%d,%d)'%(self.pt.x,self.pt.y)
 *     def __richcmp__(a, b, int op):             # <<<<<<<<<<<<<<
 *         cdef Point ap
 *         cdef Point bp
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("FastImage.Point.__richcmp__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_ap);
  __Pyx_XDECREF((PyObject *)__pyx_v_bp);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":179
 *     # leave as short names to make life easy when looking for .sz.width (fast C) vs. .w (slow Python)
 *     property x:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.pt.x
 *     property y:
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_5Point_1x_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9FastImage_5Point_1x_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_5Point_1x___get__(((struct __pyx_obj_9FastImage_Point *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_5Point_1x___get__(struct __pyx_obj_9FastImage_Point *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "FastImage.pyx":180
 *     property x:
 *         def __get__(self):
 *             return self.pt.x             # <<<<<<<<<<<<<<
 *     property y:
 *         def __get__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->pt.x); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 180, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":179
 *     # leave as short names to make life easy when looking for .sz.width (fast C) vs. .w (slow Python)
 *     property x:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.pt.x
 *     property y:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("FastImage.Point.x.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":182
 *             return self.pt.x
 *     property y:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.pt.y
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_5Point_1y_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9FastImage_5Point_1y_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_5Point_1y___get__(((struct __pyx_obj_9FastImage_Point *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_5Point_1y___get__(struct __pyx_obj_9FastImage_Point *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "FastImage.pyx":183
 *     property y:
 *         def __get__(self):
 *             return self.pt.y             # <<<<<<<<<<<<<<
 * 
 * cdef class FastImageBase:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->pt.y); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 183, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":182
 *             return self.pt.x
 *     property y:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.pt.y
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("FastImage.Point.y.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":186
 * 
 * cdef class FastImageBase:
 *     def __cinit__(self,*args,**kw):             # <<<<<<<<<<<<<<
 *         self.view = 1
 * 
 */

/* Python wrapper */
static int __pyx_pw_9FastImage_13FastImageBase_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9FastImage_13FastImageBase_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_args = 0;
  CYTHON_UNUSED PyObject *__pyx_v_kw = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 1))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_9FastImage_13FastImageBase___cinit__(((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_v_self), __pyx_v_args, __pyx_v_kw);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_kw);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9FastImage_13FastImageBase___cinit__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kw) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "FastImage.pyx":187
 * cdef class FastImageBase:
 *     def __cinit__(self,*args,**kw):
 *         self.view = 1             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, Size size):
 */
  __pyx_v_self->view = 1;

  /* "FastImage.pyx":186
 * 
 * cdef class FastImageBase:
 *     def __cinit__(self,*args,**kw):             # <<<<<<<<<<<<<<
 *         self.view = 1
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":189
 *         self.view = 1
 * 
 *     def __init__(self, Size size):             # <<<<<<<<<<<<<<
 *         self.imsiz = size
 * 
 */

/* Python wrapper */
static int __pyx_pw_9FastImage_13FastImageBase_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9FastImage_13FastImageBase_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_size,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 189, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 189, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImageBase.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 189, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_13FastImageBase_2__init__(((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_v_self), __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9FastImage_13FastImageBase_2__init__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "FastImage.pyx":190
 * 
 *     def __init__(self, Size size):
 *         self.imsiz = size             # <<<<<<<<<<<<<<
 * 
 *         # purely for making __array_struct__ without extra malloc:
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_size));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
  __Pyx_GOTREF(__pyx_v_self->imsiz);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->imsiz));
  __pyx_v_self->imsiz = __pyx_v_size;

  /* "FastImage.pyx":193
 * 
 *         # purely for making __array_struct__ without extra malloc:
 *         self.shape[0] = size.sz.height             # <<<<<<<<<<<<<<
 *         self.shape[1] = size.sz.width
 * 
 */
  __pyx_t_1 = __pyx_v_size->sz.height;
  (__pyx_v_self->shape[0]) = __pyx_t_1;

  /* "FastImage.pyx":194
 *         # purely for making __array_struct__ without extra malloc:
 *         self.shape[0] = size.sz.height
 *         self.shape[1] = size.sz.width             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
 */
  __pyx_t_1 = __pyx_v_size->sz.width;
  (__pyx_v_self->shape[1]) = __pyx_t_1;

  /* "FastImage.pyx":189
 *         self.view = 1
 * 
 *     def __init__(self, Size size):             # <<<<<<<<<<<<<<
 *         self.imsiz = size
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":196
 *         self.shape[1] = size.sz.width
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         global FASTIMAGEDEBUG
 *         if self.view==0:
 */

/* Python wrapper */
static void __pyx_pw_9FastImage_13FastImageBase_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_9FastImage_13FastImageBase_5__dealloc__(PyObject *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_pf_9FastImage_13FastImageBase_4__dealloc__(((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_9FastImage_13FastImageBase_4__dealloc__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "FastImage.pyx":198
 *     def __dealloc__(self):
 *         global FASTIMAGEDEBUG
 *         if self.view==0:             # <<<<<<<<<<<<<<
 *             if FASTIMAGEDEBUG>=1:
 *                 print self,'freeing memory %#x'%<intptr_t>self.im
 */
  __pyx_t_1 = ((__pyx_v_self->view == 0) != 0);
  if (__pyx_t_1) {

    /* "FastImage.pyx":199
 *         global FASTIMAGEDEBUG
 *         if self.view==0:
 *             if FASTIMAGEDEBUG>=1:             # <<<<<<<<<<<<<<
 *                 print self,'freeing memory %#x'%<intptr_t>self.im
 *             ipp.ippiFree(self.im)
 */
    __pyx_t_1 = ((__pyx_v_9FastImage_FASTIMAGEDEBUG >= 1) != 0);
    if (__pyx_t_1) {

      /* "FastImage.pyx":200
 *         if self.view==0:
 *             if FASTIMAGEDEBUG>=1:
 *                 print self,'freeing memory %#x'%<intptr_t>self.im             # <<<<<<<<<<<<<<
 *             ipp.ippiFree(self.im)
 * 
 */
      __pyx_t_2 = __Pyx_PyInt_From_intptr_t(((intptr_t)__pyx_v_self->im)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 200, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyString_Format(__pyx_kp_s_freeing_memory_x, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 200, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 200, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_INCREF(((PyObject *)__pyx_v_self));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
      PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
      __Pyx_GIVEREF(__pyx_t_3);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
      __pyx_t_3 = 0;
      if (__Pyx_Print(0, __pyx_t_2, 1) < 0) __PYX_ERR(0, 200, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "FastImage.pyx":199
 *         global FASTIMAGEDEBUG
 *         if self.view==0:
 *             if FASTIMAGEDEBUG>=1:             # <<<<<<<<<<<<<<
 *                 print self,'freeing memory %#x'%<intptr_t>self.im
 *             ipp.ippiFree(self.im)
 */
    }

    /* "FastImage.pyx":201
 *             if FASTIMAGEDEBUG>=1:
 *                 print self,'freeing memory %#x'%<intptr_t>self.im
 *             ipp.ippiFree(self.im)             # <<<<<<<<<<<<<<
 * 
 *     def show_mem_and_step(self):
 */
    ippiFree(__pyx_v_self->im);

    /* "FastImage.pyx":198
 *     def __dealloc__(self):
 *         global FASTIMAGEDEBUG
 *         if self.view==0:             # <<<<<<<<<<<<<<
 *             if FASTIMAGEDEBUG>=1:
 *                 print self,'freeing memory %#x'%<intptr_t>self.im
 */
  }

  /* "FastImage.pyx":196
 *         self.shape[1] = size.sz.width
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         global FASTIMAGEDEBUG
 *         if self.view==0:
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("FastImage.FastImageBase.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":203
 *             ipp.ippiFree(self.im)
 * 
 *     def show_mem_and_step(self):             # <<<<<<<<<<<<<<
 *         print self,'got mem %#x step %d'%(<intptr_t>self.im,
 *                                           self.step)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_13FastImageBase_7show_mem_and_step(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9FastImage_13FastImageBase_7show_mem_and_step(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("show_mem_and_step (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_13FastImageBase_6show_mem_and_step(((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_13FastImageBase_6show_mem_and_step(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("show_mem_and_step", 0);

  /* "FastImage.pyx":204
 * 
 *     def show_mem_and_step(self):
 *         print self,'got mem %#x step %d'%(<intptr_t>self.im,             # <<<<<<<<<<<<<<
 *                                           self.step)
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_intptr_t(((intptr_t)__pyx_v_self->im)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "FastImage.pyx":205
 *     def show_mem_and_step(self):
 *         print self,'got mem %#x step %d'%(<intptr_t>self.im,
 *                                           self.step)             # <<<<<<<<<<<<<<
 * 
 *     def stringview(self):
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->step); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "FastImage.pyx":204
 * 
 *     def show_mem_and_step(self):
 *         print self,'got mem %#x step %d'%(<intptr_t>self.im,             # <<<<<<<<<<<<<<
 *                                           self.step)
 * 
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_got_mem_x_step_d, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_2 = 0;
  if (__Pyx_Print(0, __pyx_t_3, 1) < 0) __PYX_ERR(0, 204, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "FastImage.pyx":203
 *             ipp.ippiFree(self.im)
 * 
 *     def show_mem_and_step(self):             # <<<<<<<<<<<<<<
 *         print self,'got mem %#x step %d'%(<intptr_t>self.im,
 *                                           self.step)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("FastImage.FastImageBase.show_mem_and_step", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":207
 *                                           self.step)
 * 
 *     def stringview(self):             # <<<<<<<<<<<<<<
 *         cdef int i,j
 *         cdef fiptr rowptr
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_13FastImageBase_9stringview(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9FastImage_13FastImageBase_9stringview(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("stringview (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_13FastImageBase_8stringview(((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_13FastImageBase_8stringview(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self) {
  int __pyx_v_i;
  int __pyx_v_j;
  __pyx_t_9FastImage_fiptr __pyx_v_rowptr;
  __pyx_t_9FastImage_fiptr __pyx_v_valptr;
  PyObject *__pyx_v_lines = NULL;
  PyObject *__pyx_v_line = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("stringview", 0);

  /* "FastImage.pyx":211
 *         cdef fiptr rowptr
 *         cdef fiptr valptr
 *         lines = []             # <<<<<<<<<<<<<<
 *         for i from 0 <= i < self.imsiz.sz.height:
 *             line = []
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_lines = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "FastImage.pyx":212
 *         cdef fiptr valptr
 *         lines = []
 *         for i from 0 <= i < self.imsiz.sz.height:             # <<<<<<<<<<<<<<
 *             line = []
 *             rowptr = self.im+i*self.step
 */
  __pyx_t_2 = __pyx_v_self->imsiz->sz.height;
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_2; __pyx_v_i++) {

    /* "FastImage.pyx":213
 *         lines = []
 *         for i from 0 <= i < self.imsiz.sz.height:
 *             line = []             # <<<<<<<<<<<<<<
 *             rowptr = self.im+i*self.step
 *             for j from 0 <= j < self.imsiz.sz.width:
 */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 213, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_line, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "FastImage.pyx":214
 *         for i from 0 <= i < self.imsiz.sz.height:
 *             line = []
 *             rowptr = self.im+i*self.step             # <<<<<<<<<<<<<<
 *             for j from 0 <= j < self.imsiz.sz.width:
 *                 valptr = rowptr+j*self.strides[1]
 */
    __pyx_v_rowptr = (__pyx_v_self->im + (__pyx_v_i * __pyx_v_self->step));

    /* "FastImage.pyx":215
 *             line = []
 *             rowptr = self.im+i*self.step
 *             for j from 0 <= j < self.imsiz.sz.width:             # <<<<<<<<<<<<<<
 *                 valptr = rowptr+j*self.strides[1]
 *                 if self.basetype == '8u':
 */
    __pyx_t_3 = __pyx_v_self->imsiz->sz.width;
    for (__pyx_v_j = 0; __pyx_v_j < __pyx_t_3; __pyx_v_j++) {

      /* "FastImage.pyx":216
 *             rowptr = self.im+i*self.step
 *             for j from 0 <= j < self.imsiz.sz.width:
 *                 valptr = rowptr+j*self.strides[1]             # <<<<<<<<<<<<<<
 *                 if self.basetype == '8u':
 *                     line.append((<ipp.Ipp8u*>valptr)[0])
 */
      __pyx_v_valptr = (__pyx_v_rowptr + (__pyx_v_j * (__pyx_v_self->strides[1])));

      /* "FastImage.pyx":217
 *             for j from 0 <= j < self.imsiz.sz.width:
 *                 valptr = rowptr+j*self.strides[1]
 *                 if self.basetype == '8u':             # <<<<<<<<<<<<<<
 *                     line.append((<ipp.Ipp8u*>valptr)[0])
 *                 elif self.basetype == '32f':
 */
      __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_self->basetype, __pyx_kp_s_8u, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 217, __pyx_L1_error)
      if (__pyx_t_4) {

        /* "FastImage.pyx":218
 *                 valptr = rowptr+j*self.strides[1]
 *                 if self.basetype == '8u':
 *                     line.append((<ipp.Ipp8u*>valptr)[0])             # <<<<<<<<<<<<<<
 *                 elif self.basetype == '32f':
 *                     line.append((<ipp.Ipp32f*>valptr)[0])
 */
        __pyx_t_1 = __Pyx_PyInt_From_Ipp8u((((Ipp8u *)__pyx_v_valptr)[0])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 218, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_5 = __Pyx_PyObject_Append(__pyx_v_line, __pyx_t_1); if (unlikely(__pyx_t_5 == -1)) __PYX_ERR(0, 218, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "FastImage.pyx":217
 *             for j from 0 <= j < self.imsiz.sz.width:
 *                 valptr = rowptr+j*self.strides[1]
 *                 if self.basetype == '8u':             # <<<<<<<<<<<<<<
 *                     line.append((<ipp.Ipp8u*>valptr)[0])
 *                 elif self.basetype == '32f':
 */
        goto __pyx_L7;
      }

      /* "FastImage.pyx":219
 *                 if self.basetype == '8u':
 *                     line.append((<ipp.Ipp8u*>valptr)[0])
 *                 elif self.basetype == '32f':             # <<<<<<<<<<<<<<
 *                     line.append((<ipp.Ipp32f*>valptr)[0])
 *             line = ' '.join(map(str,line))
 */
      __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_self->basetype, __pyx_kp_s_32f, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 219, __pyx_L1_error)
      if (__pyx_t_4) {

        /* "FastImage.pyx":220
 *                     line.append((<ipp.Ipp8u*>valptr)[0])
 *                 elif self.basetype == '32f':
 *                     line.append((<ipp.Ipp32f*>valptr)[0])             # <<<<<<<<<<<<<<
 *             line = ' '.join(map(str,line))
 *             lines.append(line)
 */
        __pyx_t_1 = PyFloat_FromDouble((((Ipp32f *)__pyx_v_valptr)[0])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 220, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_5 = __Pyx_PyObject_Append(__pyx_v_line, __pyx_t_1); if (unlikely(__pyx_t_5 == -1)) __PYX_ERR(0, 220, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "FastImage.pyx":219
 *                 if self.basetype == '8u':
 *                     line.append((<ipp.Ipp8u*>valptr)[0])
 *                 elif self.basetype == '32f':             # <<<<<<<<<<<<<<
 *                     line.append((<ipp.Ipp32f*>valptr)[0])
 *             line = ' '.join(map(str,line))
 */
      }
      __pyx_L7:;
    }

    /* "FastImage.pyx":221
 *                 elif self.basetype == '32f':
 *                     line.append((<ipp.Ipp32f*>valptr)[0])
 *             line = ' '.join(map(str,line))             # <<<<<<<<<<<<<<
 *             lines.append(line)
 *         lines = '\n'.join(lines)
 */
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)(&PyString_Type)));
    __Pyx_GIVEREF(((PyObject *)(&PyString_Type)));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)(&PyString_Type)));
    __Pyx_INCREF(__pyx_v_line);
    __Pyx_GIVEREF(__pyx_v_line);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_line);
    __pyx_t_6 = __Pyx_PyObject_Call(__pyx_builtin_map, __pyx_t_1, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyString_Join(__pyx_kp_s__7, __pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 221, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF_SET(__pyx_v_line, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "FastImage.pyx":222
 *                     line.append((<ipp.Ipp32f*>valptr)[0])
 *             line = ' '.join(map(str,line))
 *             lines.append(line)             # <<<<<<<<<<<<<<
 *         lines = '\n'.join(lines)
 *         return lines
 */
    __pyx_t_5 = __Pyx_PyObject_Append(__pyx_v_lines, __pyx_v_line); if (unlikely(__pyx_t_5 == -1)) __PYX_ERR(0, 222, __pyx_L1_error)
  }

  /* "FastImage.pyx":223
 *             line = ' '.join(map(str,line))
 *             lines.append(line)
 *         lines = '\n'.join(lines)             # <<<<<<<<<<<<<<
 *         return lines
 * 
 */
  __pyx_t_1 = __Pyx_PyString_Join(__pyx_kp_s__8, __pyx_v_lines); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 223, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF_SET(__pyx_v_lines, __pyx_t_1);
  __pyx_t_1 = 0;

  /* "FastImage.pyx":224
 *             lines.append(line)
 *         lines = '\n'.join(lines)
 *         return lines             # <<<<<<<<<<<<<<
 * 
 *     def dump_to_file(self,fobj):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_lines);
  __pyx_r = __pyx_v_lines;
  goto __pyx_L0;

  /* "FastImage.pyx":207
 *                                           self.step)
 * 
 *     def stringview(self):             # <<<<<<<<<<<<<<
 *         cdef int i,j
 *         cdef fiptr rowptr
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("FastImage.FastImageBase.stringview", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_lines);
  __Pyx_XDECREF(__pyx_v_line);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":226
 *         return lines
 * 
 *     def dump_to_file(self,fobj):             # <<<<<<<<<<<<<<
 *         cdef int i
 *         cdef int nbytes
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_13FastImageBase_11dump_to_file(PyObject *__pyx_v_self, PyObject *__pyx_v_fobj); /*proto*/
static PyObject *__pyx_pw_9FastImage_13FastImageBase_11dump_to_file(PyObject *__pyx_v_self, PyObject *__pyx_v_fobj) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("dump_to_file (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_13FastImageBase_10dump_to_file(((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_v_self), ((PyObject *)__pyx_v_fobj));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_13FastImageBase_10dump_to_file(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self, PyObject *__pyx_v_fobj) {
  int __pyx_v_i;
  int __pyx_v_nbytes;
  int __pyx_v_nbytes_tot;
  int __pyx_v_bytes_per_pixel;
  __pyx_t_9FastImage_fiptr __pyx_v_rowptr;
  FILE *__pyx_v_fd;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("dump_to_file", 0);

  /* "FastImage.pyx":234
 *         cdef fiptr valptr
 *         cdef c_lib.FILE* fd
 *         nbytes_tot = 0             # <<<<<<<<<<<<<<
 *         bytes_per_pixel = self.strides[1]
 *         if not c_python.PyFile_Check(fobj):
 */
  __pyx_v_nbytes_tot = 0;

  /* "FastImage.pyx":235
 *         cdef c_lib.FILE* fd
 *         nbytes_tot = 0
 *         bytes_per_pixel = self.strides[1]             # <<<<<<<<<<<<<<
 *         if not c_python.PyFile_Check(fobj):
 *             raise ValueError('need file object')
 */
  __pyx_v_bytes_per_pixel = (__pyx_v_self->strides[1]);

  /* "FastImage.pyx":236
 *         nbytes_tot = 0
 *         bytes_per_pixel = self.strides[1]
 *         if not c_python.PyFile_Check(fobj):             # <<<<<<<<<<<<<<
 *             raise ValueError('need file object')
 *         fd = c_python.PyFile_AsFile(fobj)
 */
  __pyx_t_1 = ((!(PyFile_Check(__pyx_v_fobj) != 0)) != 0);
  if (__pyx_t_1) {

    /* "FastImage.pyx":237
 *         bytes_per_pixel = self.strides[1]
 *         if not c_python.PyFile_Check(fobj):
 *             raise ValueError('need file object')             # <<<<<<<<<<<<<<
 *         fd = c_python.PyFile_AsFile(fobj)
 *         with nogil:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 237, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 237, __pyx_L1_error)

    /* "FastImage.pyx":236
 *         nbytes_tot = 0
 *         bytes_per_pixel = self.strides[1]
 *         if not c_python.PyFile_Check(fobj):             # <<<<<<<<<<<<<<
 *             raise ValueError('need file object')
 *         fd = c_python.PyFile_AsFile(fobj)
 */
  }

  /* "FastImage.pyx":238
 *         if not c_python.PyFile_Check(fobj):
 *             raise ValueError('need file object')
 *         fd = c_python.PyFile_AsFile(fobj)             # <<<<<<<<<<<<<<
 *         with nogil:
 *             for i from 0 <= i < self.imsiz.sz.height:
 */
  __pyx_v_fd = PyFile_AsFile(__pyx_v_fobj);

  /* "FastImage.pyx":239
 *             raise ValueError('need file object')
 *         fd = c_python.PyFile_AsFile(fobj)
 *         with nogil:             # <<<<<<<<<<<<<<
 *             for i from 0 <= i < self.imsiz.sz.height:
 *                 rowptr = self.im+i*self.step
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "FastImage.pyx":240
 *         fd = c_python.PyFile_AsFile(fobj)
 *         with nogil:
 *             for i from 0 <= i < self.imsiz.sz.height:             # <<<<<<<<<<<<<<
 *                 rowptr = self.im+i*self.step
 *                 nbytes = c_lib.fwrite( rowptr, 1, self.imsiz.sz.width*bytes_per_pixel, fd )
 */
        __pyx_t_3 = __pyx_v_self->imsiz->sz.height;
        for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_3; __pyx_v_i++) {

          /* "FastImage.pyx":241
 *         with nogil:
 *             for i from 0 <= i < self.imsiz.sz.height:
 *                 rowptr = self.im+i*self.step             # <<<<<<<<<<<<<<
 *                 nbytes = c_lib.fwrite( rowptr, 1, self.imsiz.sz.width*bytes_per_pixel, fd )
 *                 nbytes_tot = nbytes_tot+nbytes
 */
          __pyx_v_rowptr = (__pyx_v_self->im + (__pyx_v_i * __pyx_v_self->step));

          /* "FastImage.pyx":242
 *             for i from 0 <= i < self.imsiz.sz.height:
 *                 rowptr = self.im+i*self.step
 *                 nbytes = c_lib.fwrite( rowptr, 1, self.imsiz.sz.width*bytes_per_pixel, fd )             # <<<<<<<<<<<<<<
 *                 nbytes_tot = nbytes_tot+nbytes
 *         return nbytes_tot
 */
          __pyx_v_nbytes = fwrite(__pyx_v_rowptr, 1, (__pyx_v_self->imsiz->sz.width * __pyx_v_bytes_per_pixel), __pyx_v_fd);

          /* "FastImage.pyx":243
 *                 rowptr = self.im+i*self.step
 *                 nbytes = c_lib.fwrite( rowptr, 1, self.imsiz.sz.width*bytes_per_pixel, fd )
 *                 nbytes_tot = nbytes_tot+nbytes             # <<<<<<<<<<<<<<
 *         return nbytes_tot
 * 
 */
          __pyx_v_nbytes_tot = (__pyx_v_nbytes_tot + __pyx_v_nbytes);
        }
      }

      /* "FastImage.pyx":239
 *             raise ValueError('need file object')
 *         fd = c_python.PyFile_AsFile(fobj)
 *         with nogil:             # <<<<<<<<<<<<<<
 *             for i from 0 <= i < self.imsiz.sz.height:
 *                 rowptr = self.im+i*self.step
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "FastImage.pyx":244
 *                 nbytes = c_lib.fwrite( rowptr, 1, self.imsiz.sz.width*bytes_per_pixel, fd )
 *                 nbytes_tot = nbytes_tot+nbytes
 *         return nbytes_tot             # <<<<<<<<<<<<<<
 * 
 *     property _step:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_nbytes_tot); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":226
 *         return lines
 * 
 *     def dump_to_file(self,fobj):             # <<<<<<<<<<<<<<
 *         cdef int i
 *         cdef int nbytes
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("FastImage.FastImageBase.dump_to_file", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":247
 * 
 *     property _step:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.step
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_13FastImageBase_5_step_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9FastImage_13FastImageBase_5_step_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_13FastImageBase_5_step___get__(((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_13FastImageBase_5_step___get__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "FastImage.pyx":248
 *     property _step:
 *         def __get__(self):
 *             return self.step             # <<<<<<<<<<<<<<
 * 
 *     property __array_struct__:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->step); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 248, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":247
 * 
 *     property _step:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.step
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("FastImage.FastImageBase._step.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":251
 * 
 *     property __array_struct__:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef PyArrayInterface* inter
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_13FastImageBase_16__array_struct___1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9FastImage_13FastImageBase_16__array_struct___1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_13FastImageBase_16__array_struct_____get__(((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_13FastImageBase_16__array_struct_____get__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self) {
  __pyx_t_9FastImage_PyArrayInterface *__pyx_v_inter;
  PyObject *__pyx_v_obj = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  Py_intptr_t *__pyx_t_4;
  __pyx_t_9FastImage_fiptr __pyx_t_5;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "FastImage.pyx":254
 *             cdef PyArrayInterface* inter
 * 
 *             inter = <PyArrayInterface*>c_lib.malloc( sizeof( PyArrayInterface ) )             # <<<<<<<<<<<<<<
 *             inter.two = 2
 *             inter.nd = 2
 */
  __pyx_v_inter = ((__pyx_t_9FastImage_PyArrayInterface *)malloc((sizeof(__pyx_t_9FastImage_PyArrayInterface))));

  /* "FastImage.pyx":255
 * 
 *             inter = <PyArrayInterface*>c_lib.malloc( sizeof( PyArrayInterface ) )
 *             inter.two = 2             # <<<<<<<<<<<<<<
 *             inter.nd = 2
 *             if self.basetype == '8u':
 */
  __pyx_v_inter->two = 2;

  /* "FastImage.pyx":256
 *             inter = <PyArrayInterface*>c_lib.malloc( sizeof( PyArrayInterface ) )
 *             inter.two = 2
 *             inter.nd = 2             # <<<<<<<<<<<<<<
 *             if self.basetype == '8u':
 *                 inter.typekind = CHAR_u
 */
  __pyx_v_inter->nd = 2;

  /* "FastImage.pyx":257
 *             inter.two = 2
 *             inter.nd = 2
 *             if self.basetype == '8u':             # <<<<<<<<<<<<<<
 *                 inter.typekind = CHAR_u
 *             elif self.basetype == '32f':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->basetype, __pyx_kp_s_8u, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 257, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "FastImage.pyx":258
 *             inter.nd = 2
 *             if self.basetype == '8u':
 *                 inter.typekind = CHAR_u             # <<<<<<<<<<<<<<
 *             elif self.basetype == '32f':
 *                 inter.typekind = CHAR_f
 */
    __pyx_v_inter->typekind = __pyx_v_9FastImage_CHAR_u;

    /* "FastImage.pyx":257
 *             inter.two = 2
 *             inter.nd = 2
 *             if self.basetype == '8u':             # <<<<<<<<<<<<<<
 *                 inter.typekind = CHAR_u
 *             elif self.basetype == '32f':
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":259
 *             if self.basetype == '8u':
 *                 inter.typekind = CHAR_u
 *             elif self.basetype == '32f':             # <<<<<<<<<<<<<<
 *                 inter.typekind = CHAR_f
 *             else:
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->basetype, __pyx_kp_s_32f, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 259, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "FastImage.pyx":260
 *                 inter.typekind = CHAR_u
 *             elif self.basetype == '32f':
 *                 inter.typekind = CHAR_f             # <<<<<<<<<<<<<<
 *             else:
 *                 raise ValueError('unknown basetype: %r'%self.basetype)
 */
    __pyx_v_inter->typekind = __pyx_v_9FastImage_CHAR_f;

    /* "FastImage.pyx":259
 *             if self.basetype == '8u':
 *                 inter.typekind = CHAR_u
 *             elif self.basetype == '32f':             # <<<<<<<<<<<<<<
 *                 inter.typekind = CHAR_f
 *             else:
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":262
 *                 inter.typekind = CHAR_f
 *             else:
 *                 raise ValueError('unknown basetype: %r'%self.basetype)             # <<<<<<<<<<<<<<
 *             inter.itemsize = self.strides[1]
 *             inter.flags = NOTSWAPPED | ALIGNED | WRITEABLE
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_unknown_basetype_r, __pyx_v_self->basetype); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 262, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "FastImage.pyx":263
 *             else:
 *                 raise ValueError('unknown basetype: %r'%self.basetype)
 *             inter.itemsize = self.strides[1]             # <<<<<<<<<<<<<<
 *             inter.flags = NOTSWAPPED | ALIGNED | WRITEABLE
 *             inter.strides = self.strides
 */
  __pyx_v_inter->itemsize = (__pyx_v_self->strides[1]);

  /* "FastImage.pyx":264
 *                 raise ValueError('unknown basetype: %r'%self.basetype)
 *             inter.itemsize = self.strides[1]
 *             inter.flags = NOTSWAPPED | ALIGNED | WRITEABLE             # <<<<<<<<<<<<<<
 *             inter.strides = self.strides
 *             inter.shape = self.shape
 */
  __pyx_v_inter->flags = ((NOTSWAPPED | ALIGNED) | WRITEABLE);

  /* "FastImage.pyx":265
 *             inter.itemsize = self.strides[1]
 *             inter.flags = NOTSWAPPED | ALIGNED | WRITEABLE
 *             inter.strides = self.strides             # <<<<<<<<<<<<<<
 *             inter.shape = self.shape
 *             inter.data = self.im
 */
  __pyx_t_4 = __pyx_v_self->strides;
  __pyx_v_inter->strides = __pyx_t_4;

  /* "FastImage.pyx":266
 *             inter.flags = NOTSWAPPED | ALIGNED | WRITEABLE
 *             inter.strides = self.strides
 *             inter.shape = self.shape             # <<<<<<<<<<<<<<
 *             inter.data = self.im
 *             c_python.Py_INCREF(self)
 */
  __pyx_t_4 = __pyx_v_self->shape;
  __pyx_v_inter->shape = __pyx_t_4;

  /* "FastImage.pyx":267
 *             inter.strides = self.strides
 *             inter.shape = self.shape
 *             inter.data = self.im             # <<<<<<<<<<<<<<
 *             c_python.Py_INCREF(self)
 *             obj = c_python.PyCObject_FromVoidPtrAndDesc( <void*>inter, <void*>self, free_array_struct)
 */
  __pyx_t_5 = __pyx_v_self->im;
  __pyx_v_inter->data = __pyx_t_5;

  /* "FastImage.pyx":268
 *             inter.shape = self.shape
 *             inter.data = self.im
 *             c_python.Py_INCREF(self)             # <<<<<<<<<<<<<<
 *             obj = c_python.PyCObject_FromVoidPtrAndDesc( <void*>inter, <void*>self, free_array_struct)
 *             return obj
 */
  Py_INCREF(((PyObject *)__pyx_v_self));

  /* "FastImage.pyx":269
 *             inter.data = self.im
 *             c_python.Py_INCREF(self)
 *             obj = c_python.PyCObject_FromVoidPtrAndDesc( <void*>inter, <void*>self, free_array_struct)             # <<<<<<<<<<<<<<
 *             return obj
 * 
 */
  __pyx_t_2 = PyCObject_FromVoidPtrAndDesc(((void *)__pyx_v_inter), ((void *)__pyx_v_self), __pyx_f_9FastImage_free_array_struct); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 269, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_obj = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "FastImage.pyx":270
 *             c_python.Py_INCREF(self)
 *             obj = c_python.PyCObject_FromVoidPtrAndDesc( <void*>inter, <void*>self, free_array_struct)
 *             return obj             # <<<<<<<<<<<<<<
 * 
 *     property __array_interface__:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_obj);
  __pyx_r = __pyx_v_obj;
  goto __pyx_L0;

  /* "FastImage.pyx":251
 * 
 *     property __array_struct__:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             cdef PyArrayInterface* inter
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("FastImage.FastImageBase.__array_struct__.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_obj);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":273
 * 
 *     property __array_interface__:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             inter = {}
 *             inter['shape'] = self.shape[0], self.shape[1]
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_13FastImageBase_19__array_interface___1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9FastImage_13FastImageBase_19__array_interface___1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_13FastImageBase_19__array_interface_____get__(((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_13FastImageBase_19__array_interface_____get__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self) {
  PyObject *__pyx_v_inter = NULL;
  int __pyx_v_readonly;
  intptr_t __pyx_v_data_ptr_int;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "FastImage.pyx":274
 *     property __array_interface__:
 *         def __get__(self):
 *             inter = {}             # <<<<<<<<<<<<<<
 *             inter['shape'] = self.shape[0], self.shape[1]
 *             inter['strides'] = self.strides[0], self.strides[1]
 */
  __pyx_t_1 = PyDict_New(); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_inter = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "FastImage.pyx":275
 *         def __get__(self):
 *             inter = {}
 *             inter['shape'] = self.shape[0], self.shape[1]             # <<<<<<<<<<<<<<
 *             inter['strides'] = self.strides[0], self.strides[1]
 *             if self.basetype == '8u':
 */
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_self->shape[0])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_self->shape[1])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  if (unlikely(PyDict_SetItem(__pyx_v_inter, __pyx_n_s_shape, __pyx_t_3) < 0)) __PYX_ERR(0, 275, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "FastImage.pyx":276
 *             inter = {}
 *             inter['shape'] = self.shape[0], self.shape[1]
 *             inter['strides'] = self.strides[0], self.strides[1]             # <<<<<<<<<<<<<<
 *             if self.basetype == '8u':
 *                 inter['typestr'] = '|u1'
 */
  __pyx_t_3 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_self->strides[0])); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_self->strides[1])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
  __pyx_t_3 = 0;
  __pyx_t_2 = 0;
  if (unlikely(PyDict_SetItem(__pyx_v_inter, __pyx_n_s_strides, __pyx_t_1) < 0)) __PYX_ERR(0, 276, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "FastImage.pyx":277
 *             inter['shape'] = self.shape[0], self.shape[1]
 *             inter['strides'] = self.strides[0], self.strides[1]
 *             if self.basetype == '8u':             # <<<<<<<<<<<<<<
 *                 inter['typestr'] = '|u1'
 *             else:
 */
  __pyx_t_4 = (__Pyx_PyString_Equals(__pyx_v_self->basetype, __pyx_kp_s_8u, Py_EQ)); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 277, __pyx_L1_error)
  if (__pyx_t_4) {

    /* "FastImage.pyx":278
 *             inter['strides'] = self.strides[0], self.strides[1]
 *             if self.basetype == '8u':
 *                 inter['typestr'] = '|u1'             # <<<<<<<<<<<<<<
 *             else:
 *                 raise NotImplementedError('')
 */
    if (unlikely(PyDict_SetItem(__pyx_v_inter, __pyx_n_s_typestr, __pyx_kp_s_u1) < 0)) __PYX_ERR(0, 278, __pyx_L1_error)

    /* "FastImage.pyx":277
 *             inter['shape'] = self.shape[0], self.shape[1]
 *             inter['strides'] = self.strides[0], self.strides[1]
 *             if self.basetype == '8u':             # <<<<<<<<<<<<<<
 *                 inter['typestr'] = '|u1'
 *             else:
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":280
 *                 inter['typestr'] = '|u1'
 *             else:
 *                 raise NotImplementedError('')             # <<<<<<<<<<<<<<
 *             readonly = True
 *             data_ptr_int = <intptr_t>self.im
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_NotImplementedError, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 280, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 280, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "FastImage.pyx":281
 *             else:
 *                 raise NotImplementedError('')
 *             readonly = True             # <<<<<<<<<<<<<<
 *             data_ptr_int = <intptr_t>self.im
 *             inter['data'] = (data_ptr_int,readonly)
 */
  __pyx_v_readonly = 1;

  /* "FastImage.pyx":282
 *                 raise NotImplementedError('')
 *             readonly = True
 *             data_ptr_int = <intptr_t>self.im             # <<<<<<<<<<<<<<
 *             inter['data'] = (data_ptr_int,readonly)
 *             return inter
 */
  __pyx_v_data_ptr_int = ((intptr_t)__pyx_v_self->im);

  /* "FastImage.pyx":283
 *             readonly = True
 *             data_ptr_int = <intptr_t>self.im
 *             inter['data'] = (data_ptr_int,readonly)             # <<<<<<<<<<<<<<
 *             return inter
 * 
 */
  __pyx_t_1 = __Pyx_PyInt_From_intptr_t(__pyx_v_data_ptr_int); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_readonly); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  if (unlikely(PyDict_SetItem(__pyx_v_inter, __pyx_n_s_data, __pyx_t_3) < 0)) __PYX_ERR(0, 283, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "FastImage.pyx":284
 *             data_ptr_int = <intptr_t>self.im
 *             inter['data'] = (data_ptr_int,readonly)
 *             return inter             # <<<<<<<<<<<<<<
 * 
 *     property size:
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_inter);
  __pyx_r = __pyx_v_inter;
  goto __pyx_L0;

  /* "FastImage.pyx":273
 * 
 *     property __array_interface__:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             inter = {}
 *             inter['shape'] = self.shape[0], self.shape[1]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("FastImage.FastImageBase.__array_interface__.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_inter);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":287
 * 
 *     property size:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.imsiz
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_13FastImageBase_4size_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_9FastImage_13FastImageBase_4size_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_13FastImageBase_4size___get__(((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_13FastImageBase_4size___get__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "FastImage.pyx":288
 *     property size:
 *         def __get__(self):
 *             return self.imsiz             # <<<<<<<<<<<<<<
 * 
 *     cdef FastImageBase c_roi( self, int left, int bottom, Size size):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->imsiz));
  __pyx_r = ((PyObject *)__pyx_v_self->imsiz);
  goto __pyx_L0;

  /* "FastImage.pyx":287
 * 
 *     property size:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self.imsiz
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":290
 *             return self.imsiz
 * 
 *     cdef FastImageBase c_roi( self, int left, int bottom, Size size):             # <<<<<<<<<<<<<<
 *         cdef FastImageBase result
 *         global FASTIMAGEDEBUG
 */

static struct __pyx_obj_9FastImage_FastImageBase *__pyx_f_9FastImage_13FastImageBase_c_roi(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self, int __pyx_v_left, int __pyx_v_bottom, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_result = 0;
  struct __pyx_obj_9FastImage_FastImageBase *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannySetupContext("c_roi", 0);

  /* "FastImage.pyx":294
 *         global FASTIMAGEDEBUG
 * 
 *         if FASTIMAGEDEBUG>=2:             # <<<<<<<<<<<<<<
 *             print self,'creating ROI viewer'
 *         result = self.__new__(type(self)) # new object of my same type
 */
  __pyx_t_1 = ((__pyx_v_9FastImage_FASTIMAGEDEBUG >= 2) != 0);
  if (__pyx_t_1) {

    /* "FastImage.pyx":295
 * 
 *         if FASTIMAGEDEBUG>=2:
 *             print self,'creating ROI viewer'             # <<<<<<<<<<<<<<
 *         result = self.__new__(type(self)) # new object of my same type
 *         FastImageBase.__init__(result, size) # run common __init__
 */
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self));
    __Pyx_INCREF(__pyx_kp_s_creating_ROI_viewer);
    __Pyx_GIVEREF(__pyx_kp_s_creating_ROI_viewer);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_kp_s_creating_ROI_viewer);
    if (__Pyx_Print(0, __pyx_t_2, 1) < 0) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "FastImage.pyx":294
 *         global FASTIMAGEDEBUG
 * 
 *         if FASTIMAGEDEBUG>=2:             # <<<<<<<<<<<<<<
 *             print self,'creating ROI viewer'
 *         result = self.__new__(type(self)) # new object of my same type
 */
  }

  /* "FastImage.pyx":296
 *         if FASTIMAGEDEBUG>=2:
 *             print self,'creating ROI viewer'
 *         result = self.__new__(type(self)) # new object of my same type             # <<<<<<<<<<<<<<
 *         FastImageBase.__init__(result, size) # run common __init__
 *         result.strides[0] = self.strides[0]
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_new); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 296, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))};
      __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 296, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))};
      __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 296, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_2);
    } else
    #endif
    {
      __pyx_t_5 = PyTuple_New(1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 296, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
      __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
      PyTuple_SET_ITEM(__pyx_t_5, 0+1, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_5, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 296, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9FastImage_FastImageBase))))) __PYX_ERR(0, 296, __pyx_L1_error)
  __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "FastImage.pyx":297
 *             print self,'creating ROI viewer'
 *         result = self.__new__(type(self)) # new object of my same type
 *         FastImageBase.__init__(result, size) # run common __init__             # <<<<<<<<<<<<<<
 *         result.strides[0] = self.strides[0]
 *         result.step = self.step
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9FastImage_FastImageBase), __pyx_n_s_init); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = NULL;
  __pyx_t_6 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_6 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, ((PyObject *)__pyx_v_result), ((PyObject *)__pyx_v_size)};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_5, ((PyObject *)__pyx_v_result), ((PyObject *)__pyx_v_size)};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_5) {
      __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5); __pyx_t_5 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_result));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_result));
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_6, ((PyObject *)__pyx_v_result));
    __Pyx_INCREF(((PyObject *)__pyx_v_size));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_6, ((PyObject *)__pyx_v_size));
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 297, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":298
 *         result = self.__new__(type(self)) # new object of my same type
 *         FastImageBase.__init__(result, size) # run common __init__
 *         result.strides[0] = self.strides[0]             # <<<<<<<<<<<<<<
 *         result.step = self.step
 *         if self.basetype == '8u':
 */
  (__pyx_v_result->strides[0]) = (__pyx_v_self->strides[0]);

  /* "FastImage.pyx":299
 *         FastImageBase.__init__(result, size) # run common __init__
 *         result.strides[0] = self.strides[0]
 *         result.step = self.step             # <<<<<<<<<<<<<<
 *         if self.basetype == '8u':
 *             result.im = IMPOS8u(<ipp.Ipp8u*>self.im,self.step,bottom,left)
 */
  __pyx_t_6 = __pyx_v_self->step;
  __pyx_v_result->step = __pyx_t_6;

  /* "FastImage.pyx":300
 *         result.strides[0] = self.strides[0]
 *         result.step = self.step
 *         if self.basetype == '8u':             # <<<<<<<<<<<<<<
 *             result.im = IMPOS8u(<ipp.Ipp8u*>self.im,self.step,bottom,left)
 *         elif self.basetype == '32f':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->basetype, __pyx_kp_s_8u, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 300, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "FastImage.pyx":301
 *         result.step = self.step
 *         if self.basetype == '8u':
 *             result.im = IMPOS8u(<ipp.Ipp8u*>self.im,self.step,bottom,left)             # <<<<<<<<<<<<<<
 *         elif self.basetype == '32f':
 *             result.im = IMPOS32f(<ipp.Ipp32f*>self.im,self.step,bottom,left)
 */
    __pyx_v_result->im = IMPOS8u(((Ipp8u *)__pyx_v_self->im), __pyx_v_self->step, __pyx_v_bottom, __pyx_v_left);

    /* "FastImage.pyx":300
 *         result.strides[0] = self.strides[0]
 *         result.step = self.step
 *         if self.basetype == '8u':             # <<<<<<<<<<<<<<
 *             result.im = IMPOS8u(<ipp.Ipp8u*>self.im,self.step,bottom,left)
 *         elif self.basetype == '32f':
 */
    goto __pyx_L4;
  }

  /* "FastImage.pyx":302
 *         if self.basetype == '8u':
 *             result.im = IMPOS8u(<ipp.Ipp8u*>self.im,self.step,bottom,left)
 *         elif self.basetype == '32f':             # <<<<<<<<<<<<<<
 *             result.im = IMPOS32f(<ipp.Ipp32f*>self.im,self.step,bottom,left)
 *         else:
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_self->basetype, __pyx_kp_s_32f, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 302, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "FastImage.pyx":303
 *             result.im = IMPOS8u(<ipp.Ipp8u*>self.im,self.step,bottom,left)
 *         elif self.basetype == '32f':
 *             result.im = IMPOS32f(<ipp.Ipp32f*>self.im,self.step,bottom,left)             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError('unknown basetype: %r'%self.basetype)
 */
    __pyx_v_result->im = IMPOS32f(((Ipp32f *)__pyx_v_self->im), __pyx_v_self->step, __pyx_v_bottom, __pyx_v_left);

    /* "FastImage.pyx":302
 *         if self.basetype == '8u':
 *             result.im = IMPOS8u(<ipp.Ipp8u*>self.im,self.step,bottom,left)
 *         elif self.basetype == '32f':             # <<<<<<<<<<<<<<
 *             result.im = IMPOS32f(<ipp.Ipp32f*>self.im,self.step,bottom,left)
 *         else:
 */
    goto __pyx_L4;
  }

  /* "FastImage.pyx":305
 *             result.im = IMPOS32f(<ipp.Ipp32f*>self.im,self.step,bottom,left)
 *         else:
 *             raise ValueError('unknown basetype: %r'%self.basetype)             # <<<<<<<<<<<<<<
 *         if FASTIMAGEDEBUG>=2:
 *             print self,'assigning ROI from self (im: %#x)\n    to (%s: 0x%0x) (L:%d,B:%d)'%(<intptr_t>self.im,
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_unknown_basetype_r, __pyx_v_self->basetype); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 305, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 305, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 305, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 305, __pyx_L1_error)
  }
  __pyx_L4:;

  /* "FastImage.pyx":306
 *         else:
 *             raise ValueError('unknown basetype: %r'%self.basetype)
 *         if FASTIMAGEDEBUG>=2:             # <<<<<<<<<<<<<<
 *             print self,'assigning ROI from self (im: %#x)\n    to (%s: 0x%0x) (L:%d,B:%d)'%(<intptr_t>self.im,
 *                                                           str(result),
 */
  __pyx_t_1 = ((__pyx_v_9FastImage_FASTIMAGEDEBUG >= 2) != 0);
  if (__pyx_t_1) {

    /* "FastImage.pyx":307
 *             raise ValueError('unknown basetype: %r'%self.basetype)
 *         if FASTIMAGEDEBUG>=2:
 *             print self,'assigning ROI from self (im: %#x)\n    to (%s: 0x%0x) (L:%d,B:%d)'%(<intptr_t>self.im,             # <<<<<<<<<<<<<<
 *                                                           str(result),
 *                                                           <intptr_t>result.im,
 */
    __pyx_t_2 = __Pyx_PyInt_From_intptr_t(((intptr_t)__pyx_v_self->im)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);

    /* "FastImage.pyx":308
 *         if FASTIMAGEDEBUG>=2:
 *             print self,'assigning ROI from self (im: %#x)\n    to (%s: 0x%0x) (L:%d,B:%d)'%(<intptr_t>self.im,
 *                                                           str(result),             # <<<<<<<<<<<<<<
 *                                                           <intptr_t>result.im,
 *                                                           left,bottom)
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)__pyx_v_result));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_result));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_result));
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)(&PyString_Type)), __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 308, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "FastImage.pyx":309
 *             print self,'assigning ROI from self (im: %#x)\n    to (%s: 0x%0x) (L:%d,B:%d)'%(<intptr_t>self.im,
 *                                                           str(result),
 *                                                           <intptr_t>result.im,             # <<<<<<<<<<<<<<
 *                                                           left,bottom)
 *         result.source_data = self # keep reference to original image in case it goes out of scope elsewhere
 */
    __pyx_t_3 = __Pyx_PyInt_From_intptr_t(((intptr_t)__pyx_v_result->im)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 309, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);

    /* "FastImage.pyx":310
 *                                                           str(result),
 *                                                           <intptr_t>result.im,
 *                                                           left,bottom)             # <<<<<<<<<<<<<<
 *         result.source_data = self # keep reference to original image in case it goes out of scope elsewhere
 *         return result
 */
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_left); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 310, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_bottom); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 310, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);

    /* "FastImage.pyx":307
 *             raise ValueError('unknown basetype: %r'%self.basetype)
 *         if FASTIMAGEDEBUG>=2:
 *             print self,'assigning ROI from self (im: %#x)\n    to (%s: 0x%0x) (L:%d,B:%d)'%(<intptr_t>self.im,             # <<<<<<<<<<<<<<
 *                                                           str(result),
 *                                                           <intptr_t>result.im,
 */
    __pyx_t_8 = PyTuple_New(5); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_8, 2, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_8, 3, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_8, 4, __pyx_t_7);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_t_3 = 0;
    __pyx_t_5 = 0;
    __pyx_t_7 = 0;
    __pyx_t_7 = __Pyx_PyString_Format(__pyx_kp_s_assigning_ROI_from_self_im_x_to, __pyx_t_8); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __pyx_t_8 = PyTuple_New(2); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_8, 0, ((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_8, 1, __pyx_t_7);
    __pyx_t_7 = 0;
    if (__Pyx_Print(0, __pyx_t_8, 1) < 0) __PYX_ERR(0, 307, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "FastImage.pyx":306
 *         else:
 *             raise ValueError('unknown basetype: %r'%self.basetype)
 *         if FASTIMAGEDEBUG>=2:             # <<<<<<<<<<<<<<
 *             print self,'assigning ROI from self (im: %#x)\n    to (%s: 0x%0x) (L:%d,B:%d)'%(<intptr_t>self.im,
 *                                                           str(result),
 */
  }

  /* "FastImage.pyx":311
 *                                                           <intptr_t>result.im,
 *                                                           left,bottom)
 *         result.source_data = self # keep reference to original image in case it goes out of scope elsewhere             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  __Pyx_GOTREF(__pyx_v_result->source_data);
  __Pyx_DECREF(__pyx_v_result->source_data);
  __pyx_v_result->source_data = ((PyObject *)__pyx_v_self);

  /* "FastImage.pyx":312
 *                                                           left,bottom)
 *         result.source_data = self # keep reference to original image in case it goes out of scope elsewhere
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def roi( self, int left, int bottom, Size size):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "FastImage.pyx":290
 *             return self.imsiz
 * 
 *     cdef FastImageBase c_roi( self, int left, int bottom, Size size):             # <<<<<<<<<<<<<<
 *         cdef FastImageBase result
 *         global FASTIMAGEDEBUG
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("FastImage.FastImageBase.c_roi", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":314
 *         return result
 * 
 *     def roi( self, int left, int bottom, Size size):             # <<<<<<<<<<<<<<
 *         # python version of C version
 *         return self.c_roi(left,bottom,size)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_13FastImageBase_13roi(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_13FastImageBase_13roi(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_left;
  int __pyx_v_bottom;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("roi (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_left,&__pyx_n_s_bottom,&__pyx_n_s_size,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_left)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_bottom)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("roi", 1, 3, 3, 1); __PYX_ERR(0, 314, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("roi", 1, 3, 3, 2); __PYX_ERR(0, 314, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "roi") < 0)) __PYX_ERR(0, 314, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_left = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_left == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 314, __pyx_L3_error)
    __pyx_v_bottom = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_bottom == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 314, __pyx_L3_error)
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("roi", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 314, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImageBase.roi", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 314, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_13FastImageBase_12roi(((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_v_self), __pyx_v_left, __pyx_v_bottom, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_13FastImageBase_12roi(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self, int __pyx_v_left, int __pyx_v_bottom, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  __Pyx_RefNannySetupContext("roi", 0);

  /* "FastImage.pyx":316
 *     def roi( self, int left, int bottom, Size size):
 *         # python version of C version
 *         return self.c_roi(left,bottom,size)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self,a):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_9FastImage_FastImageBase *)__pyx_v_self->__pyx_vtab)->c_roi(__pyx_v_self, __pyx_v_left, __pyx_v_bottom, __pyx_v_size)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 316, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":314
 *         return result
 * 
 *     def roi( self, int left, int bottom, Size size):             # <<<<<<<<<<<<<<
 *         # python version of C version
 *         return self.c_roi(left,bottom,size)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("FastImage.FastImageBase.roi", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":318
 *         return self.c_roi(left,bottom,size)
 * 
 *     def __getitem__(self,a):             # <<<<<<<<<<<<<<
 *         cdef int i,j
 *         cdef fiptr valptr
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_13FastImageBase_15__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_a); /*proto*/
static PyObject *__pyx_pw_9FastImage_13FastImageBase_15__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_a) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_13FastImageBase_14__getitem__(((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_v_self), ((PyObject *)__pyx_v_a));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_13FastImageBase_14__getitem__(struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_self, PyObject *__pyx_v_a) {
  int __pyx_v_i;
  int __pyx_v_j;
  __pyx_t_9FastImage_fiptr __pyx_v_valptr;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *(*__pyx_t_4)(PyObject *);
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "FastImage.pyx":322
 *         cdef fiptr valptr
 * 
 *         i,j = a # a must be sequence of ints             # <<<<<<<<<<<<<<
 * 
 *         valptr = (self.im+i*self.step)+j*self.strides[1]
 */
  if ((likely(PyTuple_CheckExact(__pyx_v_a))) || (PyList_CheckExact(__pyx_v_a))) {
    PyObject* sequence = __pyx_v_a;
    #if !CYTHON_COMPILING_IN_PYPY
    Py_ssize_t size = Py_SIZE(sequence);
    #else
    Py_ssize_t size = PySequence_Size(sequence);
    #endif
    if (unlikely(size != 2)) {
      if (size > 2) __Pyx_RaiseTooManyValuesError(2);
      else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
      __PYX_ERR(0, 322, __pyx_L1_error)
    }
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    if (likely(PyTuple_CheckExact(sequence))) {
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyTuple_GET_ITEM(sequence, 1); 
    } else {
      __pyx_t_1 = PyList_GET_ITEM(sequence, 0); 
      __pyx_t_2 = PyList_GET_ITEM(sequence, 1); 
    }
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_t_2);
    #else
    __pyx_t_1 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    #endif
  } else {
    Py_ssize_t index = -1;
    __pyx_t_3 = PyObject_GetIter(__pyx_v_a); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = Py_TYPE(__pyx_t_3)->tp_iternext;
    index = 0; __pyx_t_1 = __pyx_t_4(__pyx_t_3); if (unlikely(!__pyx_t_1)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_1);
    index = 1; __pyx_t_2 = __pyx_t_4(__pyx_t_3); if (unlikely(!__pyx_t_2)) goto __pyx_L3_unpacking_failed;
    __Pyx_GOTREF(__pyx_t_2);
    if (__Pyx_IternextUnpackEndCheck(__pyx_t_4(__pyx_t_3), 2) < 0) __PYX_ERR(0, 322, __pyx_L1_error)
    __pyx_t_4 = NULL;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L4_unpacking_done;
    __pyx_L3_unpacking_failed:;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
    if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
    __PYX_ERR(0, 322, __pyx_L1_error)
    __pyx_L4_unpacking_done:;
  }
  __pyx_t_5 = __Pyx_PyInt_As_int(__pyx_t_1); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 322, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = __Pyx_PyInt_As_int(__pyx_t_2); if (unlikely((__pyx_t_6 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 322, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_i = __pyx_t_5;
  __pyx_v_j = __pyx_t_6;

  /* "FastImage.pyx":324
 *         i,j = a # a must be sequence of ints
 * 
 *         valptr = (self.im+i*self.step)+j*self.strides[1]             # <<<<<<<<<<<<<<
 * 
 *         if self.basetype == '8u':
 */
  __pyx_v_valptr = ((__pyx_v_self->im + (__pyx_v_i * __pyx_v_self->step)) + (__pyx_v_j * (__pyx_v_self->strides[1])));

  /* "FastImage.pyx":326
 *         valptr = (self.im+i*self.step)+j*self.strides[1]
 * 
 *         if self.basetype == '8u':             # <<<<<<<<<<<<<<
 *             return (<ipp.Ipp8u*>valptr)[0]
 *         elif self.basetype == '32f':
 */
  __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_v_self->basetype, __pyx_kp_s_8u, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 326, __pyx_L1_error)
  if (__pyx_t_7) {

    /* "FastImage.pyx":327
 * 
 *         if self.basetype == '8u':
 *             return (<ipp.Ipp8u*>valptr)[0]             # <<<<<<<<<<<<<<
 *         elif self.basetype == '32f':
 *             return (<ipp.Ipp32f*>valptr)[0]
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyInt_From_Ipp8u((((Ipp8u *)__pyx_v_valptr)[0])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 327, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "FastImage.pyx":326
 *         valptr = (self.im+i*self.step)+j*self.strides[1]
 * 
 *         if self.basetype == '8u':             # <<<<<<<<<<<<<<
 *             return (<ipp.Ipp8u*>valptr)[0]
 *         elif self.basetype == '32f':
 */
  }

  /* "FastImage.pyx":328
 *         if self.basetype == '8u':
 *             return (<ipp.Ipp8u*>valptr)[0]
 *         elif self.basetype == '32f':             # <<<<<<<<<<<<<<
 *             return (<ipp.Ipp32f*>valptr)[0]
 *         else:
 */
  __pyx_t_7 = (__Pyx_PyString_Equals(__pyx_v_self->basetype, __pyx_kp_s_32f, Py_EQ)); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 328, __pyx_L1_error)
  if (__pyx_t_7) {

    /* "FastImage.pyx":329
 *             return (<ipp.Ipp8u*>valptr)[0]
 *         elif self.basetype == '32f':
 *             return (<ipp.Ipp32f*>valptr)[0]             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError('unknown basetype: %r'%self.basetype)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyFloat_FromDouble((((Ipp32f *)__pyx_v_valptr)[0])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 329, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "FastImage.pyx":328
 *         if self.basetype == '8u':
 *             return (<ipp.Ipp8u*>valptr)[0]
 *         elif self.basetype == '32f':             # <<<<<<<<<<<<<<
 *             return (<ipp.Ipp32f*>valptr)[0]
 *         else:
 */
  }

  /* "FastImage.pyx":331
 *             return (<ipp.Ipp32f*>valptr)[0]
 *         else:
 *             raise ValueError('unknown basetype: %r'%self.basetype)             # <<<<<<<<<<<<<<
 * 
 * ##    def __getitem__(self,*args,**kw):
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_unknown_basetype_r, __pyx_v_self->basetype); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 331, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 331, __pyx_L1_error)
  }

  /* "FastImage.pyx":318
 *         return self.c_roi(left,bottom,size)
 * 
 *     def __getitem__(self,a):             # <<<<<<<<<<<<<<
 *         cdef int i,j
 *         cdef fiptr valptr
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("FastImage.FastImageBase.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":341
 *     """a fast 2D array of 8u"""
 * 
 *     def __cinit__(self,*args,**kw):             # <<<<<<<<<<<<<<
 *         self.strides[1] = 1
 *         self.basetype = '8u'
 */

/* Python wrapper */
static int __pyx_pw_9FastImage_11FastImage8u_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9FastImage_11FastImage8u_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_args = 0;
  CYTHON_UNUSED PyObject *__pyx_v_kw = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 1))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u___cinit__(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_args, __pyx_v_kw);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_kw);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9FastImage_11FastImage8u___cinit__(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kw) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "FastImage.pyx":342
 * 
 *     def __cinit__(self,*args,**kw):
 *         self.strides[1] = 1             # <<<<<<<<<<<<<<
 *         self.basetype = '8u'
 * 
 */
  (__pyx_v_self->__pyx_base.strides[1]) = 1;

  /* "FastImage.pyx":343
 *     def __cinit__(self,*args,**kw):
 *         self.strides[1] = 1
 *         self.basetype = '8u'             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, Size size ):
 */
  __Pyx_INCREF(__pyx_kp_s_8u);
  __Pyx_GIVEREF(__pyx_kp_s_8u);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.basetype);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.basetype);
  __pyx_v_self->__pyx_base.basetype = __pyx_kp_s_8u;

  /* "FastImage.pyx":341
 *     """a fast 2D array of 8u"""
 * 
 *     def __cinit__(self,*args,**kw):             # <<<<<<<<<<<<<<
 *         self.strides[1] = 1
 *         self.basetype = '8u'
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":345
 *         self.basetype = '8u'
 * 
 *     def __init__(self, Size size ):             # <<<<<<<<<<<<<<
 *         global FASTIMAGEDEBUG
 * 
 */

/* Python wrapper */
static int __pyx_pw_9FastImage_11FastImage8u_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9FastImage_11FastImage8u_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_size,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 345, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 345, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 345, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_2__init__(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9FastImage_11FastImage8u_2__init__(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "FastImage.pyx":348
 *         global FASTIMAGEDEBUG
 * 
 *         FastImageBase.__init__(self, size)             # <<<<<<<<<<<<<<
 *         if FASTIMAGEDEBUG>=1:
 *             print self,'requesting memory (size W:%d H:%d)'%(self.imsiz.sz.width,
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9FastImage_FastImageBase), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 348, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, ((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_size)};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 348, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, ((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_size)};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 348, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 348, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, ((PyObject *)__pyx_v_self));
    __Pyx_INCREF(((PyObject *)__pyx_v_size));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, ((PyObject *)__pyx_v_size));
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 348, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "FastImage.pyx":349
 * 
 *         FastImageBase.__init__(self, size)
 *         if FASTIMAGEDEBUG>=1:             # <<<<<<<<<<<<<<
 *             print self,'requesting memory (size W:%d H:%d)'%(self.imsiz.sz.width,
 *                                                              self.imsiz.sz.height)
 */
  __pyx_t_6 = ((__pyx_v_9FastImage_FASTIMAGEDEBUG >= 1) != 0);
  if (__pyx_t_6) {

    /* "FastImage.pyx":350
 *         FastImageBase.__init__(self, size)
 *         if FASTIMAGEDEBUG>=1:
 *             print self,'requesting memory (size W:%d H:%d)'%(self.imsiz.sz.width,             # <<<<<<<<<<<<<<
 *                                                              self.imsiz.sz.height)
 *             if FASTIMAGEDEBUG>=3:
 */
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.imsiz->sz.width); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 350, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "FastImage.pyx":351
 *         if FASTIMAGEDEBUG>=1:
 *             print self,'requesting memory (size W:%d H:%d)'%(self.imsiz.sz.width,
 *                                                              self.imsiz.sz.height)             # <<<<<<<<<<<<<<
 *             if FASTIMAGEDEBUG>=3:
 *                 sys.stderr.write('-='*20+'\n')
 */
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.imsiz->sz.height); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 351, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);

    /* "FastImage.pyx":350
 *         FastImageBase.__init__(self, size)
 *         if FASTIMAGEDEBUG>=1:
 *             print self,'requesting memory (size W:%d H:%d)'%(self.imsiz.sz.width,             # <<<<<<<<<<<<<<
 *                                                              self.imsiz.sz.height)
 *             if FASTIMAGEDEBUG>=3:
 */
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 350, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_requesting_memory_size_W_d_H_d, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 350, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 350, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
    __pyx_t_2 = 0;
    if (__Pyx_Print(0, __pyx_t_5, 1) < 0) __PYX_ERR(0, 350, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "FastImage.pyx":352
 *             print self,'requesting memory (size W:%d H:%d)'%(self.imsiz.sz.width,
 *                                                              self.imsiz.sz.height)
 *             if FASTIMAGEDEBUG>=3:             # <<<<<<<<<<<<<<
 *                 sys.stderr.write('-='*20+'\n')
 *                 traceback.print_stack()
 */
    __pyx_t_6 = ((__pyx_v_9FastImage_FASTIMAGEDEBUG >= 3) != 0);
    if (__pyx_t_6) {

      /* "FastImage.pyx":353
 *                                                              self.imsiz.sz.height)
 *             if FASTIMAGEDEBUG>=3:
 *                 sys.stderr.write('-='*20+'\n')             # <<<<<<<<<<<<<<
 *                 traceback.print_stack()
 *                 sys.stderr.write('-='*20+'\n\n')
 */
      __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_sys); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 353, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_stderr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 353, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_write); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 353, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyNumber_Multiply(__pyx_kp_s__12, __pyx_int_20); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 353, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_kp_s__8); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 353, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_2, function);
        }
      }
      if (!__pyx_t_1) {
        __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 353, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_5);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_2)) {
          PyObject *__pyx_temp[2] = {__pyx_t_1, __pyx_t_3};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 353, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
          PyObject *__pyx_temp[2] = {__pyx_t_1, __pyx_t_3};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 353, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(1+1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 353, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_1); __pyx_t_1 = NULL;
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_7, 0+1, __pyx_t_3);
          __pyx_t_3 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_7, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 353, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "FastImage.pyx":354
 *             if FASTIMAGEDEBUG>=3:
 *                 sys.stderr.write('-='*20+'\n')
 *                 traceback.print_stack()             # <<<<<<<<<<<<<<
 *                 sys.stderr.write('-='*20+'\n\n')
 *                 sys.stderr.flush()
 */
      __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_traceback); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 354, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_print_stack); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 354, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      if (__pyx_t_2) {
        __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 354, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else {
        __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 354, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "FastImage.pyx":355
 *                 sys.stderr.write('-='*20+'\n')
 *                 traceback.print_stack()
 *                 sys.stderr.write('-='*20+'\n\n')             # <<<<<<<<<<<<<<
 *                 sys.stderr.flush()
 * 
 */
      __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_sys); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_stderr); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_write); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = PyNumber_Multiply(__pyx_kp_s__12, __pyx_int_20); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_kp_s__13); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 355, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      if (!__pyx_t_2) {
        __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 355, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_5);
      } else {
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_t_3};
          __pyx_t_5 = __Pyx_PyFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 355, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_7)) {
          PyObject *__pyx_temp[2] = {__pyx_t_2, __pyx_t_3};
          __pyx_t_5 = __Pyx_PyCFunction_FastCall(__pyx_t_7, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 355, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_1 = PyTuple_New(1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 355, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_GIVEREF(__pyx_t_2); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2); __pyx_t_2 = NULL;
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_1, 0+1, __pyx_t_3);
          __pyx_t_3 = 0;
          __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_7, __pyx_t_1, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 355, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        }
      }
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "FastImage.pyx":356
 *                 traceback.print_stack()
 *                 sys.stderr.write('-='*20+'\n\n')
 *                 sys.stderr.flush()             # <<<<<<<<<<<<<<
 * 
 *         self.im=ipp.ippiMalloc_8u_C1( size.sz.width, size.sz.height, &self.step )
 */
      __pyx_t_7 = __Pyx_GetModuleGlobalName(__pyx_n_s_sys); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 356, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_stderr); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 356, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_flush); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 356, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_7);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_7, function);
        }
      }
      if (__pyx_t_1) {
        __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 356, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_7); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 356, __pyx_L1_error)
      }
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "FastImage.pyx":352
 *             print self,'requesting memory (size W:%d H:%d)'%(self.imsiz.sz.width,
 *                                                              self.imsiz.sz.height)
 *             if FASTIMAGEDEBUG>=3:             # <<<<<<<<<<<<<<
 *                 sys.stderr.write('-='*20+'\n')
 *                 traceback.print_stack()
 */
    }

    /* "FastImage.pyx":349
 * 
 *         FastImageBase.__init__(self, size)
 *         if FASTIMAGEDEBUG>=1:             # <<<<<<<<<<<<<<
 *             print self,'requesting memory (size W:%d H:%d)'%(self.imsiz.sz.width,
 *                                                              self.imsiz.sz.height)
 */
  }

  /* "FastImage.pyx":358
 *                 sys.stderr.flush()
 * 
 *         self.im=ipp.ippiMalloc_8u_C1( size.sz.width, size.sz.height, &self.step )             # <<<<<<<<<<<<<<
 *         if self.step<size.sz.width:
 *             raise MemoryError('self.step is %d (width=%d, height=%d)'%(
 */
  __pyx_v_self->__pyx_base.im = ippiMalloc_8u_C1(__pyx_v_size->sz.width, __pyx_v_size->sz.height, (&__pyx_v_self->__pyx_base.step));

  /* "FastImage.pyx":359
 * 
 *         self.im=ipp.ippiMalloc_8u_C1( size.sz.width, size.sz.height, &self.step )
 *         if self.step<size.sz.width:             # <<<<<<<<<<<<<<
 *             raise MemoryError('self.step is %d (width=%d, height=%d)'%(
 *                 self.step, size.sz.width, size.sz.height))
 */
  __pyx_t_6 = ((__pyx_v_self->__pyx_base.step < __pyx_v_size->sz.width) != 0);
  if (__pyx_t_6) {

    /* "FastImage.pyx":361
 *         if self.step<size.sz.width:
 *             raise MemoryError('self.step is %d (width=%d, height=%d)'%(
 *                 self.step, size.sz.width, size.sz.height))             # <<<<<<<<<<<<<<
 *         if self.im==NULL: raise MemoryError("Error allocating memory")
 *         self.strides[0] = self.step
 */
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.step); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 361, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_7 = __Pyx_PyInt_From_int(__pyx_v_size->sz.width); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 361, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_size->sz.height); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 361, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 361, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_7);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_7);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_1);
    __pyx_t_5 = 0;
    __pyx_t_7 = 0;
    __pyx_t_1 = 0;

    /* "FastImage.pyx":360
 *         self.im=ipp.ippiMalloc_8u_C1( size.sz.width, size.sz.height, &self.step )
 *         if self.step<size.sz.width:
 *             raise MemoryError('self.step is %d (width=%d, height=%d)'%(             # <<<<<<<<<<<<<<
 *                 self.step, size.sz.width, size.sz.height))
 *         if self.im==NULL: raise MemoryError("Error allocating memory")
 */
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_self_step_is_d_width_d_height_d, __pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 360, __pyx_L1_error)

    /* "FastImage.pyx":359
 * 
 *         self.im=ipp.ippiMalloc_8u_C1( size.sz.width, size.sz.height, &self.step )
 *         if self.step<size.sz.width:             # <<<<<<<<<<<<<<
 *             raise MemoryError('self.step is %d (width=%d, height=%d)'%(
 *                 self.step, size.sz.width, size.sz.height))
 */
  }

  /* "FastImage.pyx":362
 *             raise MemoryError('self.step is %d (width=%d, height=%d)'%(
 *                 self.step, size.sz.width, size.sz.height))
 *         if self.im==NULL: raise MemoryError("Error allocating memory")             # <<<<<<<<<<<<<<
 *         self.strides[0] = self.step
 *         if FASTIMAGEDEBUG>=1:
 */
  __pyx_t_6 = ((__pyx_v_self->__pyx_base.im == NULL) != 0);
  if (__pyx_t_6) {
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__14, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 362, __pyx_L1_error)
  }

  /* "FastImage.pyx":363
 *                 self.step, size.sz.width, size.sz.height))
 *         if self.im==NULL: raise MemoryError("Error allocating memory")
 *         self.strides[0] = self.step             # <<<<<<<<<<<<<<
 *         if FASTIMAGEDEBUG>=1:
 *             self.show_mem_and_step()
 */
  __pyx_t_4 = __pyx_v_self->__pyx_base.step;
  (__pyx_v_self->__pyx_base.strides[0]) = __pyx_t_4;

  /* "FastImage.pyx":364
 *         if self.im==NULL: raise MemoryError("Error allocating memory")
 *         self.strides[0] = self.step
 *         if FASTIMAGEDEBUG>=1:             # <<<<<<<<<<<<<<
 *             self.show_mem_and_step()
 *         self.view = 0
 */
  __pyx_t_6 = ((__pyx_v_9FastImage_FASTIMAGEDEBUG >= 1) != 0);
  if (__pyx_t_6) {

    /* "FastImage.pyx":365
 *         self.strides[0] = self.step
 *         if FASTIMAGEDEBUG>=1:
 *             self.show_mem_and_step()             # <<<<<<<<<<<<<<
 *         self.view = 0
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_show_mem_and_step); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 365, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_7 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_7)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_7);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    if (__pyx_t_7) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 365, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    } else {
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 365, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "FastImage.pyx":364
 *         if self.im==NULL: raise MemoryError("Error allocating memory")
 *         self.strides[0] = self.step
 *         if FASTIMAGEDEBUG>=1:             # <<<<<<<<<<<<<<
 *             self.show_mem_and_step()
 *         self.view = 0
 */
  }

  /* "FastImage.pyx":366
 *         if FASTIMAGEDEBUG>=1:
 *             self.show_mem_and_step()
 *         self.view = 0             # <<<<<<<<<<<<<<
 * 
 *     def set_val(self, int val, Size size):
 */
  __pyx_v_self->__pyx_base.view = 0;

  /* "FastImage.pyx":345
 *         self.basetype = '8u'
 * 
 *     def __init__(self, Size size ):             # <<<<<<<<<<<<<<
 *         global FASTIMAGEDEBUG
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("FastImage.FastImage8u.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":368
 *         self.view = 0
 * 
 *     def set_val(self, int val, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSet_8u_C1R( val, <ipp.Ipp8u*>self.im, self.step, size.sz ))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_5set_val(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_5set_val(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_val;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_val (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_val,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_val)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_val", 1, 2, 2, 1); __PYX_ERR(0, 368, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_val") < 0)) __PYX_ERR(0, 368, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_val = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_val == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 368, __pyx_L3_error)
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_val", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 368, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.set_val", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 368, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_4set_val(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_val, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_4set_val(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, int __pyx_v_val, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_val", 0);

  /* "FastImage.pyx":369
 * 
 *     def set_val(self, int val, Size size):
 *         CHK_HAVEGIL( ipp.ippiSet_8u_C1R( val, <ipp.Ipp8u*>self.im, self.step, size.sz ))             # <<<<<<<<<<<<<<
 * 
 *     def set_val_masked(self, int val, FastImage8u mask, Size size):
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSet_8u_C1R(__pyx_v_val, ((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 369, __pyx_L1_error)

  /* "FastImage.pyx":368
 *         self.view = 0
 * 
 *     def set_val(self, int val, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSet_8u_C1R( val, <ipp.Ipp8u*>self.im, self.step, size.sz ))
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.set_val", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":371
 *         CHK_HAVEGIL( ipp.ippiSet_8u_C1R( val, <ipp.Ipp8u*>self.im, self.step, size.sz ))
 * 
 *     def set_val_masked(self, int val, FastImage8u mask, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSet_8u_C1MR( val, <ipp.Ipp8u*>self.im, self.step, size.sz,
 *                                           <ipp.Ipp8u*>mask.im, mask.step))
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_7set_val_masked(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_7set_val_masked(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_val;
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_mask = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_val_masked (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_val,&__pyx_n_s_mask,&__pyx_n_s_size,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_val)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_mask)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_val_masked", 1, 3, 3, 1); __PYX_ERR(0, 371, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_val_masked", 1, 3, 3, 2); __PYX_ERR(0, 371, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_val_masked") < 0)) __PYX_ERR(0, 371, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_val = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_val == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 371, __pyx_L3_error)
    __pyx_v_mask = ((struct __pyx_obj_9FastImage_FastImage8u *)values[1]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_val_masked", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 371, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.set_val_masked", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_mask), __pyx_ptype_9FastImage_FastImage8u, 1, "mask", 0))) __PYX_ERR(0, 371, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 371, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_6set_val_masked(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_val, __pyx_v_mask, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_6set_val_masked(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, int __pyx_v_val, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_mask, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_val_masked", 0);

  /* "FastImage.pyx":372
 * 
 *     def set_val_masked(self, int val, FastImage8u mask, Size size):
 *         CHK_HAVEGIL( ipp.ippiSet_8u_C1MR( val, <ipp.Ipp8u*>self.im, self.step, size.sz,             # <<<<<<<<<<<<<<
 *                                           <ipp.Ipp8u*>mask.im, mask.step))
 * 
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSet_8u_C1MR(__pyx_v_val, ((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz, ((Ipp8u *)__pyx_v_mask->__pyx_base.im), __pyx_v_mask->__pyx_base.step)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 372, __pyx_L1_error)

  /* "FastImage.pyx":371
 *         CHK_HAVEGIL( ipp.ippiSet_8u_C1R( val, <ipp.Ipp8u*>self.im, self.step, size.sz ))
 * 
 *     def set_val_masked(self, int val, FastImage8u mask, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSet_8u_C1MR( val, <ipp.Ipp8u*>self.im, self.step, size.sz,
 *                                           <ipp.Ipp8u*>mask.im, mask.step))
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.set_val_masked", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":375
 *                                           <ipp.Ipp8u*>mask.im, mask.step))
 * 
 *     cdef void fast_set_val_masked( self, ipp.Ipp8u val, FastImage8u mask, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSet_8u_C1MR( val, <ipp.Ipp8u*>self.im, self.step, size.sz,
 *                                         <ipp.Ipp8u*>mask.im, mask.step))
 */

static void __pyx_f_9FastImage_11FastImage8u_fast_set_val_masked(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, Ipp8u __pyx_v_val, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_mask, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fast_set_val_masked", 0);

  /* "FastImage.pyx":376
 * 
 *     cdef void fast_set_val_masked( self, ipp.Ipp8u val, FastImage8u mask, Size size):
 *         CHK_HAVEGIL( ipp.ippiSet_8u_C1MR( val, <ipp.Ipp8u*>self.im, self.step, size.sz,             # <<<<<<<<<<<<<<
 *                                         <ipp.Ipp8u*>mask.im, mask.step))
 * 
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSet_8u_C1MR(__pyx_v_val, ((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz, ((Ipp8u *)__pyx_v_mask->__pyx_base.im), __pyx_v_mask->__pyx_base.step)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 376, __pyx_L1_error)

  /* "FastImage.pyx":375
 *                                           <ipp.Ipp8u*>mask.im, mask.step))
 * 
 *     cdef void fast_set_val_masked( self, ipp.Ipp8u val, FastImage8u mask, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSet_8u_C1MR( val, <ipp.Ipp8u*>self.im, self.step, size.sz,
 *                                         <ipp.Ipp8u*>mask.im, mask.step))
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("FastImage.FastImage8u.fast_set_val_masked", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":379
 *                                         <ipp.Ipp8u*>mask.im, mask.step))
 * 
 *     def get_32f_copy(self,Size size):             # <<<<<<<<<<<<<<
 *         cdef FastImage32f result
 *         result = FastImage32f(size)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_9get_32f_copy(PyObject *__pyx_v_self, PyObject *__pyx_v_size); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_9get_32f_copy(PyObject *__pyx_v_self, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_32f_copy (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 379, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_8get_32f_copy(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), ((struct __pyx_obj_9FastImage_Size *)__pyx_v_size));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_8get_32f_copy(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("get_32f_copy", 0);

  /* "FastImage.pyx":381
 *     def get_32f_copy(self,Size size):
 *         cdef FastImage32f result
 *         result = FastImage32f(size)             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiConvert_8u32f_C1R(<ipp.Ipp8u*>self.im, self.step,
 *                                                <ipp.Ipp32f*>result.im, result.step,
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_size));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_size));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_FastImage32f), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 381, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "FastImage.pyx":382
 *         cdef FastImage32f result
 *         result = FastImage32f(size)
 *         CHK_HAVEGIL( ipp.ippiConvert_8u32f_C1R(<ipp.Ipp8u*>self.im, self.step,             # <<<<<<<<<<<<<<
 *                                                <ipp.Ipp32f*>result.im, result.step,
 *                                                size.sz ))
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiConvert_8u32f_C1R(((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp32f *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 382, __pyx_L1_error)

  /* "FastImage.pyx":385
 *                                                <ipp.Ipp32f*>result.im, result.step,
 *                                                size.sz ))
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def get_32f_copy_put(self,FastImage32f result,Size size):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "FastImage.pyx":379
 *                                         <ipp.Ipp8u*>mask.im, mask.step))
 * 
 *     def get_32f_copy(self,Size size):             # <<<<<<<<<<<<<<
 *         cdef FastImage32f result
 *         result = FastImage32f(size)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("FastImage.FastImage8u.get_32f_copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":387
 *         return result
 * 
 *     def get_32f_copy_put(self,FastImage32f result,Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiConvert_8u32f_C1R(<ipp.Ipp8u*>self.im, self.step,
 *                                                <ipp.Ipp32f*>result.im, result.step,
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_11get_32f_copy_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_11get_32f_copy_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_32f_copy_put (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_result,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_result)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_32f_copy_put", 1, 2, 2, 1); __PYX_ERR(0, 387, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_32f_copy_put") < 0)) __PYX_ERR(0, 387, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImage32f *)values[0]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_32f_copy_put", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 387, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.get_32f_copy_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_result), __pyx_ptype_9FastImage_FastImage32f, 1, "result", 0))) __PYX_ERR(0, 387, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 387, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_10get_32f_copy_put(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_result, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_10get_32f_copy_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_32f_copy_put", 0);

  /* "FastImage.pyx":388
 * 
 *     def get_32f_copy_put(self,FastImage32f result,Size size):
 *         CHK_HAVEGIL( ipp.ippiConvert_8u32f_C1R(<ipp.Ipp8u*>self.im, self.step,             # <<<<<<<<<<<<<<
 *                                                <ipp.Ipp32f*>result.im, result.step,
 *                                                size.sz ))
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiConvert_8u32f_C1R(((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp32f *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 388, __pyx_L1_error)

  /* "FastImage.pyx":387
 *         return result
 * 
 *     def get_32f_copy_put(self,FastImage32f result,Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiConvert_8u32f_C1R(<ipp.Ipp8u*>self.im, self.step,
 *                                                <ipp.Ipp32f*>result.im, result.step,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.get_32f_copy_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":392
 *                                                size.sz ))
 * 
 *     cdef void fast_get_32f_copy_put(self,FastImage32f result,Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiConvert_8u32f_C1R(<ipp.Ipp8u*>self.im, self.step,
 *                                              <ipp.Ipp32f*>result.im, result.step,
 */

static void __pyx_f_9FastImage_11FastImage8u_fast_get_32f_copy_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fast_get_32f_copy_put", 0);

  /* "FastImage.pyx":393
 * 
 *     cdef void fast_get_32f_copy_put(self,FastImage32f result,Size size):
 *         CHK_HAVEGIL( ipp.ippiConvert_8u32f_C1R(<ipp.Ipp8u*>self.im, self.step,             # <<<<<<<<<<<<<<
 *                                              <ipp.Ipp32f*>result.im, result.step,
 *                                              size.sz ))
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiConvert_8u32f_C1R(((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp32f *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 393, __pyx_L1_error)

  /* "FastImage.pyx":392
 *                                                size.sz ))
 * 
 *     cdef void fast_get_32f_copy_put(self,FastImage32f result,Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiConvert_8u32f_C1R(<ipp.Ipp8u*>self.im, self.step,
 *                                              <ipp.Ipp32f*>result.im, result.step,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("FastImage.FastImage8u.fast_get_32f_copy_put", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":397
 *                                              size.sz ))
 * 
 *     def get_8u_copy(self,Size size):             # <<<<<<<<<<<<<<
 *         cdef FastImage8u result
 *         result = FastImage8u(size)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_13get_8u_copy(PyObject *__pyx_v_self, PyObject *__pyx_v_size); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_13get_8u_copy(PyObject *__pyx_v_self, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_8u_copy (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 397, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_12get_8u_copy(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), ((struct __pyx_obj_9FastImage_Size *)__pyx_v_size));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_12get_8u_copy(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("get_8u_copy", 0);

  /* "FastImage.pyx":399
 *     def get_8u_copy(self,Size size):
 *         cdef FastImage8u result
 *         result = FastImage8u(size)             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiCopy_8u_C1R(<ipp.Ipp8u*>self.im, self.step,
 *                                          <ipp.Ipp8u*>result.im, result.step,
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 399, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_size));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_size));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_FastImage8u), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 399, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "FastImage.pyx":400
 *         cdef FastImage8u result
 *         result = FastImage8u(size)
 *         CHK_HAVEGIL( ipp.ippiCopy_8u_C1R(<ipp.Ipp8u*>self.im, self.step,             # <<<<<<<<<<<<<<
 *                                          <ipp.Ipp8u*>result.im, result.step,
 *                                          size.sz ))
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiCopy_8u_C1R(((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp8u *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 400, __pyx_L1_error)

  /* "FastImage.pyx":403
 *                                          <ipp.Ipp8u*>result.im, result.step,
 *                                          size.sz ))
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def get_8u_copy_put(self,FastImage8u result,Size size):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "FastImage.pyx":397
 *                                              size.sz ))
 * 
 *     def get_8u_copy(self,Size size):             # <<<<<<<<<<<<<<
 *         cdef FastImage8u result
 *         result = FastImage8u(size)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("FastImage.FastImage8u.get_8u_copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":405
 *         return result
 * 
 *     def get_8u_copy_put(self,FastImage8u result,Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiCopy_8u_C1R(<ipp.Ipp8u*>self.im, self.step,
 *                                          <ipp.Ipp8u*>result.im, result.step,
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_15get_8u_copy_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_15get_8u_copy_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_8u_copy_put (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_result,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_result)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_8u_copy_put", 1, 2, 2, 1); __PYX_ERR(0, 405, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_8u_copy_put") < 0)) __PYX_ERR(0, 405, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImage8u *)values[0]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_8u_copy_put", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 405, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.get_8u_copy_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_result), __pyx_ptype_9FastImage_FastImage8u, 1, "result", 0))) __PYX_ERR(0, 405, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 405, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_14get_8u_copy_put(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_result, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_14get_8u_copy_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_8u_copy_put", 0);

  /* "FastImage.pyx":406
 * 
 *     def get_8u_copy_put(self,FastImage8u result,Size size):
 *         CHK_HAVEGIL( ipp.ippiCopy_8u_C1R(<ipp.Ipp8u*>self.im, self.step,             # <<<<<<<<<<<<<<
 *                                          <ipp.Ipp8u*>result.im, result.step,
 *                                          size.sz ))
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiCopy_8u_C1R(((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp8u *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 406, __pyx_L1_error)

  /* "FastImage.pyx":405
 *         return result
 * 
 *     def get_8u_copy_put(self,FastImage8u result,Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiCopy_8u_C1R(<ipp.Ipp8u*>self.im, self.step,
 *                                          <ipp.Ipp8u*>result.im, result.step,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.get_8u_copy_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":410
 *                                          size.sz ))
 * 
 *     def get_absdiff_put(self,FastImage8u other,FastImage8u result,Size size):             # <<<<<<<<<<<<<<
 *         """result = |self - other|"""
 *         CHK_HAVEGIL( ipp.ippiAbsDiff_8u_C1R(<ipp.Ipp8u*>other.im, other.step,
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_17get_absdiff_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9FastImage_11FastImage8u_16get_absdiff_put[] = "result = |self - other|";
static PyObject *__pyx_pw_9FastImage_11FastImage8u_17get_absdiff_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_absdiff_put (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_result,&__pyx_n_s_size,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_result)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_absdiff_put", 1, 3, 3, 1); __PYX_ERR(0, 410, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_absdiff_put", 1, 3, 3, 2); __PYX_ERR(0, 410, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_absdiff_put") < 0)) __PYX_ERR(0, 410, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_other = ((struct __pyx_obj_9FastImage_FastImage8u *)values[0]);
    __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImage8u *)values[1]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_absdiff_put", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 410, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.get_absdiff_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage8u, 1, "other", 0))) __PYX_ERR(0, 410, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_result), __pyx_ptype_9FastImage_FastImage8u, 1, "result", 0))) __PYX_ERR(0, 410, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 410, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_16get_absdiff_put(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_other, __pyx_v_result, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_16get_absdiff_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_absdiff_put", 0);

  /* "FastImage.pyx":412
 *     def get_absdiff_put(self,FastImage8u other,FastImage8u result,Size size):
 *         """result = |self - other|"""
 *         CHK_HAVEGIL( ipp.ippiAbsDiff_8u_C1R(<ipp.Ipp8u*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                             <ipp.Ipp8u*>self.im, self.step,
 *                                             <ipp.Ipp8u*>result.im, result.step,
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiAbsDiff_8u_C1R(((Ipp8u *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, ((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp8u *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 412, __pyx_L1_error)

  /* "FastImage.pyx":410
 *                                          size.sz ))
 * 
 *     def get_absdiff_put(self,FastImage8u other,FastImage8u result,Size size):             # <<<<<<<<<<<<<<
 *         """result = |self - other|"""
 *         CHK_HAVEGIL( ipp.ippiAbsDiff_8u_C1R(<ipp.Ipp8u*>other.im, other.step,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.get_absdiff_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":417
 *                                             size.sz))
 * 
 *     cdef void fast_get_absdiff_put(self, FastImage8u other, FastImage8u result, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiAbsDiff_8u_C1R(<ipp.Ipp8u*>other.im,other.step,
 *                                           <ipp.Ipp8u*>self.im,self.step,
 */

static void __pyx_f_9FastImage_11FastImage8u_fast_get_absdiff_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fast_get_absdiff_put", 0);

  /* "FastImage.pyx":418
 * 
 *     cdef void fast_get_absdiff_put(self, FastImage8u other, FastImage8u result, Size size):
 *         CHK_HAVEGIL( ipp.ippiAbsDiff_8u_C1R(<ipp.Ipp8u*>other.im,other.step,             # <<<<<<<<<<<<<<
 *                                           <ipp.Ipp8u*>self.im,self.step,
 *                                           <ipp.Ipp8u*>result.im,result.step,
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiAbsDiff_8u_C1R(((Ipp8u *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, ((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp8u *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 418, __pyx_L1_error)

  /* "FastImage.pyx":417
 *                                             size.sz))
 * 
 *     cdef void fast_get_absdiff_put(self, FastImage8u other, FastImage8u result, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiAbsDiff_8u_C1R(<ipp.Ipp8u*>other.im,other.step,
 *                                           <ipp.Ipp8u*>self.im,self.step,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("FastImage.FastImage8u.fast_get_absdiff_put", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":423
 *                                           size.sz))
 * 
 *     def get_sub_put(self, FastImage8u other, FastImage8u result, Size size):             # <<<<<<<<<<<<<<
 *         """result = self - other"""
 *         CHK_HAVEGIL( ipp.ippiSub_8u_C1RSfs(<ipp.Ipp8u*>other.im, other.step,
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_19get_sub_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9FastImage_11FastImage8u_18get_sub_put[] = "result = self - other";
static PyObject *__pyx_pw_9FastImage_11FastImage8u_19get_sub_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_sub_put (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_result,&__pyx_n_s_size,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_result)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_sub_put", 1, 3, 3, 1); __PYX_ERR(0, 423, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_sub_put", 1, 3, 3, 2); __PYX_ERR(0, 423, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_sub_put") < 0)) __PYX_ERR(0, 423, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_other = ((struct __pyx_obj_9FastImage_FastImage8u *)values[0]);
    __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImage8u *)values[1]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_sub_put", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 423, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.get_sub_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage8u, 1, "other", 0))) __PYX_ERR(0, 423, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_result), __pyx_ptype_9FastImage_FastImage8u, 1, "result", 0))) __PYX_ERR(0, 423, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 423, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_18get_sub_put(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_other, __pyx_v_result, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_18get_sub_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_sub_put", 0);

  /* "FastImage.pyx":425
 *     def get_sub_put(self, FastImage8u other, FastImage8u result, Size size):
 *         """result = self - other"""
 *         CHK_HAVEGIL( ipp.ippiSub_8u_C1RSfs(<ipp.Ipp8u*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                            <ipp.Ipp8u*>self.im, self.step,
 *                                            <ipp.Ipp8u*>result.im, result.step,
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSub_8u_C1RSfs(((Ipp8u *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, ((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp8u *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz, 0)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 425, __pyx_L1_error)

  /* "FastImage.pyx":423
 *                                           size.sz))
 * 
 *     def get_sub_put(self, FastImage8u other, FastImage8u result, Size size):             # <<<<<<<<<<<<<<
 *         """result = self - other"""
 *         CHK_HAVEGIL( ipp.ippiSub_8u_C1RSfs(<ipp.Ipp8u*>other.im, other.step,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.get_sub_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":429
 *                                            <ipp.Ipp8u*>result.im, result.step,
 *                                            size.sz,0))
 *     cdef void fast_get_sub_put(self, FastImage8u other, FastImage8u result, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSub_8u_C1RSfs(<ipp.Ipp8u*>other.im, other.step,
 *                                          <ipp.Ipp8u*>self.im, self.step,
 */

static void __pyx_f_9FastImage_11FastImage8u_fast_get_sub_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fast_get_sub_put", 0);

  /* "FastImage.pyx":430
 *                                            size.sz,0))
 *     cdef void fast_get_sub_put(self, FastImage8u other, FastImage8u result, Size size):
 *         CHK_HAVEGIL( ipp.ippiSub_8u_C1RSfs(<ipp.Ipp8u*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                          <ipp.Ipp8u*>self.im, self.step,
 *                                          <ipp.Ipp8u*>result.im, result.step,
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSub_8u_C1RSfs(((Ipp8u *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, ((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp8u *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz, 0)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 430, __pyx_L1_error)

  /* "FastImage.pyx":429
 *                                            <ipp.Ipp8u*>result.im, result.step,
 *                                            size.sz,0))
 *     cdef void fast_get_sub_put(self, FastImage8u other, FastImage8u result, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSub_8u_C1RSfs(<ipp.Ipp8u*>other.im, other.step,
 *                                          <ipp.Ipp8u*>self.im, self.step,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("FastImage.FastImage8u.fast_get_sub_put", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":435
 *                                          size.sz,0))
 * 
 *     def toself_threshold( self, int compare_value, int new_value, Size size, int op):             # <<<<<<<<<<<<<<
 *         """self = self (op) compare_value ? new_value : self"""
 *         print 'compare_value',compare_value
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_21toself_threshold(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9FastImage_11FastImage8u_20toself_threshold[] = "self = self (op) compare_value ? new_value : self";
static PyObject *__pyx_pw_9FastImage_11FastImage8u_21toself_threshold(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_compare_value;
  int __pyx_v_new_value;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  int __pyx_v_op;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toself_threshold (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_compare_value,&__pyx_n_s_new_value,&__pyx_n_s_size,&__pyx_n_s_op,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_compare_value)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_new_value)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("toself_threshold", 1, 4, 4, 1); __PYX_ERR(0, 435, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("toself_threshold", 1, 4, 4, 2); __PYX_ERR(0, 435, __pyx_L3_error)
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_op)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("toself_threshold", 1, 4, 4, 3); __PYX_ERR(0, 435, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "toself_threshold") < 0)) __PYX_ERR(0, 435, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_compare_value = __Pyx_PyInt_As_int(values[0]); if (unlikely((__pyx_v_compare_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 435, __pyx_L3_error)
    __pyx_v_new_value = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_new_value == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 435, __pyx_L3_error)
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[2]);
    __pyx_v_op = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_op == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 435, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("toself_threshold", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 435, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.toself_threshold", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 435, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_20toself_threshold(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_compare_value, __pyx_v_new_value, __pyx_v_size, __pyx_v_op);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_20toself_threshold(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, int __pyx_v_compare_value, int __pyx_v_new_value, struct __pyx_obj_9FastImage_Size *__pyx_v_size, int __pyx_v_op) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("toself_threshold", 0);

  /* "FastImage.pyx":437
 *     def toself_threshold( self, int compare_value, int new_value, Size size, int op):
 *         """self = self (op) compare_value ? new_value : self"""
 *         print 'compare_value',compare_value             # <<<<<<<<<<<<<<
 *         print 'new_value',new_value
 *         CHK_HAVEGIL( ipp.ippiThreshold_Val_8u_C1IR(<ipp.Ipp8u*>self.im,self.step,
 */
  __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_compare_value); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_compare_value);
  __Pyx_GIVEREF(__pyx_n_s_compare_value);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_compare_value);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __pyx_t_1 = 0;
  if (__Pyx_Print(0, __pyx_t_2, 1) < 0) __PYX_ERR(0, 437, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":438
 *         """self = self (op) compare_value ? new_value : self"""
 *         print 'compare_value',compare_value
 *         print 'new_value',new_value             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiThreshold_Val_8u_C1IR(<ipp.Ipp8u*>self.im,self.step,
 *                                                    size.sz, compare_value, new_value,
 */
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_new_value); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_new_value);
  __Pyx_GIVEREF(__pyx_n_s_new_value);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_new_value);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
  __pyx_t_2 = 0;
  if (__Pyx_Print(0, __pyx_t_1, 1) < 0) __PYX_ERR(0, 438, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "FastImage.pyx":439
 *         print 'compare_value',compare_value
 *         print 'new_value',new_value
 *         CHK_HAVEGIL( ipp.ippiThreshold_Val_8u_C1IR(<ipp.Ipp8u*>self.im,self.step,             # <<<<<<<<<<<<<<
 *                                                    size.sz, compare_value, new_value,
 *                                                    <ipp.IppCmpOp>op))
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiThreshold_Val_8u_C1IR(((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz, __pyx_v_compare_value, __pyx_v_new_value, ((IppCmpOp)__pyx_v_op))); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 439, __pyx_L1_error)

  /* "FastImage.pyx":435
 *                                          size.sz,0))
 * 
 *     def toself_threshold( self, int compare_value, int new_value, Size size, int op):             # <<<<<<<<<<<<<<
 *         """self = self (op) compare_value ? new_value : self"""
 *         print 'compare_value',compare_value
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("FastImage.FastImage8u.toself_threshold", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":443
 *                                                    <ipp.IppCmpOp>op))
 * 
 *     def get_compare( self, other, Size size, int op ):             # <<<<<<<<<<<<<<
 *         cdef FastImage8u result
 *         result = FastImage8u(size)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_23get_compare(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_23get_compare(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  int __pyx_v_op;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_compare (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_size,&__pyx_n_s_op,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_compare", 1, 3, 3, 1); __PYX_ERR(0, 443, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_op)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_compare", 1, 3, 3, 2); __PYX_ERR(0, 443, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_compare") < 0)) __PYX_ERR(0, 443, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_other = values[0];
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
    __pyx_v_op = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_op == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 443, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_compare", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 443, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.get_compare", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 443, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_22get_compare(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_other, __pyx_v_size, __pyx_v_op);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_22get_compare(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, PyObject *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size, int __pyx_v_op) {
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("get_compare", 0);

  /* "FastImage.pyx":445
 *     def get_compare( self, other, Size size, int op ):
 *         cdef FastImage8u result
 *         result = FastImage8u(size)             # <<<<<<<<<<<<<<
 *         self.get_compare_put(other, result, size, op)
 *         return result
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_size));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_size));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_FastImage8u), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 445, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "FastImage.pyx":446
 *         cdef FastImage8u result
 *         result = FastImage8u(size)
 *         self.get_compare_put(other, result, size, op)             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_compare_put); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_op); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 446, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[5] = {__pyx_t_4, __pyx_v_other, ((PyObject *)__pyx_v_result), ((PyObject *)__pyx_v_size), __pyx_t_3};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 446, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_1)) {
    PyObject *__pyx_temp[5] = {__pyx_t_4, __pyx_v_other, ((PyObject *)__pyx_v_result), ((PyObject *)__pyx_v_size), __pyx_t_3};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_1, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 446, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(4+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 446, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_other);
    __Pyx_GIVEREF(__pyx_v_other);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_other);
    __Pyx_INCREF(((PyObject *)__pyx_v_result));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_result));
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, ((PyObject *)__pyx_v_result));
    __Pyx_INCREF(((PyObject *)__pyx_v_size));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
    PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, ((PyObject *)__pyx_v_size));
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 3+__pyx_t_5, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 446, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":447
 *         result = FastImage8u(size)
 *         self.get_compare_put(other, result, size, op)
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def get_compare_put( self, other, FastImage8u dest, Size size, int op ):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "FastImage.pyx":443
 *                                                    <ipp.IppCmpOp>op))
 * 
 *     def get_compare( self, other, Size size, int op ):             # <<<<<<<<<<<<<<
 *         cdef FastImage8u result
 *         result = FastImage8u(size)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("FastImage.FastImage8u.get_compare", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":449
 *         return result
 * 
 *     def get_compare_put( self, other, FastImage8u dest, Size size, int op ):             # <<<<<<<<<<<<<<
 *         cdef FastImage8u other8u
 *         cdef int other_int
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_25get_compare_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_25get_compare_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_dest = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  int __pyx_v_op;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_compare_put (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_dest,&__pyx_n_s_size,&__pyx_n_s_op,0};
    PyObject* values[4] = {0,0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dest)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_compare_put", 1, 4, 4, 1); __PYX_ERR(0, 449, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_compare_put", 1, 4, 4, 2); __PYX_ERR(0, 449, __pyx_L3_error)
        }
        case  3:
        if (likely((values[3] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_op)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_compare_put", 1, 4, 4, 3); __PYX_ERR(0, 449, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_compare_put") < 0)) __PYX_ERR(0, 449, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 4) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
      values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
    }
    __pyx_v_other = values[0];
    __pyx_v_dest = ((struct __pyx_obj_9FastImage_FastImage8u *)values[1]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[2]);
    __pyx_v_op = __Pyx_PyInt_As_int(values[3]); if (unlikely((__pyx_v_op == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 449, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_compare_put", 1, 4, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 449, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.get_compare_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dest), __pyx_ptype_9FastImage_FastImage8u, 1, "dest", 0))) __PYX_ERR(0, 449, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 449, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_24get_compare_put(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_other, __pyx_v_dest, __pyx_v_size, __pyx_v_op);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_24get_compare_put(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, PyObject *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_dest, struct __pyx_obj_9FastImage_Size *__pyx_v_size, int __pyx_v_op) {
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other8u = 0;
  int __pyx_v_other_int;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  __Pyx_RefNannySetupContext("get_compare_put", 0);

  /* "FastImage.pyx":452
 *         cdef FastImage8u other8u
 *         cdef int other_int
 *         if type(other)==int:             # <<<<<<<<<<<<<<
 *             other_int = other
 *             CHK_HAVEGIL( ipp.ippiCompareC_8u_C1R( <ipp.Ipp8u*>self.im,self.step,
 */
  __pyx_t_1 = PyObject_RichCompare(((PyObject *)Py_TYPE(__pyx_v_other)), ((PyObject *)(&PyInt_Type)), Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 452, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "FastImage.pyx":453
 *         cdef int other_int
 *         if type(other)==int:
 *             other_int = other             # <<<<<<<<<<<<<<
 *             CHK_HAVEGIL( ipp.ippiCompareC_8u_C1R( <ipp.Ipp8u*>self.im,self.step,
 *                                                   other_int,
 */
    __pyx_t_3 = __Pyx_PyInt_As_int(__pyx_v_other); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 453, __pyx_L1_error)
    __pyx_v_other_int = __pyx_t_3;

    /* "FastImage.pyx":454
 *         if type(other)==int:
 *             other_int = other
 *             CHK_HAVEGIL( ipp.ippiCompareC_8u_C1R( <ipp.Ipp8u*>self.im,self.step,             # <<<<<<<<<<<<<<
 *                                                   other_int,
 *                                                   <ipp.Ipp8u*>dest.im,dest.step,
 */
    __pyx_f_9FastImage_CHK_HAVEGIL(ippiCompareC_8u_C1R(((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_other_int, ((Ipp8u *)__pyx_v_dest->__pyx_base.im), __pyx_v_dest->__pyx_base.step, __pyx_v_size->sz, ((IppCmpOp)__pyx_v_op))); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 454, __pyx_L1_error)

    /* "FastImage.pyx":452
 *         cdef FastImage8u other8u
 *         cdef int other_int
 *         if type(other)==int:             # <<<<<<<<<<<<<<
 *             other_int = other
 *             CHK_HAVEGIL( ipp.ippiCompareC_8u_C1R( <ipp.Ipp8u*>self.im,self.step,
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":458
 *                                                   <ipp.Ipp8u*>dest.im,dest.step,
 *                                                   size.sz,<ipp.IppCmpOp>op))
 *         elif type(other)==FastImage8u:             # <<<<<<<<<<<<<<
 *             other8u = other
 *             CHK_HAVEGIL( ipp.ippiCompare_8u_C1R( <ipp.Ipp8u*>self.im,self.step,
 */
  __pyx_t_1 = PyObject_RichCompare(((PyObject *)Py_TYPE(__pyx_v_other)), ((PyObject *)__pyx_ptype_9FastImage_FastImage8u), Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 458, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "FastImage.pyx":459
 *                                                   size.sz,<ipp.IppCmpOp>op))
 *         elif type(other)==FastImage8u:
 *             other8u = other             # <<<<<<<<<<<<<<
 *             CHK_HAVEGIL( ipp.ippiCompare_8u_C1R( <ipp.Ipp8u*>self.im,self.step,
 *                                                  <ipp.Ipp8u*>other8u.im,other8u.step,
 */
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_9FastImage_FastImage8u))))) __PYX_ERR(0, 459, __pyx_L1_error)
    __pyx_t_1 = __pyx_v_other;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_other8u = ((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "FastImage.pyx":460
 *         elif type(other)==FastImage8u:
 *             other8u = other
 *             CHK_HAVEGIL( ipp.ippiCompare_8u_C1R( <ipp.Ipp8u*>self.im,self.step,             # <<<<<<<<<<<<<<
 *                                                  <ipp.Ipp8u*>other8u.im,other8u.step,
 *                                                  <ipp.Ipp8u*>dest.im,dest.step,
 */
    __pyx_f_9FastImage_CHK_HAVEGIL(ippiCompare_8u_C1R(((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp8u *)__pyx_v_other8u->__pyx_base.im), __pyx_v_other8u->__pyx_base.step, ((Ipp8u *)__pyx_v_dest->__pyx_base.im), __pyx_v_dest->__pyx_base.step, __pyx_v_size->sz, ((IppCmpOp)__pyx_v_op))); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 460, __pyx_L1_error)

    /* "FastImage.pyx":458
 *                                                   <ipp.Ipp8u*>dest.im,dest.step,
 *                                                   size.sz,<ipp.IppCmpOp>op))
 *         elif type(other)==FastImage8u:             # <<<<<<<<<<<<<<
 *             other8u = other
 *             CHK_HAVEGIL( ipp.ippiCompare_8u_C1R( <ipp.Ipp8u*>self.im,self.step,
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":465
 *                                                  size.sz,<ipp.IppCmpOp>op))
 *         else:
 *             raise TypeError("can only compare against ints and FastImage8u types")             # <<<<<<<<<<<<<<
 * 
 *     cdef void fast_get_compare_int_put_greater( self, int other_int, FastImage8u dest, Size size):
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_TypeError, __pyx_tuple__15, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 465, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 465, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "FastImage.pyx":449
 *         return result
 * 
 *     def get_compare_put( self, other, FastImage8u dest, Size size, int op ):             # <<<<<<<<<<<<<<
 *         cdef FastImage8u other8u
 *         cdef int other_int
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("FastImage.FastImage8u.get_compare_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_other8u);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":467
 *             raise TypeError("can only compare against ints and FastImage8u types")
 * 
 *     cdef void fast_get_compare_int_put_greater( self, int other_int, FastImage8u dest, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiCompareC_8u_C1R( <ipp.Ipp8u*>self.im,self.step,
 *                                             other_int,
 */

static void __pyx_f_9FastImage_11FastImage8u_fast_get_compare_int_put_greater(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, int __pyx_v_other_int, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_dest, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fast_get_compare_int_put_greater", 0);

  /* "FastImage.pyx":468
 * 
 *     cdef void fast_get_compare_int_put_greater( self, int other_int, FastImage8u dest, Size size):
 *         CHK_HAVEGIL( ipp.ippiCompareC_8u_C1R( <ipp.Ipp8u*>self.im,self.step,             # <<<<<<<<<<<<<<
 *                                             other_int,
 *                                             <ipp.Ipp8u*>dest.im,dest.step,
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiCompareC_8u_C1R(((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_other_int, ((Ipp8u *)__pyx_v_dest->__pyx_base.im), __pyx_v_dest->__pyx_base.step, __pyx_v_size->sz, ippCmpGreater)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 468, __pyx_L1_error)

  /* "FastImage.pyx":467
 *             raise TypeError("can only compare against ints and FastImage8u types")
 * 
 *     cdef void fast_get_compare_int_put_greater( self, int other_int, FastImage8u dest, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiCompareC_8u_C1R( <ipp.Ipp8u*>self.im,self.step,
 *                                             other_int,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("FastImage.FastImage8u.fast_get_compare_int_put_greater", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":474
 *                                             ipp.ippCmpGreater))
 * 
 *     def min_index(self, Size size):             # <<<<<<<<<<<<<<
 *         cdef ipp.IppStatus sts
 *         cdef int index_x, index_y
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_27min_index(PyObject *__pyx_v_self, PyObject *__pyx_v_size); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_27min_index(PyObject *__pyx_v_self, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("min_index (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 474, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_26min_index(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), ((struct __pyx_obj_9FastImage_Size *)__pyx_v_size));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_26min_index(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  IppStatus __pyx_v_sts;
  int __pyx_v_index_x;
  int __pyx_v_index_y;
  Ipp8u __pyx_v_min_val;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  __Pyx_RefNannySetupContext("min_index", 0);

  /* "FastImage.pyx":479
 *         cdef ipp.Ipp8u min_val
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             sts = ipp.ippiMinIndx_8u_C1R(
 *                 <ipp.Ipp8u*>self.im,self.step,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "FastImage.pyx":480
 * 
 *         with nogil:
 *             sts = ipp.ippiMinIndx_8u_C1R(             # <<<<<<<<<<<<<<
 *                 <ipp.Ipp8u*>self.im,self.step,
 *                  size.sz, &min_val, &index_x, &index_y)
 */
        __pyx_v_sts = ippiMinIndx_8u_C1R(((Ipp8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz, (&__pyx_v_min_val), (&__pyx_v_index_x), (&__pyx_v_index_y));
      }

      /* "FastImage.pyx":479
 *         cdef ipp.Ipp8u min_val
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             sts = ipp.ippiMinIndx_8u_C1R(
 *                 <ipp.Ipp8u*>self.im,self.step,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "FastImage.pyx":483
 *                 <ipp.Ipp8u*>self.im,self.step,
 *                  size.sz, &min_val, &index_x, &index_y)
 *         CHK_HAVEGIL(sts)             # <<<<<<<<<<<<<<
 *         return min_val, index_x, index_y
 * 
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(__pyx_v_sts); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 483, __pyx_L1_error)

  /* "FastImage.pyx":484
 *                  size.sz, &min_val, &index_x, &index_y)
 *         CHK_HAVEGIL(sts)
 *         return min_val, index_x, index_y             # <<<<<<<<<<<<<<
 * 
 *     def max_index(self, Size size):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_Ipp8u(__pyx_v_min_val); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_index_x); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_index_y); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 484, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":474
 *                                             ipp.ippCmpGreater))
 * 
 *     def min_index(self, Size size):             # <<<<<<<<<<<<<<
 *         cdef ipp.IppStatus sts
 *         cdef int index_x, index_y
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("FastImage.FastImage8u.min_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":486
 *         return min_val, index_x, index_y
 * 
 *     def max_index(self, Size size):             # <<<<<<<<<<<<<<
 *         cdef fic.FicStatus sts
 *         cdef int index_x, index_y
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_29max_index(PyObject *__pyx_v_self, PyObject *__pyx_v_size); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_29max_index(PyObject *__pyx_v_self, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("max_index (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 486, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_28max_index(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), ((struct __pyx_obj_9FastImage_Size *)__pyx_v_size));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_28max_index(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  FicStatus __pyx_v_sts;
  int __pyx_v_index_x;
  int __pyx_v_index_y;
  Ipp8u __pyx_v_max_val;
  FiciSize __pyx_v_sz;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("max_index", 0);

  /* "FastImage.pyx":492
 *         cdef fic.FiciSize sz
 * 
 *         sz.width = size.sz.width             # <<<<<<<<<<<<<<
 *         sz.height = size.sz.height
 *         with nogil:
 */
  __pyx_t_1 = __pyx_v_size->sz.width;
  __pyx_v_sz.width = __pyx_t_1;

  /* "FastImage.pyx":493
 * 
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height             # <<<<<<<<<<<<<<
 *         with nogil:
 *             sts = fic.ficiMaxIndx_8u_C1R(
 */
  __pyx_t_1 = __pyx_v_size->sz.height;
  __pyx_v_sz.height = __pyx_t_1;

  /* "FastImage.pyx":494
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         with nogil:             # <<<<<<<<<<<<<<
 *             sts = fic.ficiMaxIndx_8u_C1R(
 *                 <fic.Fic8u*>self.im,self.step,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "FastImage.pyx":495
 *         sz.height = size.sz.height
 *         with nogil:
 *             sts = fic.ficiMaxIndx_8u_C1R(             # <<<<<<<<<<<<<<
 *                 <fic.Fic8u*>self.im,self.step,
 *                  sz, &max_val, &index_x, &index_y)
 */
        __pyx_v_sts = ficiMaxIndx_8u_C1R(((Fic8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_sz, (&__pyx_v_max_val), (&__pyx_v_index_x), (&__pyx_v_index_y));
      }

      /* "FastImage.pyx":494
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         with nogil:             # <<<<<<<<<<<<<<
 *             sts = fic.ficiMaxIndx_8u_C1R(
 *                 <fic.Fic8u*>self.im,self.step,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "FastImage.pyx":498
 *                 <fic.Fic8u*>self.im,self.step,
 *                  sz, &max_val, &index_x, &index_y)
 *         CHK_FIC_HAVEGIL(sts)             # <<<<<<<<<<<<<<
 *         return max_val, index_x, index_y
 * 
 */
  __pyx_f_9FastImage_CHK_FIC_HAVEGIL(__pyx_v_sts); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 498, __pyx_L1_error)

  /* "FastImage.pyx":499
 *                  sz, &max_val, &index_x, &index_y)
 *         CHK_FIC_HAVEGIL(sts)
 *         return max_val, index_x, index_y             # <<<<<<<<<<<<<<
 * 
 *     def to_yuv422(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyInt_From_Ipp8u(__pyx_v_max_val); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_index_x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_index_y); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 499, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_4);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":486
 *         return min_val, index_x, index_y
 * 
 *     def max_index(self, Size size):             # <<<<<<<<<<<<<<
 *         cdef fic.FicStatus sts
 *         cdef int index_x, index_y
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("FastImage.FastImage8u.max_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":501
 *         return max_val, index_x, index_y
 * 
 *     def to_yuv422(self):             # <<<<<<<<<<<<<<
 *         cdef FastImage8u y, u, v
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_31to_yuv422(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_31to_yuv422(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("to_yuv422 (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_30to_yuv422(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_30to_yuv422(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self) {
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_y = 0;
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_u = 0;
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_v = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  __Pyx_RefNannySetupContext("to_yuv422", 0);

  /* "FastImage.pyx":504
 *         cdef FastImage8u y, u, v
 * 
 *         y = self.get_8u_copy(self.imsiz)             # <<<<<<<<<<<<<<
 *         u = asfastimage(128*numpy.ones_like(y))
 *         v = u
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_8u_copy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 504, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_self->__pyx_base.imsiz)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 504, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_self->__pyx_base.imsiz)};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 504, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_self->__pyx_base.imsiz)};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 504, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 504, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.imsiz));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.imsiz));
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_self->__pyx_base.imsiz));
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 504, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9FastImage_FastImage8u))))) __PYX_ERR(0, 504, __pyx_L1_error)
  __pyx_v_y = ((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "FastImage.pyx":505
 * 
 *         y = self.get_8u_copy(self.imsiz)
 *         u = asfastimage(128*numpy.ones_like(y))             # <<<<<<<<<<<<<<
 *         v = u
 *         return y,u,v
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_asfastimage); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_GetModuleGlobalName(__pyx_n_s_numpy); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_3, __pyx_n_s_ones_like); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_t_5, ((PyObject *)__pyx_v_y)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 505, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_y)};
      __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 505, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_y)};
      __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 505, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_4);
    } else
    #endif
    {
      __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 505, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_y));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_y));
      PyTuple_SET_ITEM(__pyx_t_6, 0+1, ((PyObject *)__pyx_v_y));
      __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_6, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 505, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_5 = PyNumber_Multiply(__pyx_int_128, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 505, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_4) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 505, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_5};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 505, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_4, __pyx_t_5};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 505, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else
    #endif
    {
      __pyx_t_6 = PyTuple_New(1+1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 505, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_6, 0+1, __pyx_t_5);
      __pyx_t_5 = 0;
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 505, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9FastImage_FastImage8u))))) __PYX_ERR(0, 505, __pyx_L1_error)
  __pyx_v_u = ((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "FastImage.pyx":506
 *         y = self.get_8u_copy(self.imsiz)
 *         u = asfastimage(128*numpy.ones_like(y))
 *         v = u             # <<<<<<<<<<<<<<
 *         return y,u,v
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_u));
  __pyx_v_v = __pyx_v_u;

  /* "FastImage.pyx":507
 *         u = asfastimage(128*numpy.ones_like(y))
 *         v = u
 *         return y,u,v             # <<<<<<<<<<<<<<
 * 
 *     def dot(self,FastImage8u other,Size size):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 507, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_y));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_y));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_y));
  __Pyx_INCREF(((PyObject *)__pyx_v_u));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_u));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_u));
  __Pyx_INCREF(((PyObject *)__pyx_v_v));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_v));
  PyTuple_SET_ITEM(__pyx_t_1, 2, ((PyObject *)__pyx_v_v));
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":501
 *         return max_val, index_x, index_y
 * 
 *     def to_yuv422(self):             # <<<<<<<<<<<<<<
 *         cdef FastImage8u y, u, v
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("FastImage.FastImage8u.to_yuv422", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_y);
  __Pyx_XDECREF((PyObject *)__pyx_v_u);
  __Pyx_XDECREF((PyObject *)__pyx_v_v);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":509
 *         return y,u,v
 * 
 *     def dot(self,FastImage8u other,Size size):             # <<<<<<<<<<<<<<
 *         cdef fic.Fic64f result
 *         cdef fic.FiciSize sz
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_33dot(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_33dot(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("dot (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("dot", 1, 2, 2, 1); __PYX_ERR(0, 509, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "dot") < 0)) __PYX_ERR(0, 509, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_other = ((struct __pyx_obj_9FastImage_FastImage8u *)values[0]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("dot", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 509, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.dot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage8u, 1, "other", 0))) __PYX_ERR(0, 509, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 509, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_32dot(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_other, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_32dot(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  Fic64f __pyx_v_result;
  FiciSize __pyx_v_sz;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("dot", 0);

  /* "FastImage.pyx":513
 *         cdef fic.FiciSize sz
 * 
 *         sz.width = size.sz.width             # <<<<<<<<<<<<<<
 *         sz.height = size.sz.height
 *         CHK_FIC_HAVEGIL( fic.ficiDotProd_8u64f_C1R(
 */
  __pyx_t_1 = __pyx_v_size->sz.width;
  __pyx_v_sz.width = __pyx_t_1;

  /* "FastImage.pyx":514
 * 
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height             # <<<<<<<<<<<<<<
 *         CHK_FIC_HAVEGIL( fic.ficiDotProd_8u64f_C1R(
 *             <fic.Fic8u*>self.im,self.step,
 */
  __pyx_t_1 = __pyx_v_size->sz.height;
  __pyx_v_sz.height = __pyx_t_1;

  /* "FastImage.pyx":515
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         CHK_FIC_HAVEGIL( fic.ficiDotProd_8u64f_C1R(             # <<<<<<<<<<<<<<
 *             <fic.Fic8u*>self.im,self.step,
 *             <fic.Fic8u*>other.im,other.step,
 */
  __pyx_f_9FastImage_CHK_FIC_HAVEGIL(ficiDotProd_8u64f_C1R(((Fic8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Fic8u *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, __pyx_v_sz, (&__pyx_v_result))); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 515, __pyx_L1_error)

  /* "FastImage.pyx":519
 *             <fic.Fic8u*>other.im,other.step,
 *             sz, &result ))
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def sobel_horiz(self, Size size, FastImage8u dest=None):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 519, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":509
 *         return y,u,v
 * 
 *     def dot(self,FastImage8u other,Size size):             # <<<<<<<<<<<<<<
 *         cdef fic.Fic64f result
 *         cdef fic.FiciSize sz
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("FastImage.FastImage8u.dot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":521
 *         return result
 * 
 *     def sobel_horiz(self, Size size, FastImage8u dest=None):             # <<<<<<<<<<<<<<
 *         cdef fic.FiciSize sz
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_35sobel_horiz(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_35sobel_horiz(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_dest = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("sobel_horiz (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_size,&__pyx_n_s_dest,0};
    PyObject* values[2] = {0,0};
    values[1] = (PyObject *)((struct __pyx_obj_9FastImage_FastImage8u *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dest);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "sobel_horiz") < 0)) __PYX_ERR(0, 521, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[0]);
    __pyx_v_dest = ((struct __pyx_obj_9FastImage_FastImage8u *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("sobel_horiz", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 521, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.sobel_horiz", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 521, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dest), __pyx_ptype_9FastImage_FastImage8u, 1, "dest", 0))) __PYX_ERR(0, 521, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_34sobel_horiz(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_size, __pyx_v_dest);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_34sobel_horiz(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_dest) {
  FiciSize __pyx_v_sz;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("sobel_horiz", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_dest);

  /* "FastImage.pyx":524
 *         cdef fic.FiciSize sz
 * 
 *         sz.width = size.sz.width             # <<<<<<<<<<<<<<
 *         sz.height = size.sz.height
 *         if dest is None:
 */
  __pyx_t_1 = __pyx_v_size->sz.width;
  __pyx_v_sz.width = __pyx_t_1;

  /* "FastImage.pyx":525
 * 
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height             # <<<<<<<<<<<<<<
 *         if dest is None:
 *             dest = FastImage8u( size )
 */
  __pyx_t_1 = __pyx_v_size->sz.height;
  __pyx_v_sz.height = __pyx_t_1;

  /* "FastImage.pyx":526
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         if dest is None:             # <<<<<<<<<<<<<<
 *             dest = FastImage8u( size )
 *         else:
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_dest) == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "FastImage.pyx":527
 *         sz.height = size.sz.height
 *         if dest is None:
 *             dest = FastImage8u( size )             # <<<<<<<<<<<<<<
 *         else:
 *             assert dest.size == size
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 527, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)__pyx_v_size));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)__pyx_v_size));
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_FastImage8u), __pyx_t_4, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 527, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_dest, ((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "FastImage.pyx":526
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         if dest is None:             # <<<<<<<<<<<<<<
 *             dest = FastImage8u( size )
 *         else:
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":529
 *             dest = FastImage8u( size )
 *         else:
 *             assert dest.size == size             # <<<<<<<<<<<<<<
 *         CHK_FIC_HAVEGIL( fic.ficiFilterSobelHoriz_8u_C1R(<fic.Fic8u*>self.im, self.step, <fic.Fic8u*>dest.im, dest.step, sz ))
 *         return dest
 */
  /*else*/ {
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_dest), __pyx_n_s_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 529, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_5, ((PyObject *)__pyx_v_size), Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 529, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 529, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 529, __pyx_L1_error)
      }
    }
    #endif
  }
  __pyx_L3:;

  /* "FastImage.pyx":530
 *         else:
 *             assert dest.size == size
 *         CHK_FIC_HAVEGIL( fic.ficiFilterSobelHoriz_8u_C1R(<fic.Fic8u*>self.im, self.step, <fic.Fic8u*>dest.im, dest.step, sz ))             # <<<<<<<<<<<<<<
 *         return dest
 * 
 */
  __pyx_f_9FastImage_CHK_FIC_HAVEGIL(ficiFilterSobelHoriz_8u_C1R(((Fic8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Fic8u *)__pyx_v_dest->__pyx_base.im), __pyx_v_dest->__pyx_base.step, __pyx_v_sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 530, __pyx_L1_error)

  /* "FastImage.pyx":531
 *             assert dest.size == size
 *         CHK_FIC_HAVEGIL( fic.ficiFilterSobelHoriz_8u_C1R(<fic.Fic8u*>self.im, self.step, <fic.Fic8u*>dest.im, dest.step, sz ))
 *         return dest             # <<<<<<<<<<<<<<
 * 
 *     def sobel_vert(self, Size size, FastImage8u dest=None):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_dest));
  __pyx_r = ((PyObject *)__pyx_v_dest);
  goto __pyx_L0;

  /* "FastImage.pyx":521
 *         return result
 * 
 *     def sobel_horiz(self, Size size, FastImage8u dest=None):             # <<<<<<<<<<<<<<
 *         cdef fic.FiciSize sz
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("FastImage.FastImage8u.sobel_horiz", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_dest);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":533
 *         return dest
 * 
 *     def sobel_vert(self, Size size, FastImage8u dest=None):             # <<<<<<<<<<<<<<
 *         cdef fic.FiciSize sz
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11FastImage8u_37sobel_vert(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_11FastImage8u_37sobel_vert(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_dest = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("sobel_vert (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_size,&__pyx_n_s_dest,0};
    PyObject* values[2] = {0,0};
    values[1] = (PyObject *)((struct __pyx_obj_9FastImage_FastImage8u *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dest);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "sobel_vert") < 0)) __PYX_ERR(0, 533, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[0]);
    __pyx_v_dest = ((struct __pyx_obj_9FastImage_FastImage8u *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("sobel_vert", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 533, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage8u.sobel_vert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 533, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dest), __pyx_ptype_9FastImage_FastImage8u, 1, "dest", 0))) __PYX_ERR(0, 533, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_11FastImage8u_36sobel_vert(((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_v_self), __pyx_v_size, __pyx_v_dest);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_11FastImage8u_36sobel_vert(struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_dest) {
  FiciSize __pyx_v_sz;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("sobel_vert", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_dest);

  /* "FastImage.pyx":536
 *         cdef fic.FiciSize sz
 * 
 *         sz.width = size.sz.width             # <<<<<<<<<<<<<<
 *         sz.height = size.sz.height
 *         if dest is None:
 */
  __pyx_t_1 = __pyx_v_size->sz.width;
  __pyx_v_sz.width = __pyx_t_1;

  /* "FastImage.pyx":537
 * 
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height             # <<<<<<<<<<<<<<
 *         if dest is None:
 *             dest = FastImage8u( size )
 */
  __pyx_t_1 = __pyx_v_size->sz.height;
  __pyx_v_sz.height = __pyx_t_1;

  /* "FastImage.pyx":538
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         if dest is None:             # <<<<<<<<<<<<<<
 *             dest = FastImage8u( size )
 *         else:
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_dest) == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "FastImage.pyx":539
 *         sz.height = size.sz.height
 *         if dest is None:
 *             dest = FastImage8u( size )             # <<<<<<<<<<<<<<
 *         else:
 *             assert dest.size == size
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 539, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)__pyx_v_size));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)__pyx_v_size));
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_FastImage8u), __pyx_t_4, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 539, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_dest, ((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_t_5));
    __pyx_t_5 = 0;

    /* "FastImage.pyx":538
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         if dest is None:             # <<<<<<<<<<<<<<
 *             dest = FastImage8u( size )
 *         else:
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":541
 *             dest = FastImage8u( size )
 *         else:
 *             assert dest.size == size             # <<<<<<<<<<<<<<
 *         CHK_FIC_HAVEGIL( fic.ficiFilterSobelVert_8u_C1R(<fic.Fic8u*>self.im, self.step, <fic.Fic8u*>dest.im, dest.step, sz ))
 *         return dest
 */
  /*else*/ {
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_dest), __pyx_n_s_size); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = PyObject_RichCompare(__pyx_t_5, ((PyObject *)__pyx_v_size), Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 541, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 541, __pyx_L1_error)
      }
    }
    #endif
  }
  __pyx_L3:;

  /* "FastImage.pyx":542
 *         else:
 *             assert dest.size == size
 *         CHK_FIC_HAVEGIL( fic.ficiFilterSobelVert_8u_C1R(<fic.Fic8u*>self.im, self.step, <fic.Fic8u*>dest.im, dest.step, sz ))             # <<<<<<<<<<<<<<
 *         return dest
 * 
 */
  __pyx_f_9FastImage_CHK_FIC_HAVEGIL(ficiFilterSobelVert_8u_C1R(((Fic8u *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Fic8u *)__pyx_v_dest->__pyx_base.im), __pyx_v_dest->__pyx_base.step, __pyx_v_sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 542, __pyx_L1_error)

  /* "FastImage.pyx":543
 *             assert dest.size == size
 *         CHK_FIC_HAVEGIL( fic.ficiFilterSobelVert_8u_C1R(<fic.Fic8u*>self.im, self.step, <fic.Fic8u*>dest.im, dest.step, sz ))
 *         return dest             # <<<<<<<<<<<<<<
 * 
 * cdef class FastImage32f(FastImageBase):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_dest));
  __pyx_r = ((PyObject *)__pyx_v_dest);
  goto __pyx_L0;

  /* "FastImage.pyx":533
 *         return dest
 * 
 *     def sobel_vert(self, Size size, FastImage8u dest=None):             # <<<<<<<<<<<<<<
 *         cdef fic.FiciSize sz
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("FastImage.FastImage8u.sobel_vert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_dest);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":547
 * cdef class FastImage32f(FastImageBase):
 * 
 *     def __cinit__(self,*args,**kw):             # <<<<<<<<<<<<<<
 *         self.strides[1] = 4
 *         self.basetype = '32f'
 */

/* Python wrapper */
static int __pyx_pw_9FastImage_12FastImage32f_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9FastImage_12FastImage32f_1__cinit__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  CYTHON_UNUSED PyObject *__pyx_v_args = 0;
  CYTHON_UNUSED PyObject *__pyx_v_kw = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__ (wrapper)", 0);
  if (unlikely(__pyx_kwds) && unlikely(!__Pyx_CheckKeywordStrings(__pyx_kwds, "__cinit__", 1))) return -1;
  __Pyx_INCREF(__pyx_args);
  __pyx_v_args = __pyx_args;
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f___cinit__(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_args, __pyx_v_kw);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_kw);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9FastImage_12FastImage32f___cinit__(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_args, CYTHON_UNUSED PyObject *__pyx_v_kw) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__cinit__", 0);

  /* "FastImage.pyx":548
 * 
 *     def __cinit__(self,*args,**kw):
 *         self.strides[1] = 4             # <<<<<<<<<<<<<<
 *         self.basetype = '32f'
 * 
 */
  (__pyx_v_self->__pyx_base.strides[1]) = 4;

  /* "FastImage.pyx":549
 *     def __cinit__(self,*args,**kw):
 *         self.strides[1] = 4
 *         self.basetype = '32f'             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, Size size ):
 */
  __Pyx_INCREF(__pyx_kp_s_32f);
  __Pyx_GIVEREF(__pyx_kp_s_32f);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.basetype);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.basetype);
  __pyx_v_self->__pyx_base.basetype = __pyx_kp_s_32f;

  /* "FastImage.pyx":547
 * cdef class FastImage32f(FastImageBase):
 * 
 *     def __cinit__(self,*args,**kw):             # <<<<<<<<<<<<<<
 *         self.strides[1] = 4
 *         self.basetype = '32f'
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":551
 *         self.basetype = '32f'
 * 
 *     def __init__(self, Size size ):             # <<<<<<<<<<<<<<
 *         global FASTIMAGEDEBUG
 * 
 */

/* Python wrapper */
static int __pyx_pw_9FastImage_12FastImage32f_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_9FastImage_12FastImage32f_3__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_size,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 551, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[0]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 551, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 551, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_2__init__(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_9FastImage_12FastImage32f_2__init__(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "FastImage.pyx":554
 *         global FASTIMAGEDEBUG
 * 
 *         FastImageBase.__init__(self, size)             # <<<<<<<<<<<<<<
 *         if FASTIMAGEDEBUG>=1:
 *             print self,'requesting 32f memory (size W:%d H:%d)'%(self.imsiz.sz.width,
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9FastImage_FastImageBase), __pyx_n_s_init); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 554, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_4 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, ((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_size)};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 554, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, ((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_size)};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_4, 2+__pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 554, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_5 = PyTuple_New(2+__pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 554, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_5, 0+__pyx_t_4, ((PyObject *)__pyx_v_self));
    __Pyx_INCREF(((PyObject *)__pyx_v_size));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
    PyTuple_SET_ITEM(__pyx_t_5, 1+__pyx_t_4, ((PyObject *)__pyx_v_size));
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 554, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "FastImage.pyx":555
 * 
 *         FastImageBase.__init__(self, size)
 *         if FASTIMAGEDEBUG>=1:             # <<<<<<<<<<<<<<
 *             print self,'requesting 32f memory (size W:%d H:%d)'%(self.imsiz.sz.width,
 *                                                              self.imsiz.sz.height)
 */
  __pyx_t_6 = ((__pyx_v_9FastImage_FASTIMAGEDEBUG >= 1) != 0);
  if (__pyx_t_6) {

    /* "FastImage.pyx":556
 *         FastImageBase.__init__(self, size)
 *         if FASTIMAGEDEBUG>=1:
 *             print self,'requesting 32f memory (size W:%d H:%d)'%(self.imsiz.sz.width,             # <<<<<<<<<<<<<<
 *                                                              self.imsiz.sz.height)
 *         self.im=ipp.ippiMalloc_32f_C1( size.sz.width, size.sz.height, &self.step )
 */
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.imsiz->sz.width); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 556, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);

    /* "FastImage.pyx":557
 *         if FASTIMAGEDEBUG>=1:
 *             print self,'requesting 32f memory (size W:%d H:%d)'%(self.imsiz.sz.width,
 *                                                              self.imsiz.sz.height)             # <<<<<<<<<<<<<<
 *         self.im=ipp.ippiMalloc_32f_C1( size.sz.width, size.sz.height, &self.step )
 *         if self.im==NULL: raise MemoryError("Error allocating memory")
 */
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_self->__pyx_base.imsiz->sz.height); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 557, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);

    /* "FastImage.pyx":556
 *         FastImageBase.__init__(self, size)
 *         if FASTIMAGEDEBUG>=1:
 *             print self,'requesting 32f memory (size W:%d H:%d)'%(self.imsiz.sz.width,             # <<<<<<<<<<<<<<
 *                                                              self.imsiz.sz.height)
 *         self.im=ipp.ippiMalloc_32f_C1( size.sz.width, size.sz.height, &self.step )
 */
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 556, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_requesting_32f_memory_size_W_d_H, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 556, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 556, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)__pyx_v_self));
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
    __pyx_t_2 = 0;
    if (__Pyx_Print(0, __pyx_t_5, 1) < 0) __PYX_ERR(0, 556, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "FastImage.pyx":555
 * 
 *         FastImageBase.__init__(self, size)
 *         if FASTIMAGEDEBUG>=1:             # <<<<<<<<<<<<<<
 *             print self,'requesting 32f memory (size W:%d H:%d)'%(self.imsiz.sz.width,
 *                                                              self.imsiz.sz.height)
 */
  }

  /* "FastImage.pyx":558
 *             print self,'requesting 32f memory (size W:%d H:%d)'%(self.imsiz.sz.width,
 *                                                              self.imsiz.sz.height)
 *         self.im=ipp.ippiMalloc_32f_C1( size.sz.width, size.sz.height, &self.step )             # <<<<<<<<<<<<<<
 *         if self.im==NULL: raise MemoryError("Error allocating memory")
 *         self.strides[0] = self.step
 */
  __pyx_v_self->__pyx_base.im = ippiMalloc_32f_C1(__pyx_v_size->sz.width, __pyx_v_size->sz.height, (&__pyx_v_self->__pyx_base.step));

  /* "FastImage.pyx":559
 *                                                              self.imsiz.sz.height)
 *         self.im=ipp.ippiMalloc_32f_C1( size.sz.width, size.sz.height, &self.step )
 *         if self.im==NULL: raise MemoryError("Error allocating memory")             # <<<<<<<<<<<<<<
 *         self.strides[0] = self.step
 *         if FASTIMAGEDEBUG>=1:
 */
  __pyx_t_6 = ((__pyx_v_self->__pyx_base.im == NULL) != 0);
  if (__pyx_t_6) {
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_MemoryError, __pyx_tuple__16, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 559, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 559, __pyx_L1_error)
  }

  /* "FastImage.pyx":560
 *         self.im=ipp.ippiMalloc_32f_C1( size.sz.width, size.sz.height, &self.step )
 *         if self.im==NULL: raise MemoryError("Error allocating memory")
 *         self.strides[0] = self.step             # <<<<<<<<<<<<<<
 *         if FASTIMAGEDEBUG>=1:
 *             self.show_mem_and_step()
 */
  __pyx_t_4 = __pyx_v_self->__pyx_base.step;
  (__pyx_v_self->__pyx_base.strides[0]) = __pyx_t_4;

  /* "FastImage.pyx":561
 *         if self.im==NULL: raise MemoryError("Error allocating memory")
 *         self.strides[0] = self.step
 *         if FASTIMAGEDEBUG>=1:             # <<<<<<<<<<<<<<
 *             self.show_mem_and_step()
 *         self.view = 0
 */
  __pyx_t_6 = ((__pyx_v_9FastImage_FASTIMAGEDEBUG >= 1) != 0);
  if (__pyx_t_6) {

    /* "FastImage.pyx":562
 *         self.strides[0] = self.step
 *         if FASTIMAGEDEBUG>=1:
 *             self.show_mem_and_step()             # <<<<<<<<<<<<<<
 *         self.view = 0
 * 
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_show_mem_and_step); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 562, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (__pyx_t_1) {
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 562, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    } else {
      __pyx_t_5 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 562, __pyx_L1_error)
    }
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "FastImage.pyx":561
 *         if self.im==NULL: raise MemoryError("Error allocating memory")
 *         self.strides[0] = self.step
 *         if FASTIMAGEDEBUG>=1:             # <<<<<<<<<<<<<<
 *             self.show_mem_and_step()
 *         self.view = 0
 */
  }

  /* "FastImage.pyx":563
 *         if FASTIMAGEDEBUG>=1:
 *             self.show_mem_and_step()
 *         self.view = 0             # <<<<<<<<<<<<<<
 * 
 *     def set_val(self, float val, Size size):
 */
  __pyx_v_self->__pyx_base.view = 0;

  /* "FastImage.pyx":551
 *         self.basetype = '32f'
 * 
 *     def __init__(self, Size size ):             # <<<<<<<<<<<<<<
 *         global FASTIMAGEDEBUG
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("FastImage.FastImage32f.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":565
 *         self.view = 0
 * 
 *     def set_val(self, float val, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSet_32f_C1R( val, <ipp.Ipp32f*>self.im, self.step, size.sz ))
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_5set_val(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_12FastImage32f_5set_val(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  float __pyx_v_val;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_val (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_val,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_val)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("set_val", 1, 2, 2, 1); __PYX_ERR(0, 565, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "set_val") < 0)) __PYX_ERR(0, 565, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_val = __pyx_PyFloat_AsFloat(values[0]); if (unlikely((__pyx_v_val == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 565, __pyx_L3_error)
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("set_val", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 565, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.set_val", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 565, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_4set_val(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_val, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_4set_val(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, float __pyx_v_val, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("set_val", 0);

  /* "FastImage.pyx":566
 * 
 *     def set_val(self, float val, Size size):
 *         CHK_HAVEGIL( ipp.ippiSet_32f_C1R( val, <ipp.Ipp32f*>self.im, self.step, size.sz ))             # <<<<<<<<<<<<<<
 * 
 *     def toself_add(self, FastImageBase other, Size size):
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSet_32f_C1R(__pyx_v_val, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 566, __pyx_L1_error)

  /* "FastImage.pyx":565
 *         self.view = 0
 * 
 *     def set_val(self, float val, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSet_32f_C1R( val, <ipp.Ipp32f*>self.im, self.step, size.sz ))
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.set_val", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":568
 *         CHK_HAVEGIL( ipp.ippiSet_32f_C1R( val, <ipp.Ipp32f*>self.im, self.step, size.sz ))
 * 
 *     def toself_add(self, FastImageBase other, Size size):             # <<<<<<<<<<<<<<
 *         """self += other"""
 *         if isinstance(other,FastImage8u):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_7toself_add(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_6toself_add[] = "self += other";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_7toself_add(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toself_add (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("toself_add", 1, 2, 2, 1); __PYX_ERR(0, 568, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "toself_add") < 0)) __PYX_ERR(0, 568, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_other = ((struct __pyx_obj_9FastImage_FastImageBase *)values[0]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("toself_add", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 568, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.toself_add", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImageBase, 1, "other", 0))) __PYX_ERR(0, 568, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 568, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_6toself_add(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_other, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_6toself_add(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("toself_add", 0);

  /* "FastImage.pyx":570
 *     def toself_add(self, FastImageBase other, Size size):
 *         """self += other"""
 *         if isinstance(other,FastImage8u):             # <<<<<<<<<<<<<<
 *             CHK_HAVEGIL( ipp.ippiAdd_8u32f_C1IR( <ipp.Ipp8u*>other.im, other.step,
 *                                                  <ipp.Ipp32f*>self.im, self.step,
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage8u); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":571
 *         """self += other"""
 *         if isinstance(other,FastImage8u):
 *             CHK_HAVEGIL( ipp.ippiAdd_8u32f_C1IR( <ipp.Ipp8u*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                                  <ipp.Ipp32f*>self.im, self.step,
 *                                                  size.sz))
 */
    __pyx_f_9FastImage_CHK_HAVEGIL(ippiAdd_8u32f_C1IR(((Ipp8u *)__pyx_v_other->im), __pyx_v_other->step, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 571, __pyx_L1_error)

    /* "FastImage.pyx":570
 *     def toself_add(self, FastImageBase other, Size size):
 *         """self += other"""
 *         if isinstance(other,FastImage8u):             # <<<<<<<<<<<<<<
 *             CHK_HAVEGIL( ipp.ippiAdd_8u32f_C1IR( <ipp.Ipp8u*>other.im, other.step,
 *                                                  <ipp.Ipp32f*>self.im, self.step,
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":574
 *                                                  <ipp.Ipp32f*>self.im, self.step,
 *                                                  size.sz))
 *         elif isinstance(other,FastImage32f):             # <<<<<<<<<<<<<<
 *             CHK_HAVEGIL( ipp.ippiAdd_32f_C1IR( <ipp.Ipp32f*>other.im, other.step,
 *                                                <ipp.Ipp32f*>self.im, self.step,
 */
  __pyx_t_2 = __Pyx_TypeCheck(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage32f); 
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "FastImage.pyx":575
 *                                                  size.sz))
 *         elif isinstance(other,FastImage32f):
 *             CHK_HAVEGIL( ipp.ippiAdd_32f_C1IR( <ipp.Ipp32f*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                                <ipp.Ipp32f*>self.im, self.step,
 *                                                size.sz))
 */
    __pyx_f_9FastImage_CHK_HAVEGIL(ippiAdd_32f_C1IR(((Ipp32f *)__pyx_v_other->im), __pyx_v_other->step, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 575, __pyx_L1_error)

    /* "FastImage.pyx":574
 *                                                  <ipp.Ipp32f*>self.im, self.step,
 *                                                  size.sz))
 *         elif isinstance(other,FastImage32f):             # <<<<<<<<<<<<<<
 *             CHK_HAVEGIL( ipp.ippiAdd_32f_C1IR( <ipp.Ipp32f*>other.im, other.step,
 *                                                <ipp.Ipp32f*>self.im, self.step,
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":579
 *                                                size.sz))
 *         else:
 *             raise ValueError("type not supported")             # <<<<<<<<<<<<<<
 * 
 *     def toself_add_weighted(self, FastImageBase other, Size size, float alpha):
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__17, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 579, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 579, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "FastImage.pyx":568
 *         CHK_HAVEGIL( ipp.ippiSet_32f_C1R( val, <ipp.Ipp32f*>self.im, self.step, size.sz ))
 * 
 *     def toself_add(self, FastImageBase other, Size size):             # <<<<<<<<<<<<<<
 *         """self += other"""
 *         if isinstance(other,FastImage8u):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("FastImage.FastImage32f.toself_add", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":581
 *             raise ValueError("type not supported")
 * 
 *     def toself_add_weighted(self, FastImageBase other, Size size, float alpha):             # <<<<<<<<<<<<<<
 *         """self = self*(1-alpha) + alpha*other"""
 *         if isinstance(other,FastImage8u):
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_9toself_add_weighted(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_8toself_add_weighted[] = "self = self*(1-alpha) + alpha*other";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_9toself_add_weighted(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  float __pyx_v_alpha;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toself_add_weighted (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_size,&__pyx_n_s_alpha,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("toself_add_weighted", 1, 3, 3, 1); __PYX_ERR(0, 581, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_alpha)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("toself_add_weighted", 1, 3, 3, 2); __PYX_ERR(0, 581, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "toself_add_weighted") < 0)) __PYX_ERR(0, 581, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_other = ((struct __pyx_obj_9FastImage_FastImageBase *)values[0]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
    __pyx_v_alpha = __pyx_PyFloat_AsFloat(values[2]); if (unlikely((__pyx_v_alpha == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 581, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("toself_add_weighted", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 581, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.toself_add_weighted", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImageBase, 1, "other", 0))) __PYX_ERR(0, 581, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 581, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_8toself_add_weighted(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_other, __pyx_v_size, __pyx_v_alpha);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_8toself_add_weighted(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size, float __pyx_v_alpha) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  __Pyx_RefNannySetupContext("toself_add_weighted", 0);

  /* "FastImage.pyx":583
 *     def toself_add_weighted(self, FastImageBase other, Size size, float alpha):
 *         """self = self*(1-alpha) + alpha*other"""
 *         if isinstance(other,FastImage8u):             # <<<<<<<<<<<<<<
 *             CHK_HAVEGIL( ipp.ippiAddWeighted_8u32f_C1IR( <ipp.Ipp8u*>other.im, other.step,
 *                                                          <ipp.Ipp32f*>self.im, self.step,
 */
  __pyx_t_1 = __Pyx_TypeCheck(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage8u); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":584
 *         """self = self*(1-alpha) + alpha*other"""
 *         if isinstance(other,FastImage8u):
 *             CHK_HAVEGIL( ipp.ippiAddWeighted_8u32f_C1IR( <ipp.Ipp8u*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                                          <ipp.Ipp32f*>self.im, self.step,
 *                                                          size.sz, alpha))
 */
    __pyx_f_9FastImage_CHK_HAVEGIL(ippiAddWeighted_8u32f_C1IR(((Ipp8u *)__pyx_v_other->im), __pyx_v_other->step, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz, __pyx_v_alpha)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 584, __pyx_L1_error)

    /* "FastImage.pyx":583
 *     def toself_add_weighted(self, FastImageBase other, Size size, float alpha):
 *         """self = self*(1-alpha) + alpha*other"""
 *         if isinstance(other,FastImage8u):             # <<<<<<<<<<<<<<
 *             CHK_HAVEGIL( ipp.ippiAddWeighted_8u32f_C1IR( <ipp.Ipp8u*>other.im, other.step,
 *                                                          <ipp.Ipp32f*>self.im, self.step,
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":587
 *                                                          <ipp.Ipp32f*>self.im, self.step,
 *                                                          size.sz, alpha))
 *         elif isinstance(other,FastImage32f):             # <<<<<<<<<<<<<<
 *             CHK_HAVEGIL( ipp.ippiAddWeighted_32f_C1IR( <ipp.Ipp32f*>other.im, other.step,
 *                                                        <ipp.Ipp32f*>self.im, self.step,
 */
  __pyx_t_2 = __Pyx_TypeCheck(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage32f); 
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "FastImage.pyx":588
 *                                                          size.sz, alpha))
 *         elif isinstance(other,FastImage32f):
 *             CHK_HAVEGIL( ipp.ippiAddWeighted_32f_C1IR( <ipp.Ipp32f*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                                        <ipp.Ipp32f*>self.im, self.step,
 *                                                        size.sz, alpha))
 */
    __pyx_f_9FastImage_CHK_HAVEGIL(ippiAddWeighted_32f_C1IR(((Ipp32f *)__pyx_v_other->im), __pyx_v_other->step, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz, __pyx_v_alpha)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 588, __pyx_L1_error)

    /* "FastImage.pyx":587
 *                                                          <ipp.Ipp32f*>self.im, self.step,
 *                                                          size.sz, alpha))
 *         elif isinstance(other,FastImage32f):             # <<<<<<<<<<<<<<
 *             CHK_HAVEGIL( ipp.ippiAddWeighted_32f_C1IR( <ipp.Ipp32f*>other.im, other.step,
 *                                                        <ipp.Ipp32f*>self.im, self.step,
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":592
 *                                                        size.sz, alpha))
 *         else:
 *             raise ValueError("type not supported")             # <<<<<<<<<<<<<<
 * 
 *     cdef void fast_toself_add_weighted_8u( self, FastImage8u other, Size size, float alpha):
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__18, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 592, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 592, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "FastImage.pyx":581
 *             raise ValueError("type not supported")
 * 
 *     def toself_add_weighted(self, FastImageBase other, Size size, float alpha):             # <<<<<<<<<<<<<<
 *         """self = self*(1-alpha) + alpha*other"""
 *         if isinstance(other,FastImage8u):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("FastImage.FastImage32f.toself_add_weighted", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":594
 *             raise ValueError("type not supported")
 * 
 *     cdef void fast_toself_add_weighted_8u( self, FastImage8u other, Size size, float alpha):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiAddWeighted_8u32f_C1IR( <ipp.Ipp8u*>other.im, other.step,
 *                                                    <ipp.Ipp32f*>self.im, self.step,
 */

static void __pyx_f_9FastImage_12FastImage32f_fast_toself_add_weighted_8u(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size, float __pyx_v_alpha) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fast_toself_add_weighted_8u", 0);

  /* "FastImage.pyx":595
 * 
 *     cdef void fast_toself_add_weighted_8u( self, FastImage8u other, Size size, float alpha):
 *         CHK_HAVEGIL( ipp.ippiAddWeighted_8u32f_C1IR( <ipp.Ipp8u*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                                    <ipp.Ipp32f*>self.im, self.step,
 *                                                    size.sz, alpha))
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiAddWeighted_8u32f_C1IR(((Ipp8u *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz, __pyx_v_alpha)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 595, __pyx_L1_error)

  /* "FastImage.pyx":594
 *             raise ValueError("type not supported")
 * 
 *     cdef void fast_toself_add_weighted_8u( self, FastImage8u other, Size size, float alpha):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiAddWeighted_8u32f_C1IR( <ipp.Ipp8u*>other.im, other.step,
 *                                                    <ipp.Ipp32f*>self.im, self.step,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("FastImage.FastImage32f.fast_toself_add_weighted_8u", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":599
 *                                                    size.sz, alpha))
 * 
 *     cdef void fast_toself_add_weighted_32f(self, FastImage32f other, Size size, float alpha):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiAddWeighted_32f_C1IR( <ipp.Ipp32f*>other.im, other.step,
 *                                                    <ipp.Ipp32f*>self.im, self.step,
 */

static void __pyx_f_9FastImage_12FastImage32f_fast_toself_add_weighted_32f(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size, float __pyx_v_alpha) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fast_toself_add_weighted_32f", 0);

  /* "FastImage.pyx":600
 * 
 *     cdef void fast_toself_add_weighted_32f(self, FastImage32f other, Size size, float alpha):
 *         CHK_HAVEGIL( ipp.ippiAddWeighted_32f_C1IR( <ipp.Ipp32f*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                                    <ipp.Ipp32f*>self.im, self.step,
 *                                                    size.sz, alpha))
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiAddWeighted_32f_C1IR(((Ipp32f *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz, __pyx_v_alpha)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 600, __pyx_L1_error)

  /* "FastImage.pyx":599
 *                                                    size.sz, alpha))
 * 
 *     cdef void fast_toself_add_weighted_32f(self, FastImage32f other, Size size, float alpha):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiAddWeighted_32f_C1IR( <ipp.Ipp32f*>other.im, other.step,
 *                                                    <ipp.Ipp32f*>self.im, self.step,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("FastImage.FastImage32f.fast_toself_add_weighted_32f", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":605
 * 
 * 
 *     def toself_add_square(self, FastImage8u other, Size size):             # <<<<<<<<<<<<<<
 *         """self += other**2"""
 *         CHK_HAVEGIL( ipp.ippiAddSquare_8u32f_C1IR( <ipp.Ipp8u*>other.im, other.step,
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_11toself_add_square(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_10toself_add_square[] = "self += other**2";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_11toself_add_square(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toself_add_square (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("toself_add_square", 1, 2, 2, 1); __PYX_ERR(0, 605, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "toself_add_square") < 0)) __PYX_ERR(0, 605, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_other = ((struct __pyx_obj_9FastImage_FastImage8u *)values[0]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("toself_add_square", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 605, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.toself_add_square", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage8u, 1, "other", 0))) __PYX_ERR(0, 605, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 605, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_10toself_add_square(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_other, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_10toself_add_square(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toself_add_square", 0);

  /* "FastImage.pyx":607
 *     def toself_add_square(self, FastImage8u other, Size size):
 *         """self += other**2"""
 *         CHK_HAVEGIL( ipp.ippiAddSquare_8u32f_C1IR( <ipp.Ipp8u*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                                    <ipp.Ipp32f*>self.im, self.step,
 *                                                      size.sz))
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiAddSquare_8u32f_C1IR(((Ipp8u *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 607, __pyx_L1_error)

  /* "FastImage.pyx":605
 * 
 * 
 *     def toself_add_square(self, FastImage8u other, Size size):             # <<<<<<<<<<<<<<
 *         """self += other**2"""
 *         CHK_HAVEGIL( ipp.ippiAddSquare_8u32f_C1IR( <ipp.Ipp8u*>other.im, other.step,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.toself_add_square", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":610
 *                                                    <ipp.Ipp32f*>self.im, self.step,
 *                                                      size.sz))
 *     def get_square_put(self, FastImage32f result, Size size):             # <<<<<<<<<<<<<<
 *         """result = self**2"""
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1R( <ipp.Ipp32f*>self.im, self.step,
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_13get_square_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_12get_square_put[] = "result = self**2";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_13get_square_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_square_put (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_result,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_result)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_square_put", 1, 2, 2, 1); __PYX_ERR(0, 610, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_square_put") < 0)) __PYX_ERR(0, 610, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImage32f *)values[0]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_square_put", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 610, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.get_square_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_result), __pyx_ptype_9FastImage_FastImage32f, 1, "result", 0))) __PYX_ERR(0, 610, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 610, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_12get_square_put(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_result, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_12get_square_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_square_put", 0);

  /* "FastImage.pyx":612
 *     def get_square_put(self, FastImage32f result, Size size):
 *         """result = self**2"""
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1R( <ipp.Ipp32f*>self.im, self.step,             # <<<<<<<<<<<<<<
 *                                           <ipp.Ipp32f*>result.im, result.step,
 *                                           size.sz))
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSqr_32f_C1R(((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp32f *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 612, __pyx_L1_error)

  /* "FastImage.pyx":610
 *                                                    <ipp.Ipp32f*>self.im, self.step,
 *                                                      size.sz))
 *     def get_square_put(self, FastImage32f result, Size size):             # <<<<<<<<<<<<<<
 *         """result = self**2"""
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1R( <ipp.Ipp32f*>self.im, self.step,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.get_square_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":616
 *                                           size.sz))
 * 
 *     cdef void fast_get_square_put(self, FastImage32f result, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1R( <ipp.Ipp32f*>self.im, self.step,
 *                                         <ipp.Ipp32f*>result.im, result.step,
 */

static void __pyx_f_9FastImage_12FastImage32f_fast_get_square_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fast_get_square_put", 0);

  /* "FastImage.pyx":617
 * 
 *     cdef void fast_get_square_put(self, FastImage32f result, Size size):
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1R( <ipp.Ipp32f*>self.im, self.step,             # <<<<<<<<<<<<<<
 *                                         <ipp.Ipp32f*>result.im, result.step,
 *                                         size.sz))
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSqr_32f_C1R(((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp32f *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 617, __pyx_L1_error)

  /* "FastImage.pyx":616
 *                                           size.sz))
 * 
 *     cdef void fast_get_square_put(self, FastImage32f result, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1R( <ipp.Ipp32f*>self.im, self.step,
 *                                         <ipp.Ipp32f*>result.im, result.step,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("FastImage.FastImage32f.fast_get_square_put", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":621
 *                                         size.sz))
 * 
 *     def get_square(self, Size size):             # <<<<<<<<<<<<<<
 *         """return self**2"""
 *         cdef FastImage32f result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_15get_square(PyObject *__pyx_v_self, PyObject *__pyx_v_size); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_14get_square[] = "return self**2";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_15get_square(PyObject *__pyx_v_self, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_square (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 621, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_14get_square(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), ((struct __pyx_obj_9FastImage_Size *)__pyx_v_size));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_14get_square(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("get_square", 0);

  /* "FastImage.pyx":624
 *         """return self**2"""
 *         cdef FastImage32f result
 *         result = FastImage32f( size )             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1R( <ipp.Ipp32f*>self.im, self.step,
 *                                           <ipp.Ipp32f*>result.im, result.step,
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 624, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_size));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_size));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_FastImage32f), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 624, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "FastImage.pyx":625
 *         cdef FastImage32f result
 *         result = FastImage32f( size )
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1R( <ipp.Ipp32f*>self.im, self.step,             # <<<<<<<<<<<<<<
 *                                           <ipp.Ipp32f*>result.im, result.step,
 *                                           size.sz))
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSqr_32f_C1R(((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp32f *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 625, __pyx_L1_error)

  /* "FastImage.pyx":628
 *                                           <ipp.Ipp32f*>result.im, result.step,
 *                                           size.sz))
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def toself_square(self,Size size):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "FastImage.pyx":621
 *                                         size.sz))
 * 
 *     def get_square(self, Size size):             # <<<<<<<<<<<<<<
 *         """return self**2"""
 *         cdef FastImage32f result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("FastImage.FastImage32f.get_square", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":630
 *         return result
 * 
 *     def toself_square(self,Size size):             # <<<<<<<<<<<<<<
 *         """self = self**2"""
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1IR( <ipp.Ipp32f*>self.im, self.step,
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_17toself_square(PyObject *__pyx_v_self, PyObject *__pyx_v_size); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_16toself_square[] = "self = self**2";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_17toself_square(PyObject *__pyx_v_self, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toself_square (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 630, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_16toself_square(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), ((struct __pyx_obj_9FastImage_Size *)__pyx_v_size));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_16toself_square(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toself_square", 0);

  /* "FastImage.pyx":632
 *     def toself_square(self,Size size):
 *         """self = self**2"""
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1IR( <ipp.Ipp32f*>self.im, self.step,             # <<<<<<<<<<<<<<
 *                                           size.sz))
 *     cdef void fast_toself_square(self,Size size):
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSqr_32f_C1IR(((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 632, __pyx_L1_error)

  /* "FastImage.pyx":630
 *         return result
 * 
 *     def toself_square(self,Size size):             # <<<<<<<<<<<<<<
 *         """self = self**2"""
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1IR( <ipp.Ipp32f*>self.im, self.step,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.toself_square", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":634
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1IR( <ipp.Ipp32f*>self.im, self.step,
 *                                           size.sz))
 *     cdef void fast_toself_square(self,Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1IR( <ipp.Ipp32f*>self.im, self.step,
 *                                          size.sz))
 */

static void __pyx_f_9FastImage_12FastImage32f_fast_toself_square(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fast_toself_square", 0);

  /* "FastImage.pyx":635
 *                                           size.sz))
 *     cdef void fast_toself_square(self,Size size):
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1IR( <ipp.Ipp32f*>self.im, self.step,             # <<<<<<<<<<<<<<
 *                                          size.sz))
 *     def toself_sqrt(self,Size size):
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSqr_32f_C1IR(((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 635, __pyx_L1_error)

  /* "FastImage.pyx":634
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1IR( <ipp.Ipp32f*>self.im, self.step,
 *                                           size.sz))
 *     cdef void fast_toself_square(self,Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1IR( <ipp.Ipp32f*>self.im, self.step,
 *                                          size.sz))
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("FastImage.FastImage32f.fast_toself_square", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":637
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1IR( <ipp.Ipp32f*>self.im, self.step,
 *                                          size.sz))
 *     def toself_sqrt(self,Size size):             # <<<<<<<<<<<<<<
 *         """self = sqrt(self)"""
 *         CHK_HAVEGIL( ipp.ippiSqrt_32f_C1IR( <ipp.Ipp32f*>self.im, self.step,
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_19toself_sqrt(PyObject *__pyx_v_self, PyObject *__pyx_v_size); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_18toself_sqrt[] = "self = sqrt(self)";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_19toself_sqrt(PyObject *__pyx_v_self, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toself_sqrt (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 637, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_18toself_sqrt(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), ((struct __pyx_obj_9FastImage_Size *)__pyx_v_size));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_18toself_sqrt(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toself_sqrt", 0);

  /* "FastImage.pyx":639
 *     def toself_sqrt(self,Size size):
 *         """self = sqrt(self)"""
 *         CHK_HAVEGIL( ipp.ippiSqrt_32f_C1IR( <ipp.Ipp32f*>self.im, self.step,             # <<<<<<<<<<<<<<
 *                                             size.sz))
 * 
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSqrt_32f_C1IR(((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 639, __pyx_L1_error)

  /* "FastImage.pyx":637
 *         CHK_HAVEGIL( ipp.ippiSqr_32f_C1IR( <ipp.Ipp32f*>self.im, self.step,
 *                                          size.sz))
 *     def toself_sqrt(self,Size size):             # <<<<<<<<<<<<<<
 *         """self = sqrt(self)"""
 *         CHK_HAVEGIL( ipp.ippiSqrt_32f_C1IR( <ipp.Ipp32f*>self.im, self.step,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.toself_sqrt", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":642
 *                                             size.sz))
 * 
 *     def get_subtracted(self,FastImage32f other,Size size):             # <<<<<<<<<<<<<<
 *         """result = self - other"""
 *         cdef FastImage32f result
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_21get_subtracted(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_20get_subtracted[] = "result = self - other";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_21get_subtracted(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_subtracted (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_subtracted", 1, 2, 2, 1); __PYX_ERR(0, 642, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_subtracted") < 0)) __PYX_ERR(0, 642, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_other = ((struct __pyx_obj_9FastImage_FastImage32f *)values[0]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_subtracted", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 642, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.get_subtracted", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage32f, 1, "other", 0))) __PYX_ERR(0, 642, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 642, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_20get_subtracted(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_other, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_20get_subtracted(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("get_subtracted", 0);

  /* "FastImage.pyx":645
 *         """result = self - other"""
 *         cdef FastImage32f result
 *         result = FastImage32f( size )             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiSub_32f_C1R(<ipp.Ipp32f*>other.im, other.step,
 *                                           <ipp.Ipp32f*>self.im, self.step,
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_size));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_size));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_FastImage32f), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 645, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "FastImage.pyx":646
 *         cdef FastImage32f result
 *         result = FastImage32f( size )
 *         CHK_HAVEGIL( ipp.ippiSub_32f_C1R(<ipp.Ipp32f*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                           <ipp.Ipp32f*>self.im, self.step,
 *                                           <ipp.Ipp32f*>result.im, result.step,
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSub_32f_C1R(((Ipp32f *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp32f *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 646, __pyx_L1_error)

  /* "FastImage.pyx":650
 *                                           <ipp.Ipp32f*>result.im, result.step,
 *                                           size.sz))
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def get_subtracted_put(self,FastImage32f other,FastImage32f result,Size size):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "FastImage.pyx":642
 *                                             size.sz))
 * 
 *     def get_subtracted(self,FastImage32f other,Size size):             # <<<<<<<<<<<<<<
 *         """result = self - other"""
 *         cdef FastImage32f result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("FastImage.FastImage32f.get_subtracted", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":652
 *         return result
 * 
 *     def get_subtracted_put(self,FastImage32f other,FastImage32f result,Size size):             # <<<<<<<<<<<<<<
 *         """result = self - other"""
 *         CHK_HAVEGIL( ipp.ippiSub_32f_C1R(<ipp.Ipp32f*>other.im, other.step,
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_23get_subtracted_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_22get_subtracted_put[] = "result = self - other";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_23get_subtracted_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_subtracted_put (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_result,&__pyx_n_s_size,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_result)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_subtracted_put", 1, 3, 3, 1); __PYX_ERR(0, 652, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_subtracted_put", 1, 3, 3, 2); __PYX_ERR(0, 652, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_subtracted_put") < 0)) __PYX_ERR(0, 652, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_other = ((struct __pyx_obj_9FastImage_FastImage32f *)values[0]);
    __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImage32f *)values[1]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_subtracted_put", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 652, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.get_subtracted_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage32f, 1, "other", 0))) __PYX_ERR(0, 652, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_result), __pyx_ptype_9FastImage_FastImage32f, 1, "result", 0))) __PYX_ERR(0, 652, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 652, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_22get_subtracted_put(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_other, __pyx_v_result, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_22get_subtracted_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_subtracted_put", 0);

  /* "FastImage.pyx":654
 *     def get_subtracted_put(self,FastImage32f other,FastImage32f result,Size size):
 *         """result = self - other"""
 *         CHK_HAVEGIL( ipp.ippiSub_32f_C1R(<ipp.Ipp32f*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                           <ipp.Ipp32f*>self.im, self.step,
 *                                           <ipp.Ipp32f*>result.im, result.step,
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSub_32f_C1R(((Ipp32f *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp32f *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 654, __pyx_L1_error)

  /* "FastImage.pyx":652
 *         return result
 * 
 *     def get_subtracted_put(self,FastImage32f other,FastImage32f result,Size size):             # <<<<<<<<<<<<<<
 *         """result = self - other"""
 *         CHK_HAVEGIL( ipp.ippiSub_32f_C1R(<ipp.Ipp32f*>other.im, other.step,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.get_subtracted_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":659
 *                                           size.sz))
 * 
 *     cdef void fast_get_subtracted_put(self,FastImage32f other,FastImage32f result,Size size):             # <<<<<<<<<<<<<<
 *         """result = self - other"""
 *         CHK_HAVEGIL( ipp.ippiSub_32f_C1R(<ipp.Ipp32f*>other.im, other.step,
 */

static void __pyx_f_9FastImage_12FastImage32f_fast_get_subtracted_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_result, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fast_get_subtracted_put", 0);

  /* "FastImage.pyx":661
 *     cdef void fast_get_subtracted_put(self,FastImage32f other,FastImage32f result,Size size):
 *         """result = self - other"""
 *         CHK_HAVEGIL( ipp.ippiSub_32f_C1R(<ipp.Ipp32f*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                           <ipp.Ipp32f*>self.im, self.step,
 *                                           <ipp.Ipp32f*>result.im, result.step,
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSub_32f_C1R(((Ipp32f *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp32f *)__pyx_v_result->__pyx_base.im), __pyx_v_result->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 661, __pyx_L1_error)

  /* "FastImage.pyx":659
 *                                           size.sz))
 * 
 *     cdef void fast_get_subtracted_put(self,FastImage32f other,FastImage32f result,Size size):             # <<<<<<<<<<<<<<
 *         """result = self - other"""
 *         CHK_HAVEGIL( ipp.ippiSub_32f_C1R(<ipp.Ipp32f*>other.im, other.step,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("FastImage.FastImage32f.fast_get_subtracted_put", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":666
 *                                           size.sz))
 * 
 *     def toself_subtract(self,FastImage32f other,Size size):             # <<<<<<<<<<<<<<
 *         """self = self - other"""
 *         CHK_HAVEGIL( ipp.ippiSub_32f_C1IR(<ipp.Ipp32f*>other.im, other.step,
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_25toself_subtract(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_24toself_subtract[] = "self = self - other";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_25toself_subtract(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toself_subtract (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("toself_subtract", 1, 2, 2, 1); __PYX_ERR(0, 666, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "toself_subtract") < 0)) __PYX_ERR(0, 666, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_other = ((struct __pyx_obj_9FastImage_FastImage32f *)values[0]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("toself_subtract", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 666, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.toself_subtract", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage32f, 1, "other", 0))) __PYX_ERR(0, 666, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 666, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_24toself_subtract(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_other, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_24toself_subtract(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toself_subtract", 0);

  /* "FastImage.pyx":668
 *     def toself_subtract(self,FastImage32f other,Size size):
 *         """self = self - other"""
 *         CHK_HAVEGIL( ipp.ippiSub_32f_C1IR(<ipp.Ipp32f*>other.im, other.step,             # <<<<<<<<<<<<<<
 *                                           <ipp.Ipp32f*>self.im, self.step,
 *                                           size.sz))
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiSub_32f_C1IR(((Ipp32f *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 668, __pyx_L1_error)

  /* "FastImage.pyx":666
 *                                           size.sz))
 * 
 *     def toself_subtract(self,FastImage32f other,Size size):             # <<<<<<<<<<<<<<
 *         """self = self - other"""
 *         CHK_HAVEGIL( ipp.ippiSub_32f_C1IR(<ipp.Ipp32f*>other.im, other.step,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.toself_subtract", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":672
 *                                           size.sz))
 * 
 *     def get_8u_copy(self,Size size):             # <<<<<<<<<<<<<<
 *         """other = self"""
 *         cdef FastImage8u other
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_27get_8u_copy(PyObject *__pyx_v_self, PyObject *__pyx_v_size); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_26get_8u_copy[] = "other = self";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_27get_8u_copy(PyObject *__pyx_v_self, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_8u_copy (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 672, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_26get_8u_copy(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), ((struct __pyx_obj_9FastImage_Size *)__pyx_v_size));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_26get_8u_copy(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("get_8u_copy", 0);

  /* "FastImage.pyx":675
 *         """other = self"""
 *         cdef FastImage8u other
 *         other = FastImage8u(size)             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiConvert_32f8u_C1R(
 *             <ipp.Ipp32f*>self.im,self.step,
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 675, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_size));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_size));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_FastImage8u), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 675, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_other = ((struct __pyx_obj_9FastImage_FastImage8u *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "FastImage.pyx":676
 *         cdef FastImage8u other
 *         other = FastImage8u(size)
 *         CHK_HAVEGIL( ipp.ippiConvert_32f8u_C1R(             # <<<<<<<<<<<<<<
 *             <ipp.Ipp32f*>self.im,self.step,
 *             <ipp.Ipp8u*>other.im,other.step,
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiConvert_32f8u_C1R(((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp8u *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, __pyx_v_size->sz, ippRndNear)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 676, __pyx_L1_error)

  /* "FastImage.pyx":680
 *             <ipp.Ipp8u*>other.im,other.step,
 *             size.sz, ipp.ippRndNear ))
 *         return other             # <<<<<<<<<<<<<<
 * 
 *     def get_8u_copy_put(self,FastImage8u other,Size size):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_other));
  __pyx_r = ((PyObject *)__pyx_v_other);
  goto __pyx_L0;

  /* "FastImage.pyx":672
 *                                           size.sz))
 * 
 *     def get_8u_copy(self,Size size):             # <<<<<<<<<<<<<<
 *         """other = self"""
 *         cdef FastImage8u other
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("FastImage.FastImage32f.get_8u_copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_other);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":682
 *         return other
 * 
 *     def get_8u_copy_put(self,FastImage8u other,Size size):             # <<<<<<<<<<<<<<
 *         """other = self"""
 *         CHK_HAVEGIL( ipp.ippiConvert_32f8u_C1R(
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_29get_8u_copy_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_28get_8u_copy_put[] = "other = self";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_29get_8u_copy_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_8u_copy_put (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_8u_copy_put", 1, 2, 2, 1); __PYX_ERR(0, 682, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_8u_copy_put") < 0)) __PYX_ERR(0, 682, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_other = ((struct __pyx_obj_9FastImage_FastImage8u *)values[0]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_8u_copy_put", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 682, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.get_8u_copy_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage8u, 1, "other", 0))) __PYX_ERR(0, 682, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 682, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_28get_8u_copy_put(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_other, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_28get_8u_copy_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_8u_copy_put", 0);

  /* "FastImage.pyx":684
 *     def get_8u_copy_put(self,FastImage8u other,Size size):
 *         """other = self"""
 *         CHK_HAVEGIL( ipp.ippiConvert_32f8u_C1R(             # <<<<<<<<<<<<<<
 *             <ipp.Ipp32f*>self.im,self.step,
 *             <ipp.Ipp8u*>other.im,other.step,
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiConvert_32f8u_C1R(((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp8u *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, __pyx_v_size->sz, ippRndNear)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 684, __pyx_L1_error)

  /* "FastImage.pyx":682
 *         return other
 * 
 *     def get_8u_copy_put(self,FastImage8u other,Size size):             # <<<<<<<<<<<<<<
 *         """other = self"""
 *         CHK_HAVEGIL( ipp.ippiConvert_32f8u_C1R(
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.get_8u_copy_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":689
 *             size.sz, ipp.ippRndNear ))
 * 
 *     cdef void fast_get_8u_copy_put(self,FastImage8u other,Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiConvert_32f8u_C1R(
 *             <ipp.Ipp32f*>self.im,self.step,
 */

static void __pyx_f_9FastImage_12FastImage32f_fast_get_8u_copy_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage8u *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fast_get_8u_copy_put", 0);

  /* "FastImage.pyx":690
 * 
 *     cdef void fast_get_8u_copy_put(self,FastImage8u other,Size size):
 *         CHK_HAVEGIL( ipp.ippiConvert_32f8u_C1R(             # <<<<<<<<<<<<<<
 *             <ipp.Ipp32f*>self.im,self.step,
 *             <ipp.Ipp8u*>other.im,other.step,
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiConvert_32f8u_C1R(((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Ipp8u *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, __pyx_v_size->sz, ippRndNear)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 690, __pyx_L1_error)

  /* "FastImage.pyx":689
 *             size.sz, ipp.ippRndNear ))
 * 
 *     cdef void fast_get_8u_copy_put(self,FastImage8u other,Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiConvert_32f8u_C1R(
 *             <ipp.Ipp32f*>self.im,self.step,
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("FastImage.FastImage32f.fast_get_8u_copy_put", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":695
 *             size.sz, ipp.ippRndNear ))
 * 
 *     def toself_multiply(self, float val, Size size):             # <<<<<<<<<<<<<<
 *         "self = self * C"
 *         CHK_HAVEGIL( ipp.ippiMulC_32f_C1IR(val, <ipp.Ipp32f*>self.im, self.step, size.sz))
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_31toself_multiply(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_30toself_multiply[] = "self = self * C";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_31toself_multiply(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  float __pyx_v_val;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toself_multiply (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_val,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_val)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("toself_multiply", 1, 2, 2, 1); __PYX_ERR(0, 695, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "toself_multiply") < 0)) __PYX_ERR(0, 695, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_val = __pyx_PyFloat_AsFloat(values[0]); if (unlikely((__pyx_v_val == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 695, __pyx_L3_error)
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("toself_multiply", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 695, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.toself_multiply", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 695, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_30toself_multiply(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_val, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_30toself_multiply(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, float __pyx_v_val, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("toself_multiply", 0);

  /* "FastImage.pyx":697
 *     def toself_multiply(self, float val, Size size):
 *         "self = self * C"
 *         CHK_HAVEGIL( ipp.ippiMulC_32f_C1IR(val, <ipp.Ipp32f*>self.im, self.step, size.sz))             # <<<<<<<<<<<<<<
 * 
 *     cdef void fast_toself_multiply(self, float val, Size size):
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiMulC_32f_C1IR(__pyx_v_val, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 697, __pyx_L1_error)

  /* "FastImage.pyx":695
 *             size.sz, ipp.ippRndNear ))
 * 
 *     def toself_multiply(self, float val, Size size):             # <<<<<<<<<<<<<<
 *         "self = self * C"
 *         CHK_HAVEGIL( ipp.ippiMulC_32f_C1IR(val, <ipp.Ipp32f*>self.im, self.step, size.sz))
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.toself_multiply", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":699
 *         CHK_HAVEGIL( ipp.ippiMulC_32f_C1IR(val, <ipp.Ipp32f*>self.im, self.step, size.sz))
 * 
 *     cdef void fast_toself_multiply(self, float val, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiMulC_32f_C1IR(val, <ipp.Ipp32f*>self.im, self.step, size.sz))
 * 
 */

static void __pyx_f_9FastImage_12FastImage32f_fast_toself_multiply(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, float __pyx_v_val, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fast_toself_multiply", 0);

  /* "FastImage.pyx":700
 * 
 *     cdef void fast_toself_multiply(self, float val, Size size):
 *         CHK_HAVEGIL( ipp.ippiMulC_32f_C1IR(val, <ipp.Ipp32f*>self.im, self.step, size.sz))             # <<<<<<<<<<<<<<
 * 
 *     def get_multiply_put(self, float val, FastImage32f other, Size size):
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiMulC_32f_C1IR(__pyx_v_val, ((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 700, __pyx_L1_error)

  /* "FastImage.pyx":699
 *         CHK_HAVEGIL( ipp.ippiMulC_32f_C1IR(val, <ipp.Ipp32f*>self.im, self.step, size.sz))
 * 
 *     cdef void fast_toself_multiply(self, float val, Size size):             # <<<<<<<<<<<<<<
 *         CHK_HAVEGIL( ipp.ippiMulC_32f_C1IR(val, <ipp.Ipp32f*>self.im, self.step, size.sz))
 * 
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("FastImage.FastImage32f.fast_toself_multiply", __pyx_clineno, __pyx_lineno, __pyx_filename, 0, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "FastImage.pyx":702
 *         CHK_HAVEGIL( ipp.ippiMulC_32f_C1IR(val, <ipp.Ipp32f*>self.im, self.step, size.sz))
 * 
 *     def get_multiply_put(self, float val, FastImage32f other, Size size):             # <<<<<<<<<<<<<<
 *         "other = self * C"
 *         CHK_HAVEGIL( ipp.ippiMulC_32f_C1R(<ipp.Ipp32f*>self.im, self.step,  val,
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_33get_multiply_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_9FastImage_12FastImage32f_32get_multiply_put[] = "other = self * C";
static PyObject *__pyx_pw_9FastImage_12FastImage32f_33get_multiply_put(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  float __pyx_v_val;
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_multiply_put (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_val,&__pyx_n_s_other,&__pyx_n_s_size,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_val)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_multiply_put", 1, 3, 3, 1); __PYX_ERR(0, 702, __pyx_L3_error)
        }
        case  2:
        if (likely((values[2] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("get_multiply_put", 1, 3, 3, 2); __PYX_ERR(0, 702, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_multiply_put") < 0)) __PYX_ERR(0, 702, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_val = __pyx_PyFloat_AsFloat(values[0]); if (unlikely((__pyx_v_val == (float)-1) && PyErr_Occurred())) __PYX_ERR(0, 702, __pyx_L3_error)
    __pyx_v_other = ((struct __pyx_obj_9FastImage_FastImage32f *)values[1]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[2]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_multiply_put", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 702, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.get_multiply_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage32f, 1, "other", 0))) __PYX_ERR(0, 702, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 702, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_32get_multiply_put(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_val, __pyx_v_other, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_32get_multiply_put(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, float __pyx_v_val, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_multiply_put", 0);

  /* "FastImage.pyx":704
 *     def get_multiply_put(self, float val, FastImage32f other, Size size):
 *         "other = self * C"
 *         CHK_HAVEGIL( ipp.ippiMulC_32f_C1R(<ipp.Ipp32f*>self.im, self.step,  val,             # <<<<<<<<<<<<<<
 *                                           <ipp.Ipp32f*>other.im,other.step, size.sz))
 * 
 */
  __pyx_f_9FastImage_CHK_HAVEGIL(ippiMulC_32f_C1R(((Ipp32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_val, ((Ipp32f *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, __pyx_v_size->sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 704, __pyx_L1_error)

  /* "FastImage.pyx":702
 *         CHK_HAVEGIL( ipp.ippiMulC_32f_C1IR(val, <ipp.Ipp32f*>self.im, self.step, size.sz))
 * 
 *     def get_multiply_put(self, float val, FastImage32f other, Size size):             # <<<<<<<<<<<<<<
 *         "other = self * C"
 *         CHK_HAVEGIL( ipp.ippiMulC_32f_C1R(<ipp.Ipp32f*>self.im, self.step,  val,
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.get_multiply_put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":707
 *                                           <ipp.Ipp32f*>other.im,other.step, size.sz))
 * 
 *     def pow(self,y):             # <<<<<<<<<<<<<<
 *         cdef FastImage32f xself
 *         xself = copy(self)
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_35pow(PyObject *__pyx_v_self, PyObject *__pyx_v_y); /*proto*/
static PyObject *__pyx_pw_9FastImage_12FastImage32f_35pow(PyObject *__pyx_v_self, PyObject *__pyx_v_y) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("pow (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_34pow(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), ((PyObject *)__pyx_v_y));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_34pow(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, PyObject *__pyx_v_y) {
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_xself = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  __Pyx_RefNannySetupContext("pow", 0);

  /* "FastImage.pyx":709
 *     def pow(self,y):
 *         cdef FastImage32f xself
 *         xself = copy(self)             # <<<<<<<<<<<<<<
 *         if y == 2:
 *             xself.toself_square(xself.imsiz)
 */
  __pyx_t_2 = __Pyx_GetModuleGlobalName(__pyx_n_s_copy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 709, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  if (!__pyx_t_3) {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 709, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  } else {
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_self)};
      __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 709, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
      PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_self)};
      __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 709, __pyx_L1_error)
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_GOTREF(__pyx_t_1);
    } else
    #endif
    {
      __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 709, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
      __Pyx_INCREF(((PyObject *)__pyx_v_self));
      __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
      PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_self));
      __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 709, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9FastImage_FastImage32f))))) __PYX_ERR(0, 709, __pyx_L1_error)
  __pyx_v_xself = ((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "FastImage.pyx":710
 *         cdef FastImage32f xself
 *         xself = copy(self)
 *         if y == 2:             # <<<<<<<<<<<<<<
 *             xself.toself_square(xself.imsiz)
 *         elif y==0.5:
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_y, __pyx_int_2, 2, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 710, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 710, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "FastImage.pyx":711
 *         xself = copy(self)
 *         if y == 2:
 *             xself.toself_square(xself.imsiz)             # <<<<<<<<<<<<<<
 *         elif y==0.5:
 *             xself.toself_sqrt(xself.imsiz)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_xself), __pyx_n_s_toself_square); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 711, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_4) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_xself->__pyx_base.imsiz)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 711, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_xself->__pyx_base.imsiz)};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 711, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_4, ((PyObject *)__pyx_v_xself->__pyx_base.imsiz)};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 711, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      {
        __pyx_t_3 = PyTuple_New(1+1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 711, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4); __pyx_t_4 = NULL;
        __Pyx_INCREF(((PyObject *)__pyx_v_xself->__pyx_base.imsiz));
        __Pyx_GIVEREF(((PyObject *)__pyx_v_xself->__pyx_base.imsiz));
        PyTuple_SET_ITEM(__pyx_t_3, 0+1, ((PyObject *)__pyx_v_xself->__pyx_base.imsiz));
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 711, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "FastImage.pyx":710
 *         cdef FastImage32f xself
 *         xself = copy(self)
 *         if y == 2:             # <<<<<<<<<<<<<<
 *             xself.toself_square(xself.imsiz)
 *         elif y==0.5:
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":712
 *         if y == 2:
 *             xself.toself_square(xself.imsiz)
 *         elif y==0.5:             # <<<<<<<<<<<<<<
 *             xself.toself_sqrt(xself.imsiz)
 *         else:
 */
  __pyx_t_1 = __Pyx_PyFloat_EqObjC(__pyx_v_y, __pyx_float_0_5, 0.5, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 712, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 712, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "FastImage.pyx":713
 *             xself.toself_square(xself.imsiz)
 *         elif y==0.5:
 *             xself.toself_sqrt(xself.imsiz)             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError("only 2 and 0.5 powers supported")
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_xself), __pyx_n_s_toself_sqrt); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 713, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    if (!__pyx_t_3) {
      __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_t_2, ((PyObject *)__pyx_v_xself->__pyx_base.imsiz)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 713, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_xself->__pyx_base.imsiz)};
        __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 713, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
        PyObject *__pyx_temp[2] = {__pyx_t_3, ((PyObject *)__pyx_v_xself->__pyx_base.imsiz)};
        __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 713, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_GOTREF(__pyx_t_1);
      } else
      #endif
      {
        __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 713, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
        __Pyx_INCREF(((PyObject *)__pyx_v_xself->__pyx_base.imsiz));
        __Pyx_GIVEREF(((PyObject *)__pyx_v_xself->__pyx_base.imsiz));
        PyTuple_SET_ITEM(__pyx_t_4, 0+1, ((PyObject *)__pyx_v_xself->__pyx_base.imsiz));
        __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 713, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "FastImage.pyx":712
 *         if y == 2:
 *             xself.toself_square(xself.imsiz)
 *         elif y==0.5:             # <<<<<<<<<<<<<<
 *             xself.toself_sqrt(xself.imsiz)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":715
 *             xself.toself_sqrt(xself.imsiz)
 *         else:
 *             raise ValueError("only 2 and 0.5 powers supported")             # <<<<<<<<<<<<<<
 *         return xself
 * 
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__19, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 715, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 715, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "FastImage.pyx":716
 *         else:
 *             raise ValueError("only 2 and 0.5 powers supported")
 *         return xself             # <<<<<<<<<<<<<<
 * 
 *     def max_index(self, Size size):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_xself));
  __pyx_r = ((PyObject *)__pyx_v_xself);
  goto __pyx_L0;

  /* "FastImage.pyx":707
 *                                           <ipp.Ipp32f*>other.im,other.step, size.sz))
 * 
 *     def pow(self,y):             # <<<<<<<<<<<<<<
 *         cdef FastImage32f xself
 *         xself = copy(self)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("FastImage.FastImage32f.pow", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_xself);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":718
 *         return xself
 * 
 *     def max_index(self, Size size):             # <<<<<<<<<<<<<<
 *         cdef fic.FicStatus sts
 *         cdef int index_x, index_y
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_37max_index(PyObject *__pyx_v_self, PyObject *__pyx_v_size); /*proto*/
static PyObject *__pyx_pw_9FastImage_12FastImage32f_37max_index(PyObject *__pyx_v_self, PyObject *__pyx_v_size) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("max_index (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 718, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_36max_index(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), ((struct __pyx_obj_9FastImage_Size *)__pyx_v_size));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_36max_index(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  FicStatus __pyx_v_sts;
  int __pyx_v_index_x;
  int __pyx_v_index_y;
  Ipp32f __pyx_v_max_val;
  FiciSize __pyx_v_sz;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("max_index", 0);

  /* "FastImage.pyx":724
 *         cdef fic.FiciSize sz
 * 
 *         sz.width = size.sz.width             # <<<<<<<<<<<<<<
 *         sz.height = size.sz.height
 *         with nogil:
 */
  __pyx_t_1 = __pyx_v_size->sz.width;
  __pyx_v_sz.width = __pyx_t_1;

  /* "FastImage.pyx":725
 * 
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height             # <<<<<<<<<<<<<<
 *         with nogil:
 *             sts = fic.ficiMaxIndx_32f_C1R(
 */
  __pyx_t_1 = __pyx_v_size->sz.height;
  __pyx_v_sz.height = __pyx_t_1;

  /* "FastImage.pyx":726
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         with nogil:             # <<<<<<<<<<<<<<
 *             sts = fic.ficiMaxIndx_32f_C1R(
 *                 <fic.Fic32f*>self.im,self.step,
 */
  {
      #ifdef WITH_THREAD
      PyThreadState *_save;
      Py_UNBLOCK_THREADS
      #endif
      /*try:*/ {

        /* "FastImage.pyx":727
 *         sz.height = size.sz.height
 *         with nogil:
 *             sts = fic.ficiMaxIndx_32f_C1R(             # <<<<<<<<<<<<<<
 *                 <fic.Fic32f*>self.im,self.step,
 *                  sz, &max_val, &index_x, &index_y)
 */
        __pyx_v_sts = ficiMaxIndx_32f_C1R(((Fic32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, __pyx_v_sz, (&__pyx_v_max_val), (&__pyx_v_index_x), (&__pyx_v_index_y));
      }

      /* "FastImage.pyx":726
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         with nogil:             # <<<<<<<<<<<<<<
 *             sts = fic.ficiMaxIndx_32f_C1R(
 *                 <fic.Fic32f*>self.im,self.step,
 */
      /*finally:*/ {
        /*normal exit:*/{
          #ifdef WITH_THREAD
          Py_BLOCK_THREADS
          #endif
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "FastImage.pyx":730
 *                 <fic.Fic32f*>self.im,self.step,
 *                  sz, &max_val, &index_x, &index_y)
 *         CHK_FIC_HAVEGIL(sts)             # <<<<<<<<<<<<<<
 *         return max_val, index_x, index_y
 * 
 */
  __pyx_f_9FastImage_CHK_FIC_HAVEGIL(__pyx_v_sts); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 730, __pyx_L1_error)

  /* "FastImage.pyx":731
 *                  sz, &max_val, &index_x, &index_y)
 *         CHK_FIC_HAVEGIL(sts)
 *         return max_val, index_x, index_y             # <<<<<<<<<<<<<<
 * 
 *     def dot(self,FastImage32f other,Size size):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_max_val); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_index_x); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_index_y); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 731, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_4);
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":718
 *         return xself
 * 
 *     def max_index(self, Size size):             # <<<<<<<<<<<<<<
 *         cdef fic.FicStatus sts
 *         cdef int index_x, index_y
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("FastImage.FastImage32f.max_index", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":733
 *         return max_val, index_x, index_y
 * 
 *     def dot(self,FastImage32f other,Size size):             # <<<<<<<<<<<<<<
 *         cdef fic.Fic64f result
 *         cdef fic.FiciSize sz
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_39dot(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_12FastImage32f_39dot(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other = 0;
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("dot (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_other,&__pyx_n_s_size,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_other)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (likely((values[1] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("dot", 1, 2, 2, 1); __PYX_ERR(0, 733, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "dot") < 0)) __PYX_ERR(0, 733, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_other = ((struct __pyx_obj_9FastImage_FastImage32f *)values[0]);
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("dot", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 733, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.dot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_9FastImage_FastImage32f, 1, "other", 0))) __PYX_ERR(0, 733, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 733, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_38dot(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_other, __pyx_v_size);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_38dot(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_other, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  Fic64f __pyx_v_result;
  FiciSize __pyx_v_sz;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  __Pyx_RefNannySetupContext("dot", 0);

  /* "FastImage.pyx":737
 *         cdef fic.FiciSize sz
 * 
 *         sz.width = size.sz.width             # <<<<<<<<<<<<<<
 *         sz.height = size.sz.height
 *         CHK_FIC_HAVEGIL( fic.ficiDotProd_32f64f_C1R(
 */
  __pyx_t_1 = __pyx_v_size->sz.width;
  __pyx_v_sz.width = __pyx_t_1;

  /* "FastImage.pyx":738
 * 
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height             # <<<<<<<<<<<<<<
 *         CHK_FIC_HAVEGIL( fic.ficiDotProd_32f64f_C1R(
 *             <fic.Fic32f*>self.im,self.step,
 */
  __pyx_t_1 = __pyx_v_size->sz.height;
  __pyx_v_sz.height = __pyx_t_1;

  /* "FastImage.pyx":739
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         CHK_FIC_HAVEGIL( fic.ficiDotProd_32f64f_C1R(             # <<<<<<<<<<<<<<
 *             <fic.Fic32f*>self.im,self.step,
 *             <fic.Fic32f*>other.im,other.step,
 */
  __pyx_f_9FastImage_CHK_FIC_HAVEGIL(ficiDotProd_32f64f_C1R(((Fic32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Fic32f *)__pyx_v_other->__pyx_base.im), __pyx_v_other->__pyx_base.step, __pyx_v_sz, (&__pyx_v_result))); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 739, __pyx_L1_error)

  /* "FastImage.pyx":743
 *             <fic.Fic32f*>other.im,other.step,
 *             sz, &result ))
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     cdef FastImage32f create_equal_shape_empty(self, Size size):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_result); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 743, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "FastImage.pyx":733
 *         return max_val, index_x, index_y
 * 
 *     def dot(self,FastImage32f other,Size size):             # <<<<<<<<<<<<<<
 *         cdef fic.Fic64f result
 *         cdef fic.FiciSize sz
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("FastImage.FastImage32f.dot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":745
 *         return result
 * 
 *     cdef FastImage32f create_equal_shape_empty(self, Size size):             # <<<<<<<<<<<<<<
 *         cdef FastImage32f dest
 *         dest = FastImage32f( size )
 */

static struct __pyx_obj_9FastImage_FastImage32f *__pyx_f_9FastImage_12FastImage32f_create_equal_shape_empty(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size) {
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_dest = 0;
  PyObject *__pyx_v_shape = NULL;
  PyObject *__pyx_v_dtype = NULL;
  PyObject *__pyx_v_tmp = NULL;
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("create_equal_shape_empty", 0);

  /* "FastImage.pyx":747
 *     cdef FastImage32f create_equal_shape_empty(self, Size size):
 *         cdef FastImage32f dest
 *         dest = FastImage32f( size )             # <<<<<<<<<<<<<<
 *         if dest.step != self.step:
 *             shape = ( size.sz.height, self.step/4 )
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 747, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_size));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_size));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_size));
  __pyx_t_2 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_FastImage32f), __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 747, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_dest = ((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "FastImage.pyx":748
 *         cdef FastImage32f dest
 *         dest = FastImage32f( size )
 *         if dest.step != self.step:             # <<<<<<<<<<<<<<
 *             shape = ( size.sz.height, self.step/4 )
 *             dtype = numpy.float32
 */
  __pyx_t_3 = ((__pyx_v_dest->__pyx_base.step != __pyx_v_self->__pyx_base.step) != 0);
  if (__pyx_t_3) {

    /* "FastImage.pyx":749
 *         dest = FastImage32f( size )
 *         if dest.step != self.step:
 *             shape = ( size.sz.height, self.step/4 )             # <<<<<<<<<<<<<<
 *             dtype = numpy.float32
 *             tmp = numpy.empty( shape, dtype=dtype)
 */
    __pyx_t_2 = __Pyx_PyInt_From_int(__pyx_v_size->sz.height); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 749, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __Pyx_PyInt_From_long(__Pyx_div_long(__pyx_v_self->__pyx_base.step, 4)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 749, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 749, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_2 = 0;
    __pyx_t_1 = 0;
    __pyx_v_shape = __pyx_t_4;
    __pyx_t_4 = 0;

    /* "FastImage.pyx":750
 *         if dest.step != self.step:
 *             shape = ( size.sz.height, self.step/4 )
 *             dtype = numpy.float32             # <<<<<<<<<<<<<<
 *             tmp = numpy.empty( shape, dtype=dtype)
 *             tmp = tmp[:,:size.sz.width]
 */
    __pyx_t_4 = __Pyx_GetModuleGlobalName(__pyx_n_s_numpy); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 750, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_float32); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 750, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_dtype = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "FastImage.pyx":751
 *             shape = ( size.sz.height, self.step/4 )
 *             dtype = numpy.float32
 *             tmp = numpy.empty( shape, dtype=dtype)             # <<<<<<<<<<<<<<
 *             tmp = tmp[:,:size.sz.width]
 *             dest = asfastimage(tmp)
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_numpy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 751, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_empty); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 751, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 751, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_shape);
    __Pyx_GIVEREF(__pyx_v_shape);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_shape);
    __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 751, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_dtype, __pyx_v_dtype) < 0) __PYX_ERR(0, 751, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 751, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_tmp = __pyx_t_5;
    __pyx_t_5 = 0;

    /* "FastImage.pyx":752
 *             dtype = numpy.float32
 *             tmp = numpy.empty( shape, dtype=dtype)
 *             tmp = tmp[:,:size.sz.width]             # <<<<<<<<<<<<<<
 *             dest = asfastimage(tmp)
 *         return dest
 */
    __pyx_t_5 = __Pyx_PyInt_From_int(__pyx_v_size->sz.width); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 752, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = PySlice_New(Py_None, __pyx_t_5, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 752, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 752, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_slice__20);
    __Pyx_GIVEREF(__pyx_slice__20);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_slice__20);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_2);
    __pyx_t_2 = 0;
    __pyx_t_2 = PyObject_GetItem(__pyx_v_tmp, __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 752, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_tmp, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "FastImage.pyx":753
 *             tmp = numpy.empty( shape, dtype=dtype)
 *             tmp = tmp[:,:size.sz.width]
 *             dest = asfastimage(tmp)             # <<<<<<<<<<<<<<
 *         return dest
 * 
 */
    __pyx_t_5 = __Pyx_GetModuleGlobalName(__pyx_n_s_asfastimage); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 753, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    if (!__pyx_t_1) {
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_v_tmp); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 753, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    } else {
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_1, __pyx_v_tmp};
        __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 753, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[2] = {__pyx_t_1, __pyx_v_tmp};
        __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-1, 1+1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 753, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_GOTREF(__pyx_t_2);
      } else
      #endif
      {
        __pyx_t_4 = PyTuple_New(1+1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 753, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_GIVEREF(__pyx_t_1); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1); __pyx_t_1 = NULL;
        __Pyx_INCREF(__pyx_v_tmp);
        __Pyx_GIVEREF(__pyx_v_tmp);
        PyTuple_SET_ITEM(__pyx_t_4, 0+1, __pyx_v_tmp);
        __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_4, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 753, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      }
    }
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_9FastImage_FastImage32f))))) __PYX_ERR(0, 753, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_dest, ((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "FastImage.pyx":748
 *         cdef FastImage32f dest
 *         dest = FastImage32f( size )
 *         if dest.step != self.step:             # <<<<<<<<<<<<<<
 *             shape = ( size.sz.height, self.step/4 )
 *             dtype = numpy.float32
 */
  }

  /* "FastImage.pyx":754
 *             tmp = tmp[:,:size.sz.width]
 *             dest = asfastimage(tmp)
 *         return dest             # <<<<<<<<<<<<<<
 * 
 *     def sobel_horiz(self, Size size, FastImage32f dest=None):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_dest));
  __pyx_r = __pyx_v_dest;
  goto __pyx_L0;

  /* "FastImage.pyx":745
 *         return result
 * 
 *     cdef FastImage32f create_equal_shape_empty(self, Size size):             # <<<<<<<<<<<<<<
 *         cdef FastImage32f dest
 *         dest = FastImage32f( size )
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("FastImage.FastImage32f.create_equal_shape_empty", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_dest);
  __Pyx_XDECREF(__pyx_v_shape);
  __Pyx_XDECREF(__pyx_v_dtype);
  __Pyx_XDECREF(__pyx_v_tmp);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":756
 *         return dest
 * 
 *     def sobel_horiz(self, Size size, FastImage32f dest=None):             # <<<<<<<<<<<<<<
 *         cdef fic.FiciSize sz
 *         cdef int created_dest
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_41sobel_horiz(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_12FastImage32f_41sobel_horiz(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_dest = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("sobel_horiz (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_size,&__pyx_n_s_dest,0};
    PyObject* values[2] = {0,0};
    values[1] = (PyObject *)((struct __pyx_obj_9FastImage_FastImage32f *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dest);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "sobel_horiz") < 0)) __PYX_ERR(0, 756, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[0]);
    __pyx_v_dest = ((struct __pyx_obj_9FastImage_FastImage32f *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("sobel_horiz", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 756, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.sobel_horiz", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 756, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dest), __pyx_ptype_9FastImage_FastImage32f, 1, "dest", 0))) __PYX_ERR(0, 756, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_40sobel_horiz(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_size, __pyx_v_dest);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_40sobel_horiz(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_dest) {
  FiciSize __pyx_v_sz;
  CYTHON_UNUSED int __pyx_v_created_dest;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("sobel_horiz", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_dest);

  /* "FastImage.pyx":760
 *         cdef int created_dest
 * 
 *         created_dest = 0             # <<<<<<<<<<<<<<
 * 
 *         sz.width = size.sz.width
 */
  __pyx_v_created_dest = 0;

  /* "FastImage.pyx":762
 *         created_dest = 0
 * 
 *         sz.width = size.sz.width             # <<<<<<<<<<<<<<
 *         sz.height = size.sz.height
 *         if dest is None:
 */
  __pyx_t_1 = __pyx_v_size->sz.width;
  __pyx_v_sz.width = __pyx_t_1;

  /* "FastImage.pyx":763
 * 
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height             # <<<<<<<<<<<<<<
 *         if dest is None:
 *             dest = self.create_equal_shape_empty(size)
 */
  __pyx_t_1 = __pyx_v_size->sz.height;
  __pyx_v_sz.height = __pyx_t_1;

  /* "FastImage.pyx":764
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         if dest is None:             # <<<<<<<<<<<<<<
 *             dest = self.create_equal_shape_empty(size)
 *         else:
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_dest) == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "FastImage.pyx":765
 *         sz.height = size.sz.height
 *         if dest is None:
 *             dest = self.create_equal_shape_empty(size)             # <<<<<<<<<<<<<<
 *         else:
 *             assert dest.size == size
 */
    __pyx_t_4 = ((PyObject *)((struct __pyx_vtabstruct_9FastImage_FastImage32f *)__pyx_v_self->__pyx_base.__pyx_vtab)->create_equal_shape_empty(__pyx_v_self, __pyx_v_size)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 765, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF_SET(__pyx_v_dest, ((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "FastImage.pyx":764
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         if dest is None:             # <<<<<<<<<<<<<<
 *             dest = self.create_equal_shape_empty(size)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":767
 *             dest = self.create_equal_shape_empty(size)
 *         else:
 *             assert dest.size == size             # <<<<<<<<<<<<<<
 *         CHK_FIC_HAVEGIL( fic.ficiFilterSobelHoriz_32f_C1R(<fic.Fic32f*>self.im, self.step, <fic.Fic32f*>dest.im, dest.step, sz ))
 *         return dest
 */
  /*else*/ {
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_dest), __pyx_n_s_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 767, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PyObject_RichCompare(__pyx_t_4, ((PyObject *)__pyx_v_size), Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 767, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 767, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 767, __pyx_L1_error)
      }
    }
    #endif
  }
  __pyx_L3:;

  /* "FastImage.pyx":768
 *         else:
 *             assert dest.size == size
 *         CHK_FIC_HAVEGIL( fic.ficiFilterSobelHoriz_32f_C1R(<fic.Fic32f*>self.im, self.step, <fic.Fic32f*>dest.im, dest.step, sz ))             # <<<<<<<<<<<<<<
 *         return dest
 * 
 */
  __pyx_f_9FastImage_CHK_FIC_HAVEGIL(ficiFilterSobelHoriz_32f_C1R(((Fic32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Fic32f *)__pyx_v_dest->__pyx_base.im), __pyx_v_dest->__pyx_base.step, __pyx_v_sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 768, __pyx_L1_error)

  /* "FastImage.pyx":769
 *             assert dest.size == size
 *         CHK_FIC_HAVEGIL( fic.ficiFilterSobelHoriz_32f_C1R(<fic.Fic32f*>self.im, self.step, <fic.Fic32f*>dest.im, dest.step, sz ))
 *         return dest             # <<<<<<<<<<<<<<
 * 
 *     def sobel_vert(self, Size size, FastImage32f dest=None):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_dest));
  __pyx_r = ((PyObject *)__pyx_v_dest);
  goto __pyx_L0;

  /* "FastImage.pyx":756
 *         return dest
 * 
 *     def sobel_horiz(self, Size size, FastImage32f dest=None):             # <<<<<<<<<<<<<<
 *         cdef fic.FiciSize sz
 *         cdef int created_dest
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("FastImage.FastImage32f.sobel_horiz", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_dest);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":771
 *         return dest
 * 
 *     def sobel_vert(self, Size size, FastImage32f dest=None):             # <<<<<<<<<<<<<<
 *         cdef fic.FiciSize sz
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_12FastImage32f_43sobel_vert(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_pw_9FastImage_12FastImage32f_43sobel_vert(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_9FastImage_Size *__pyx_v_size = 0;
  struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_dest = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("sobel_vert (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_size,&__pyx_n_s_dest,0};
    PyObject* values[2] = {0,0};
    values[1] = (PyObject *)((struct __pyx_obj_9FastImage_FastImage32f *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = PyDict_GetItem(__pyx_kwds, __pyx_n_s_size)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        case  1:
        if (kw_args > 0) {
          PyObject* value = PyDict_GetItem(__pyx_kwds, __pyx_n_s_dest);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "sobel_vert") < 0)) __PYX_ERR(0, 771, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_size = ((struct __pyx_obj_9FastImage_Size *)values[0]);
    __pyx_v_dest = ((struct __pyx_obj_9FastImage_FastImage32f *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("sobel_vert", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 771, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("FastImage.FastImage32f.sobel_vert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_size), __pyx_ptype_9FastImage_Size, 1, "size", 0))) __PYX_ERR(0, 771, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_dest), __pyx_ptype_9FastImage_FastImage32f, 1, "dest", 0))) __PYX_ERR(0, 771, __pyx_L1_error)
  __pyx_r = __pyx_pf_9FastImage_12FastImage32f_42sobel_vert(((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_v_self), __pyx_v_size, __pyx_v_dest);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_12FastImage32f_42sobel_vert(struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_self, struct __pyx_obj_9FastImage_Size *__pyx_v_size, struct __pyx_obj_9FastImage_FastImage32f *__pyx_v_dest) {
  FiciSize __pyx_v_sz;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  __Pyx_RefNannySetupContext("sobel_vert", 0);
  __Pyx_INCREF((PyObject *)__pyx_v_dest);

  /* "FastImage.pyx":774
 *         cdef fic.FiciSize sz
 * 
 *         sz.width = size.sz.width             # <<<<<<<<<<<<<<
 *         sz.height = size.sz.height
 *         if dest is None:
 */
  __pyx_t_1 = __pyx_v_size->sz.width;
  __pyx_v_sz.width = __pyx_t_1;

  /* "FastImage.pyx":775
 * 
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height             # <<<<<<<<<<<<<<
 *         if dest is None:
 *             dest = self.create_equal_shape_empty(size)
 */
  __pyx_t_1 = __pyx_v_size->sz.height;
  __pyx_v_sz.height = __pyx_t_1;

  /* "FastImage.pyx":776
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         if dest is None:             # <<<<<<<<<<<<<<
 *             dest = self.create_equal_shape_empty(size)
 *         else:
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_dest) == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "FastImage.pyx":777
 *         sz.height = size.sz.height
 *         if dest is None:
 *             dest = self.create_equal_shape_empty(size)             # <<<<<<<<<<<<<<
 *         else:
 *             assert dest.size == size
 */
    __pyx_t_4 = ((PyObject *)((struct __pyx_vtabstruct_9FastImage_FastImage32f *)__pyx_v_self->__pyx_base.__pyx_vtab)->create_equal_shape_empty(__pyx_v_self, __pyx_v_size)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 777, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF_SET(__pyx_v_dest, ((struct __pyx_obj_9FastImage_FastImage32f *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "FastImage.pyx":776
 *         sz.width = size.sz.width
 *         sz.height = size.sz.height
 *         if dest is None:             # <<<<<<<<<<<<<<
 *             dest = self.create_equal_shape_empty(size)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "FastImage.pyx":779
 *             dest = self.create_equal_shape_empty(size)
 *         else:
 *             assert dest.size == size             # <<<<<<<<<<<<<<
 *         CHK_FIC_HAVEGIL( fic.ficiFilterSobelVert_32f_C1R(<fic.Fic32f*>self.im, self.step, <fic.Fic32f*>dest.im, dest.step, sz ))
 *         return dest
 */
  /*else*/ {
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(!Py_OptimizeFlag)) {
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_dest), __pyx_n_s_size); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 779, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_5 = PyObject_RichCompare(__pyx_t_4, ((PyObject *)__pyx_v_size), Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 779, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 779, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_3)) {
        PyErr_SetNone(PyExc_AssertionError);
        __PYX_ERR(0, 779, __pyx_L1_error)
      }
    }
    #endif
  }
  __pyx_L3:;

  /* "FastImage.pyx":780
 *         else:
 *             assert dest.size == size
 *         CHK_FIC_HAVEGIL( fic.ficiFilterSobelVert_32f_C1R(<fic.Fic32f*>self.im, self.step, <fic.Fic32f*>dest.im, dest.step, sz ))             # <<<<<<<<<<<<<<
 *         return dest
 * 
 */
  __pyx_f_9FastImage_CHK_FIC_HAVEGIL(ficiFilterSobelVert_32f_C1R(((Fic32f *)__pyx_v_self->__pyx_base.im), __pyx_v_self->__pyx_base.step, ((Fic32f *)__pyx_v_dest->__pyx_base.im), __pyx_v_dest->__pyx_base.step, __pyx_v_sz)); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 780, __pyx_L1_error)

  /* "FastImage.pyx":781
 *             assert dest.size == size
 *         CHK_FIC_HAVEGIL( fic.ficiFilterSobelVert_32f_C1R(<fic.Fic32f*>self.im, self.step, <fic.Fic32f*>dest.im, dest.step, sz ))
 *         return dest             # <<<<<<<<<<<<<<
 * 
 * def asfastimage( object arr ):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_dest));
  __pyx_r = ((PyObject *)__pyx_v_dest);
  goto __pyx_L0;

  /* "FastImage.pyx":771
 *         return dest
 * 
 *     def sobel_vert(self, Size size, FastImage32f dest=None):             # <<<<<<<<<<<<<<
 *         cdef fic.FiciSize sz
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("FastImage.FastImage32f.sobel_vert", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_dest);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":783
 *         return dest
 * 
 * def asfastimage( object arr ):             # <<<<<<<<<<<<<<
 *     """return a FastImage view of an array"""
 *     cdef PyArrayInterface* inter
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_9asfastimage(PyObject *__pyx_self, PyObject *__pyx_v_arr); /*proto*/
static char __pyx_doc_9FastImage_8asfastimage[] = "return a FastImage view of an array";
static PyMethodDef __pyx_mdef_9FastImage_9asfastimage = {"asfastimage", (PyCFunction)__pyx_pw_9FastImage_9asfastimage, METH_O, __pyx_doc_9FastImage_8asfastimage};
static PyObject *__pyx_pw_9FastImage_9asfastimage(PyObject *__pyx_self, PyObject *__pyx_v_arr) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("asfastimage (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_8asfastimage(__pyx_self, ((PyObject *)__pyx_v_arr));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_8asfastimage(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_arr) {
  __pyx_t_9FastImage_PyArrayInterface *__pyx_v_inter;
  struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_result = 0;
  PyObject *__pyx_v_attr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_t_8;
  void *__pyx_t_9;
  __Pyx_RefNannySetupContext("asfastimage", 0);

  /* "FastImage.pyx":790
 *     global FASTIMAGEDEBUG
 * 
 *     attr = arr.__array_struct__             # <<<<<<<<<<<<<<
 *     if not c_python.PyCObject_Check(attr):
 *         raise ValueError("invalid __array_struct__")
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_arr, __pyx_n_s_array_struct); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 790, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_attr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "FastImage.pyx":791
 * 
 *     attr = arr.__array_struct__
 *     if not c_python.PyCObject_Check(attr):             # <<<<<<<<<<<<<<
 *         raise ValueError("invalid __array_struct__")
 * 
 */
  __pyx_t_2 = ((!(PyCObject_Check(__pyx_v_attr) != 0)) != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":792
 *     attr = arr.__array_struct__
 *     if not c_python.PyCObject_Check(attr):
 *         raise ValueError("invalid __array_struct__")             # <<<<<<<<<<<<<<
 * 
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__21, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 792, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 792, __pyx_L1_error)

    /* "FastImage.pyx":791
 * 
 *     attr = arr.__array_struct__
 *     if not c_python.PyCObject_Check(attr):             # <<<<<<<<<<<<<<
 *         raise ValueError("invalid __array_struct__")
 * 
 */
  }

  /* "FastImage.pyx":794
 *         raise ValueError("invalid __array_struct__")
 * 
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)             # <<<<<<<<<<<<<<
 *     if inter.two != 2:
 *         raise ValueError("invalid __array_struct__")
 */
  __pyx_v_inter = ((__pyx_t_9FastImage_PyArrayInterface *)PyCObject_AsVoidPtr(__pyx_v_attr));

  /* "FastImage.pyx":795
 * 
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)
 *     if inter.two != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError("invalid __array_struct__")
 * 
 */
  __pyx_t_2 = ((__pyx_v_inter->two != 2) != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":796
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)
 *     if inter.two != 2:
 *         raise ValueError("invalid __array_struct__")             # <<<<<<<<<<<<<<
 * 
 *     if (inter.flags & ARR_HAS_DESCR) == (ARR_HAS_DESCR):
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__22, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 796, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 796, __pyx_L1_error)

    /* "FastImage.pyx":795
 * 
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)
 *     if inter.two != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError("invalid __array_struct__")
 * 
 */
  }

  /* "FastImage.pyx":798
 *         raise ValueError("invalid __array_struct__")
 * 
 *     if (inter.flags & ARR_HAS_DESCR) == (ARR_HAS_DESCR):             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot handle arrays with descr")
 * 
 */
  __pyx_t_2 = (((__pyx_v_inter->flags & ARR_HAS_DESCR) == ARR_HAS_DESCR) != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":799
 * 
 *     if (inter.flags & ARR_HAS_DESCR) == (ARR_HAS_DESCR):
 *         raise ValueError("cannot handle arrays with descr")             # <<<<<<<<<<<<<<
 * 
 *     # TODO: don't really know what these flags all mean, figure out if
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__23, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 799, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 799, __pyx_L1_error)

    /* "FastImage.pyx":798
 *         raise ValueError("invalid __array_struct__")
 * 
 *     if (inter.flags & ARR_HAS_DESCR) == (ARR_HAS_DESCR):             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot handle arrays with descr")
 * 
 */
  }

  /* "FastImage.pyx":803
 *     # TODO: don't really know what these flags all mean, figure out if
 *     # this is OK:
 *     if (inter.flags & (ALIGNED | WRITEABLE)) != (ALIGNED | WRITEABLE):             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot handle misaligned or not writeable arrays.")
 * 
 */
  __pyx_t_2 = (((__pyx_v_inter->flags & (ALIGNED | WRITEABLE)) != (ALIGNED | WRITEABLE)) != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":804
 *     # this is OK:
 *     if (inter.flags & (ALIGNED | WRITEABLE)) != (ALIGNED | WRITEABLE):
 *         raise ValueError("cannot handle misaligned or not writeable arrays.")             # <<<<<<<<<<<<<<
 * 
 *     if inter.nd != 2:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__24, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 804, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 804, __pyx_L1_error)

    /* "FastImage.pyx":803
 *     # TODO: don't really know what these flags all mean, figure out if
 *     # this is OK:
 *     if (inter.flags & (ALIGNED | WRITEABLE)) != (ALIGNED | WRITEABLE):             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot handle misaligned or not writeable arrays.")
 * 
 */
  }

  /* "FastImage.pyx":806
 *         raise ValueError("cannot handle misaligned or not writeable arrays.")
 * 
 *     if inter.nd != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError("only 2D arrays may be viewed as a FastImage")
 * 
 */
  __pyx_t_2 = ((__pyx_v_inter->nd != 2) != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":807
 * 
 *     if inter.nd != 2:
 *         raise ValueError("only 2D arrays may be viewed as a FastImage")             # <<<<<<<<<<<<<<
 * 
 *     if FASTIMAGEDEBUG>=2:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__25, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 807, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 807, __pyx_L1_error)

    /* "FastImage.pyx":806
 *         raise ValueError("cannot handle misaligned or not writeable arrays.")
 * 
 *     if inter.nd != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError("only 2D arrays may be viewed as a FastImage")
 * 
 */
  }

  /* "FastImage.pyx":809
 *         raise ValueError("only 2D arrays may be viewed as a FastImage")
 * 
 *     if FASTIMAGEDEBUG>=2:             # <<<<<<<<<<<<<<
 *         print 'asfastimage() calling __new__'
 *     if inter.typekind == CHAR_u and inter.itemsize==1:
 */
  __pyx_t_2 = ((__pyx_v_9FastImage_FASTIMAGEDEBUG >= 2) != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":810
 * 
 *     if FASTIMAGEDEBUG>=2:
 *         print 'asfastimage() calling __new__'             # <<<<<<<<<<<<<<
 *     if inter.typekind == CHAR_u and inter.itemsize==1:
 *         result = FastImage8u.__new__(FastImage8u)
 */
    if (__Pyx_PrintOne(0, __pyx_kp_s_asfastimage_calling___new) < 0) __PYX_ERR(0, 810, __pyx_L1_error)

    /* "FastImage.pyx":809
 *         raise ValueError("only 2D arrays may be viewed as a FastImage")
 * 
 *     if FASTIMAGEDEBUG>=2:             # <<<<<<<<<<<<<<
 *         print 'asfastimage() calling __new__'
 *     if inter.typekind == CHAR_u and inter.itemsize==1:
 */
  }

  /* "FastImage.pyx":811
 *     if FASTIMAGEDEBUG>=2:
 *         print 'asfastimage() calling __new__'
 *     if inter.typekind == CHAR_u and inter.itemsize==1:             # <<<<<<<<<<<<<<
 *         result = FastImage8u.__new__(FastImage8u)
 * ##    elif inter.typekind == "b"[0] and inter.itemsize==1: # "bool" true or false
 */
  __pyx_t_3 = ((__pyx_v_inter->typekind == __pyx_v_9FastImage_CHAR_u) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L10_bool_binop_done;
  }
  __pyx_t_3 = ((__pyx_v_inter->itemsize == 1) != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L10_bool_binop_done:;
  if (__pyx_t_2) {

    /* "FastImage.pyx":812
 *         print 'asfastimage() calling __new__'
 *     if inter.typekind == CHAR_u and inter.itemsize==1:
 *         result = FastImage8u.__new__(FastImage8u)             # <<<<<<<<<<<<<<
 * ##    elif inter.typekind == "b"[0] and inter.itemsize==1: # "bool" true or false
 * ##        result = FastImage8u.__new__(FastImage8u)
 */
    __pyx_t_1 = __pyx_tp_new_9FastImage_FastImage8u(((PyTypeObject *)__pyx_ptype_9FastImage_FastImage8u), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 812, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9FastImage_FastImageBase)))) __PYX_ERR(0, 812, __pyx_L1_error)
    __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "FastImage.pyx":811
 *     if FASTIMAGEDEBUG>=2:
 *         print 'asfastimage() calling __new__'
 *     if inter.typekind == CHAR_u and inter.itemsize==1:             # <<<<<<<<<<<<<<
 *         result = FastImage8u.__new__(FastImage8u)
 * ##    elif inter.typekind == "b"[0] and inter.itemsize==1: # "bool" true or false
 */
    goto __pyx_L9;
  }

  /* "FastImage.pyx":815
 * ##    elif inter.typekind == "b"[0] and inter.itemsize==1: # "bool" true or false
 * ##        result = FastImage8u.__new__(FastImage8u)
 *     elif inter.typekind == CHAR_f and inter.itemsize==4:             # <<<<<<<<<<<<<<
 *         result = FastImage32f.__new__(FastImage32f)
 *     else:
 */
  __pyx_t_3 = ((__pyx_v_inter->typekind == __pyx_v_9FastImage_CHAR_f) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L12_bool_binop_done;
  }
  __pyx_t_3 = ((__pyx_v_inter->itemsize == 4) != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L12_bool_binop_done:;
  if (__pyx_t_2) {

    /* "FastImage.pyx":816
 * ##        result = FastImage8u.__new__(FastImage8u)
 *     elif inter.typekind == CHAR_f and inter.itemsize==4:
 *         result = FastImage32f.__new__(FastImage32f)             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError('typekind "%s", itemsize %d not supported'%(chr(inter.typekind),inter.itemsize))
 */
    __pyx_t_1 = __pyx_tp_new_9FastImage_FastImage32f(((PyTypeObject *)__pyx_ptype_9FastImage_FastImage32f), __pyx_empty_tuple, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 816, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_9FastImage_FastImageBase)))) __PYX_ERR(0, 816, __pyx_L1_error)
    __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "FastImage.pyx":815
 * ##    elif inter.typekind == "b"[0] and inter.itemsize==1: # "bool" true or false
 * ##        result = FastImage8u.__new__(FastImage8u)
 *     elif inter.typekind == CHAR_f and inter.itemsize==4:             # <<<<<<<<<<<<<<
 *         result = FastImage32f.__new__(FastImage32f)
 *     else:
 */
    goto __pyx_L9;
  }

  /* "FastImage.pyx":818
 *         result = FastImage32f.__new__(FastImage32f)
 *     else:
 *         raise ValueError('typekind "%s", itemsize %d not supported'%(chr(inter.typekind),inter.itemsize))             # <<<<<<<<<<<<<<
 *     if (inter.shape[0] > INT_MAX) or (inter.shape[1] > INT_MAX) or (inter.strides[0] > INT_MAX):
 *         raise ValueError("cannot handle such large data")
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyInt_From_char(__pyx_v_inter->typekind); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 818, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 818, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_chr, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 818, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_inter->itemsize); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 818, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 818, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_4);
    __pyx_t_1 = 0;
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_typekind_s_itemsize_d_not_suppor, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 818, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 818, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 818, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 818, __pyx_L1_error)
  }
  __pyx_L9:;

  /* "FastImage.pyx":819
 *     else:
 *         raise ValueError('typekind "%s", itemsize %d not supported'%(chr(inter.typekind),inter.itemsize))
 *     if (inter.shape[0] > INT_MAX) or (inter.shape[1] > INT_MAX) or (inter.strides[0] > INT_MAX):             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot handle such large data")
 *     FastImageBase.__init__(result, Size(inter.shape[1],inter.shape[0])) # run common __init__
 */
  __pyx_t_3 = (((__pyx_v_inter->shape[0]) > INT_MAX) != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_3 = (((__pyx_v_inter->shape[1]) > INT_MAX) != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_3 = (((__pyx_v_inter->strides[0]) > INT_MAX) != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_2) {

    /* "FastImage.pyx":820
 *         raise ValueError('typekind "%s", itemsize %d not supported'%(chr(inter.typekind),inter.itemsize))
 *     if (inter.shape[0] > INT_MAX) or (inter.shape[1] > INT_MAX) or (inter.strides[0] > INT_MAX):
 *         raise ValueError("cannot handle such large data")             # <<<<<<<<<<<<<<
 *     FastImageBase.__init__(result, Size(inter.shape[1],inter.shape[0])) # run common __init__
 *     result.strides[0] = inter.strides[0]
 */
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__26, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 820, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_Raise(__pyx_t_4, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __PYX_ERR(0, 820, __pyx_L1_error)

    /* "FastImage.pyx":819
 *     else:
 *         raise ValueError('typekind "%s", itemsize %d not supported'%(chr(inter.typekind),inter.itemsize))
 *     if (inter.shape[0] > INT_MAX) or (inter.shape[1] > INT_MAX) or (inter.strides[0] > INT_MAX):             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot handle such large data")
 *     FastImageBase.__init__(result, Size(inter.shape[1],inter.shape[0])) # run common __init__
 */
  }

  /* "FastImage.pyx":821
 *     if (inter.shape[0] > INT_MAX) or (inter.shape[1] > INT_MAX) or (inter.strides[0] > INT_MAX):
 *         raise ValueError("cannot handle such large data")
 *     FastImageBase.__init__(result, Size(inter.shape[1],inter.shape[0])) # run common __init__             # <<<<<<<<<<<<<<
 *     result.strides[0] = inter.strides[0]
 *     result.step = inter.strides[0]
 */
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_9FastImage_FastImageBase), __pyx_n_s_init); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 821, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_inter->shape[1])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 821, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_6 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_inter->shape[0])); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 821, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = PyTuple_New(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 821, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_6);
  PyTuple_SET_ITEM(__pyx_t_7, 1, __pyx_t_6);
  __pyx_t_1 = 0;
  __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_Size), __pyx_t_7, NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 821, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = NULL;
  __pyx_t_8 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
    __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
    if (likely(__pyx_t_7)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_7);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_5, function);
      __pyx_t_8 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_7, ((PyObject *)__pyx_v_result), __pyx_t_6};
    __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 821, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
    PyObject *__pyx_temp[3] = {__pyx_t_7, ((PyObject *)__pyx_v_result), __pyx_t_6};
    __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_8, 2+__pyx_t_8); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 821, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  } else
  #endif
  {
    __pyx_t_1 = PyTuple_New(2+__pyx_t_8); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 821, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (__pyx_t_7) {
      __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_7); __pyx_t_7 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_result));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_result));
    PyTuple_SET_ITEM(__pyx_t_1, 0+__pyx_t_8, ((PyObject *)__pyx_v_result));
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_1, 1+__pyx_t_8, __pyx_t_6);
    __pyx_t_6 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_1, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 821, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "FastImage.pyx":822
 *         raise ValueError("cannot handle such large data")
 *     FastImageBase.__init__(result, Size(inter.shape[1],inter.shape[0])) # run common __init__
 *     result.strides[0] = inter.strides[0]             # <<<<<<<<<<<<<<
 *     result.step = inter.strides[0]
 *     result.im = inter.data
 */
  (__pyx_v_result->strides[0]) = (__pyx_v_inter->strides[0]);

  /* "FastImage.pyx":823
 *     FastImageBase.__init__(result, Size(inter.shape[1],inter.shape[0])) # run common __init__
 *     result.strides[0] = inter.strides[0]
 *     result.step = inter.strides[0]             # <<<<<<<<<<<<<<
 *     result.im = inter.data
 * 
 */
  __pyx_v_result->step = (__pyx_v_inter->strides[0]);

  /* "FastImage.pyx":824
 *     result.strides[0] = inter.strides[0]
 *     result.step = inter.strides[0]
 *     result.im = inter.data             # <<<<<<<<<<<<<<
 * 
 *     result.source_data = arr # keep reference to original image in case it goes out of scope elsewhere
 */
  __pyx_t_9 = __pyx_v_inter->data;
  __pyx_v_result->im = __pyx_t_9;

  /* "FastImage.pyx":826
 *     result.im = inter.data
 * 
 *     result.source_data = arr # keep reference to original image in case it goes out of scope elsewhere             # <<<<<<<<<<<<<<
 * 
 *     if FASTIMAGEDEBUG>=2:
 */
  __Pyx_INCREF(__pyx_v_arr);
  __Pyx_GIVEREF(__pyx_v_arr);
  __Pyx_GOTREF(__pyx_v_result->source_data);
  __Pyx_DECREF(__pyx_v_result->source_data);
  __pyx_v_result->source_data = __pyx_v_arr;

  /* "FastImage.pyx":828
 *     result.source_data = arr # keep reference to original image in case it goes out of scope elsewhere
 * 
 *     if FASTIMAGEDEBUG>=2:             # <<<<<<<<<<<<<<
 *         print '  result',result
 *         print '    result.view',result.view
 */
  __pyx_t_2 = ((__pyx_v_9FastImage_FASTIMAGEDEBUG >= 2) != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":829
 * 
 *     if FASTIMAGEDEBUG>=2:
 *         print '  result',result             # <<<<<<<<<<<<<<
 *         print '    result.view',result.view
 * 
 */
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 829, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_kp_s_result_2);
    __Pyx_GIVEREF(__pyx_kp_s_result_2);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_kp_s_result_2);
    __Pyx_INCREF(((PyObject *)__pyx_v_result));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_result));
    PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_result));
    if (__Pyx_Print(0, __pyx_t_4, 1) < 0) __PYX_ERR(0, 829, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "FastImage.pyx":830
 *     if FASTIMAGEDEBUG>=2:
 *         print '  result',result
 *         print '    result.view',result.view             # <<<<<<<<<<<<<<
 * 
 *     return result
 */
    __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_result->view); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 830, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 830, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_kp_s_result_view);
    __Pyx_GIVEREF(__pyx_kp_s_result_view);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_kp_s_result_view);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_4);
    __pyx_t_4 = 0;
    if (__Pyx_Print(0, __pyx_t_5, 1) < 0) __PYX_ERR(0, 830, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "FastImage.pyx":828
 *     result.source_data = arr # keep reference to original image in case it goes out of scope elsewhere
 * 
 *     if FASTIMAGEDEBUG>=2:             # <<<<<<<<<<<<<<
 *         print '  result',result
 *         print '    result.view',result.view
 */
  }

  /* "FastImage.pyx":832
 *         print '    result.view',result.view
 * 
 *     return result             # <<<<<<<<<<<<<<
 * 
 * def copy( object arr ):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "FastImage.pyx":783
 *         return dest
 * 
 * def asfastimage( object arr ):             # <<<<<<<<<<<<<<
 *     """return a FastImage view of an array"""
 *     cdef PyArrayInterface* inter
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("FastImage.asfastimage", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_attr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "FastImage.pyx":834
 *     return result
 * 
 * def copy( object arr ):             # <<<<<<<<<<<<<<
 *     """return a FastImage copy of an array"""
 *     cdef PyArrayInterface* inter
 */

/* Python wrapper */
static PyObject *__pyx_pw_9FastImage_11copy(PyObject *__pyx_self, PyObject *__pyx_v_arr); /*proto*/
static char __pyx_doc_9FastImage_10copy[] = "return a FastImage copy of an array";
static PyMethodDef __pyx_mdef_9FastImage_11copy = {"copy", (PyCFunction)__pyx_pw_9FastImage_11copy, METH_O, __pyx_doc_9FastImage_10copy};
static PyObject *__pyx_pw_9FastImage_11copy(PyObject *__pyx_self, PyObject *__pyx_v_arr) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  __pyx_r = __pyx_pf_9FastImage_10copy(__pyx_self, ((PyObject *)__pyx_v_arr));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_9FastImage_10copy(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_arr) {
  __pyx_t_9FastImage_PyArrayInterface *__pyx_v_inter;
  struct __pyx_obj_9FastImage_FastImageBase *__pyx_v_result = 0;
  int __pyx_v_i;
  PyObject *__pyx_v_attr = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_intptr_t __pyx_t_6;
  __Pyx_RefNannySetupContext("copy", 0);

  /* "FastImage.pyx":840
 *     cdef int i
 * 
 *     attr = arr.__array_struct__             # <<<<<<<<<<<<<<
 *     if not c_python.PyCObject_Check(attr):
 *         raise ValueError("invalid __array_struct__")
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_arr, __pyx_n_s_array_struct); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 840, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_attr = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "FastImage.pyx":841
 * 
 *     attr = arr.__array_struct__
 *     if not c_python.PyCObject_Check(attr):             # <<<<<<<<<<<<<<
 *         raise ValueError("invalid __array_struct__")
 * 
 */
  __pyx_t_2 = ((!(PyCObject_Check(__pyx_v_attr) != 0)) != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":842
 *     attr = arr.__array_struct__
 *     if not c_python.PyCObject_Check(attr):
 *         raise ValueError("invalid __array_struct__")             # <<<<<<<<<<<<<<
 * 
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__27, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 842, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 842, __pyx_L1_error)

    /* "FastImage.pyx":841
 * 
 *     attr = arr.__array_struct__
 *     if not c_python.PyCObject_Check(attr):             # <<<<<<<<<<<<<<
 *         raise ValueError("invalid __array_struct__")
 * 
 */
  }

  /* "FastImage.pyx":844
 *         raise ValueError("invalid __array_struct__")
 * 
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)             # <<<<<<<<<<<<<<
 *     if inter.two != 2:
 *         raise ValueError("invalid __array_struct__")
 */
  __pyx_v_inter = ((__pyx_t_9FastImage_PyArrayInterface *)PyCObject_AsVoidPtr(__pyx_v_attr));

  /* "FastImage.pyx":845
 * 
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)
 *     if inter.two != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError("invalid __array_struct__")
 * 
 */
  __pyx_t_2 = ((__pyx_v_inter->two != 2) != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":846
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)
 *     if inter.two != 2:
 *         raise ValueError("invalid __array_struct__")             # <<<<<<<<<<<<<<
 * 
 *     if (inter.flags & ARR_HAS_DESCR) == (ARR_HAS_DESCR):
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__28, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 846, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 846, __pyx_L1_error)

    /* "FastImage.pyx":845
 * 
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)
 *     if inter.two != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError("invalid __array_struct__")
 * 
 */
  }

  /* "FastImage.pyx":848
 *         raise ValueError("invalid __array_struct__")
 * 
 *     if (inter.flags & ARR_HAS_DESCR) == (ARR_HAS_DESCR):             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot handle arrays with descr")
 * 
 */
  __pyx_t_2 = (((__pyx_v_inter->flags & ARR_HAS_DESCR) == ARR_HAS_DESCR) != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":849
 * 
 *     if (inter.flags & ARR_HAS_DESCR) == (ARR_HAS_DESCR):
 *         raise ValueError("cannot handle arrays with descr")             # <<<<<<<<<<<<<<
 * 
 *     # TODO: don't really know what these flags all mean, figure out if
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__29, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 849, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 849, __pyx_L1_error)

    /* "FastImage.pyx":848
 *         raise ValueError("invalid __array_struct__")
 * 
 *     if (inter.flags & ARR_HAS_DESCR) == (ARR_HAS_DESCR):             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot handle arrays with descr")
 * 
 */
  }

  /* "FastImage.pyx":853
 *     # TODO: don't really know what these flags all mean, figure out if
 *     # this is OK:
 *     if (inter.flags & (ALIGNED)) != (ALIGNED):             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot handle misaligned arrays.")
 * 
 */
  __pyx_t_2 = (((__pyx_v_inter->flags & ALIGNED) != ALIGNED) != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":854
 *     # this is OK:
 *     if (inter.flags & (ALIGNED)) != (ALIGNED):
 *         raise ValueError("cannot handle misaligned arrays.")             # <<<<<<<<<<<<<<
 * 
 *     if inter.nd != 2:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__30, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 854, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 854, __pyx_L1_error)

    /* "FastImage.pyx":853
 *     # TODO: don't really know what these flags all mean, figure out if
 *     # this is OK:
 *     if (inter.flags & (ALIGNED)) != (ALIGNED):             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot handle misaligned arrays.")
 * 
 */
  }

  /* "FastImage.pyx":856
 *         raise ValueError("cannot handle misaligned arrays.")
 * 
 *     if inter.nd != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError("only 2D arrays may be copied to a FastImage")
 * 
 */
  __pyx_t_2 = ((__pyx_v_inter->nd != 2) != 0);
  if (__pyx_t_2) {

    /* "FastImage.pyx":857
 * 
 *     if inter.nd != 2:
 *         raise ValueError("only 2D arrays may be copied to a FastImage")             # <<<<<<<<<<<<<<
 * 
 *     if (inter.shape[0] > INT_MAX) or (inter.shape[1] > INT_MAX) or (inter.strides[0] > INT_MAX):
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__31, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 857, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 857, __pyx_L1_error)

    /* "FastImage.pyx":856
 *         raise ValueError("cannot handle misaligned arrays.")
 * 
 *     if inter.nd != 2:             # <<<<<<<<<<<<<<
 *         raise ValueError("only 2D arrays may be copied to a FastImage")
 * 
 */
  }

  /* "FastImage.pyx":859
 *         raise ValueError("only 2D arrays may be copied to a FastImage")
 * 
 *     if (inter.shape[0] > INT_MAX) or (inter.shape[1] > INT_MAX) or (inter.strides[0] > INT_MAX):             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot handle such large data")
 * 
 */
  __pyx_t_3 = (((__pyx_v_inter->shape[0]) > INT_MAX) != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_3 = (((__pyx_v_inter->shape[1]) > INT_MAX) != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L9_bool_binop_done;
  }
  __pyx_t_3 = (((__pyx_v_inter->strides[0]) > INT_MAX) != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L9_bool_binop_done:;
  if (__pyx_t_2) {

    /* "FastImage.pyx":860
 * 
 *     if (inter.shape[0] > INT_MAX) or (inter.shape[1] > INT_MAX) or (inter.strides[0] > INT_MAX):
 *         raise ValueError("cannot handle such large data")             # <<<<<<<<<<<<<<
 * 
 *     if inter.typekind == CHAR_u and inter.itemsize==1:
 */
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__32, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 860, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 860, __pyx_L1_error)

    /* "FastImage.pyx":859
 *         raise ValueError("only 2D arrays may be copied to a FastImage")
 * 
 *     if (inter.shape[0] > INT_MAX) or (inter.shape[1] > INT_MAX) or (inter.strides[0] > INT_MAX):             # <<<<<<<<<<<<<<
 *         raise ValueError("cannot handle such large data")
 * 
 */
  }

  /* "FastImage.pyx":862
 *         raise ValueError("cannot handle such large data")
 * 
 *     if inter.typekind == CHAR_u and inter.itemsize==1:             # <<<<<<<<<<<<<<
 *         result = FastImage8u(Size(inter.shape[1],inter.shape[0])) # allocate memory
 *     elif inter.typekind == CHAR_f and inter.itemsize==4:
 */
  __pyx_t_3 = ((__pyx_v_inter->typekind == __pyx_v_9FastImage_CHAR_u) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L13_bool_binop_done;
  }
  __pyx_t_3 = ((__pyx_v_inter->itemsize == 1) != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L13_bool_binop_done:;
  if (__pyx_t_2) {

    /* "FastImage.pyx":863
 * 
 *     if inter.typekind == CHAR_u and inter.itemsize==1:
 *         result = FastImage8u(Size(inter.shape[1],inter.shape[0])) # allocate memory             # <<<<<<<<<<<<<<
 *     elif inter.typekind == CHAR_f and inter.itemsize==4:
 *         result = FastImage32f(Size(inter.shape[1],inter.shape[0])) # allocate memory
 */
    __pyx_t_1 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_inter->shape[1])); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_inter->shape[0])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_4);
    __pyx_t_1 = 0;
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_Size), __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __pyx_t_4 = 0;
    __pyx_t_4 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_FastImage8u), __pyx_t_5, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 863, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "FastImage.pyx":862
 *         raise ValueError("cannot handle such large data")
 * 
 *     if inter.typekind == CHAR_u and inter.itemsize==1:             # <<<<<<<<<<<<<<
 *         result = FastImage8u(Size(inter.shape[1],inter.shape[0])) # allocate memory
 *     elif inter.typekind == CHAR_f and inter.itemsize==4:
 */
    goto __pyx_L12;
  }

  /* "FastImage.pyx":864
 *     if inter.typekind == CHAR_u and inter.itemsize==1:
 *         result = FastImage8u(Size(inter.shape[1],inter.shape[0])) # allocate memory
 *     elif inter.typekind == CHAR_f and inter.itemsize==4:             # <<<<<<<<<<<<<<
 *         result = FastImage32f(Size(inter.shape[1],inter.shape[0])) # allocate memory
 *     else:
 */
  __pyx_t_3 = ((__pyx_v_inter->typekind == __pyx_v_9FastImage_CHAR_f) != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L15_bool_binop_done;
  }
  __pyx_t_3 = ((__pyx_v_inter->itemsize == 4) != 0);
  __pyx_t_2 = __pyx_t_3;
  __pyx_L15_bool_binop_done:;
  if (__pyx_t_2) {

    /* "FastImage.pyx":865
 *         result = FastImage8u(Size(inter.shape[1],inter.shape[0])) # allocate memory
 *     elif inter.typekind == CHAR_f and inter.itemsize==4:
 *         result = FastImage32f(Size(inter.shape[1],inter.shape[0])) # allocate memory             # <<<<<<<<<<<<<<
 *     else:
 *         raise ValueError('typekind "%s", itemsize %d not supported'%(chr(inter.typekind),inter.itemsize))
 */
    __pyx_t_4 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_inter->shape[1])); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 865, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyInt_From_Py_intptr_t((__pyx_v_inter->shape[0])); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 865, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 865, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_5);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_Size), __pyx_t_1, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 865, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 865, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_9FastImage_FastImage32f), __pyx_t_1, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 865, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_result = ((struct __pyx_obj_9FastImage_FastImageBase *)__pyx_t_5);
    __pyx_t_5 = 0;

    /* "FastImage.pyx":864
 *     if inter.typekind == CHAR_u and inter.itemsize==1:
 *         result = FastImage8u(Size(inter.shape[1],inter.shape[0])) # allocate memory
 *     elif inter.typekind == CHAR_f and inter.itemsize==4:             # <<<<<<<<<<<<<<
 *         result = FastImage32f(Size(inter.shape[1],inter.shape[0])) # allocate memory
 *     else:
 */
    goto __pyx_L12;
  }

  /* "FastImage.pyx":867
 *         result = FastImage32f(Size(inter.shape[1],inter.shape[0])) # allocate memory
 *     else:
 *         raise ValueError('typekind "%s", itemsize %d not supported'%(chr(inter.typekind),inter.itemsize))             # <<<<<<<<<<<<<<
 * 
 *     # TODO: make sure that we can rely on blocks like this:
 */
  /*else*/ {
    __pyx_t_5 = __Pyx_PyInt_From_char(__pyx_v_inter->typekind); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_5);
    __pyx_t_5 = 0;
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_builtin_chr, __pyx_t_1, NULL); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyInt_From_int(__pyx_v_inter->itemsize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_typekind_s_itemsize_d_not_suppor, __pyx_t_4); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
    __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 867, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 867, __pyx_L1_error)
  }
  __pyx_L12:;

  /* "FastImage.pyx":870
 * 
 *     # TODO: make sure that we can rely on blocks like this:
 *     for i from 0 <= i < inter.shape[0]:             # <<<<<<<<<<<<<<
 *         c_lib.memcpy( result.im + i*result.step,
 *                       inter.data + i*inter.strides[0],
 */
  __pyx_t_6 = (__pyx_v_inter->shape[0]);
  for (__pyx_v_i = 0; __pyx_v_i < __pyx_t_6; __pyx_v_i++) {

    /* "FastImage.pyx":871
 *     # TODO: make sure that we can rely on blocks like this:
 *     for i from 0 <= i < inter.shape[0]:
 *         c_lib.memcpy( result.im + i*result.step,             # <<<<<<<<<<<<<<
 *                       inter.data + i*inter.strides[0],
 *                       inter.shape[1]*result.strides[1] )
 */
    memcpy((__pyx_v_result->im + (__pyx_v_i * __pyx_v_result->step)), (__pyx_v_inter->data + (__pyx_v_i * (__pyx_v_inter->strides[0]))), ((__pyx_v_inter->shape[1]) * (__pyx_v_result->strides[1])));
  }

  /* "FastImage.pyx":875
 *                       inter.shape[1]*result.strides[1] )
 * 
 *     return result             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_result));
  __pyx_r = ((PyObject *)__pyx_v_result);
  goto __pyx_L0;

  /* "FastImage.pyx":834
 *     return result
 * 
 * def copy( object arr ):             # <<<<<<<<<<<<<<
 *     """return a FastImage copy of an array"""
 *     cdef PyArrayInterface* inter
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("FastImage.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_attr);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_tp_new_9FastImage_Size(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_9FastImage_Size(PyObject *o) {
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_9FastImage_4Size_h(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9FastImage_4Size_1h_1__get__(o);
}

static PyObject *__pyx_getprop_9FastImage_4Size_w(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9FastImage_4Size_1w_1__get__(o);
}

static PyMethodDef __pyx_methods_9FastImage_Size[] = {
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9FastImage_Size[] = {
  {(char *)"h", __pyx_getprop_9FastImage_4Size_h, 0, (char *)0, 0},
  {(char *)"w", __pyx_getprop_9FastImage_4Size_w, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9FastImage_Size = {
  PyVarObject_HEAD_INIT(0, 0)
  "FastImage.Size", /*tp_name*/
  sizeof(struct __pyx_obj_9FastImage_Size), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9FastImage_Size, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_9FastImage_4Size_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  "size of 2D image\n\n    sz=Size(width,height)\n    ", /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  __pyx_pw_9FastImage_4Size_5__richcmp__, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9FastImage_Size, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9FastImage_Size, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9FastImage_4Size_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9FastImage_Size, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyObject *__pyx_tp_new_9FastImage_Point(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  return o;
}

static void __pyx_tp_dealloc_9FastImage_Point(PyObject *o) {
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && (!PyType_IS_GC(Py_TYPE(o)) || !_PyGC_FINALIZED(o))) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  (*Py_TYPE(o)->tp_free)(o);
}

static PyObject *__pyx_getprop_9FastImage_5Point_x(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9FastImage_5Point_1x_1__get__(o);
}

static PyObject *__pyx_getprop_9FastImage_5Point_y(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9FastImage_5Point_1y_1__get__(o);
}

static PyMethodDef __pyx_methods_9FastImage_Point[] = {
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9FastImage_Point[] = {
  {(char *)"x", __pyx_getprop_9FastImage_5Point_x, 0, (char *)0, 0},
  {(char *)"y", __pyx_getprop_9FastImage_5Point_y, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9FastImage_Point = {
  PyVarObject_HEAD_INIT(0, 0)
  "FastImage.Point", /*tp_name*/
  sizeof(struct __pyx_obj_9FastImage_Point), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9FastImage_Point, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_9FastImage_5Point_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  __pyx_pw_9FastImage_5Point_5__richcmp__, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9FastImage_Point, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9FastImage_Point, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9FastImage_5Point_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9FastImage_Point, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_9FastImage_FastImageBase __pyx_vtable_9FastImage_FastImageBase;

static PyObject *__pyx_tp_new_9FastImage_FastImageBase(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9FastImage_FastImageBase *p;
  PyObject *o;
  if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
    o = (*t->tp_alloc)(t, 0);
  } else {
    o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
  }
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9FastImage_FastImageBase *)o);
  p->__pyx_vtab = __pyx_vtabptr_9FastImage_FastImageBase;
  p->basetype = Py_None; Py_INCREF(Py_None);
  p->imsiz = ((struct __pyx_obj_9FastImage_Size *)Py_None); Py_INCREF(Py_None);
  p->source_data = Py_None; Py_INCREF(Py_None);
  if (unlikely(__pyx_pw_9FastImage_13FastImageBase_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static void __pyx_tp_dealloc_9FastImage_FastImageBase(PyObject *o) {
  struct __pyx_obj_9FastImage_FastImageBase *p = (struct __pyx_obj_9FastImage_FastImageBase *)o;
  #if PY_VERSION_HEX >= 0x030400a1
  if (unlikely(Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    ++Py_REFCNT(o);
    __pyx_pw_9FastImage_13FastImageBase_5__dealloc__(o);
    --Py_REFCNT(o);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->basetype);
  Py_CLEAR(p->imsiz);
  Py_CLEAR(p->source_data);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_9FastImage_FastImageBase(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_9FastImage_FastImageBase *p = (struct __pyx_obj_9FastImage_FastImageBase *)o;
  if (p->basetype) {
    e = (*v)(p->basetype, a); if (e) return e;
  }
  if (p->imsiz) {
    e = (*v)(((PyObject*)p->imsiz), a); if (e) return e;
  }
  if (p->source_data) {
    e = (*v)(p->source_data, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_9FastImage_FastImageBase(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_9FastImage_FastImageBase *p = (struct __pyx_obj_9FastImage_FastImageBase *)o;
  tmp = ((PyObject*)p->basetype);
  p->basetype = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->imsiz);
  p->imsiz = ((struct __pyx_obj_9FastImage_Size *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->source_data);
  p->source_data = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
static PyObject *__pyx_sq_item_9FastImage_FastImageBase(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static PyObject *__pyx_getprop_9FastImage_13FastImageBase__step(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9FastImage_13FastImageBase_5_step_1__get__(o);
}

static PyObject *__pyx_getprop_9FastImage_13FastImageBase___array_struct__(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9FastImage_13FastImageBase_16__array_struct___1__get__(o);
}

static PyObject *__pyx_getprop_9FastImage_13FastImageBase___array_interface__(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9FastImage_13FastImageBase_19__array_interface___1__get__(o);
}

static PyObject *__pyx_getprop_9FastImage_13FastImageBase_size(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_9FastImage_13FastImageBase_4size_1__get__(o);
}

static PyMethodDef __pyx_methods_9FastImage_FastImageBase[] = {
  {"show_mem_and_step", (PyCFunction)__pyx_pw_9FastImage_13FastImageBase_7show_mem_and_step, METH_NOARGS, 0},
  {"stringview", (PyCFunction)__pyx_pw_9FastImage_13FastImageBase_9stringview, METH_NOARGS, 0},
  {"dump_to_file", (PyCFunction)__pyx_pw_9FastImage_13FastImageBase_11dump_to_file, METH_O, 0},
  {"roi", (PyCFunction)__pyx_pw_9FastImage_13FastImageBase_13roi, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_9FastImage_FastImageBase[] = {
  {(char *)"_step", __pyx_getprop_9FastImage_13FastImageBase__step, 0, (char *)0, 0},
  {(char *)"__array_struct__", __pyx_getprop_9FastImage_13FastImageBase___array_struct__, 0, (char *)0, 0},
  {(char *)"__array_interface__", __pyx_getprop_9FastImage_13FastImageBase___array_interface__, 0, (char *)0, 0},
  {(char *)"size", __pyx_getprop_9FastImage_13FastImageBase_size, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_FastImageBase = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_9FastImage_FastImageBase, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_FastImageBase = {
  0, /*mp_length*/
  __pyx_pw_9FastImage_13FastImageBase_15__getitem__, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_9FastImage_FastImageBase = {
  PyVarObject_HEAD_INIT(0, 0)
  "FastImage.FastImageBase", /*tp_name*/
  sizeof(struct __pyx_obj_9FastImage_FastImageBase), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9FastImage_FastImageBase, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_FastImageBase, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_FastImageBase, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_9FastImage_FastImageBase, /*tp_traverse*/
  __pyx_tp_clear_9FastImage_FastImageBase, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9FastImage_FastImageBase, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_9FastImage_FastImageBase, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9FastImage_13FastImageBase_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9FastImage_FastImageBase, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_9FastImage_FastImage32f __pyx_vtable_9FastImage_FastImage32f;

static PyObject *__pyx_tp_new_9FastImage_FastImage32f(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9FastImage_FastImage32f *p;
  PyObject *o = __pyx_tp_new_9FastImage_FastImageBase(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9FastImage_FastImage32f *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9FastImage_FastImageBase*)__pyx_vtabptr_9FastImage_FastImage32f;
  if (unlikely(__pyx_pw_9FastImage_12FastImage32f_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static PyMethodDef __pyx_methods_9FastImage_FastImage32f[] = {
  {"set_val", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_5set_val, METH_VARARGS|METH_KEYWORDS, 0},
  {"toself_add", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_7toself_add, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9FastImage_12FastImage32f_6toself_add},
  {"toself_add_weighted", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_9toself_add_weighted, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9FastImage_12FastImage32f_8toself_add_weighted},
  {"toself_add_square", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_11toself_add_square, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9FastImage_12FastImage32f_10toself_add_square},
  {"get_square_put", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_13get_square_put, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9FastImage_12FastImage32f_12get_square_put},
  {"get_square", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_15get_square, METH_O, __pyx_doc_9FastImage_12FastImage32f_14get_square},
  {"toself_square", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_17toself_square, METH_O, __pyx_doc_9FastImage_12FastImage32f_16toself_square},
  {"toself_sqrt", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_19toself_sqrt, METH_O, __pyx_doc_9FastImage_12FastImage32f_18toself_sqrt},
  {"get_subtracted", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_21get_subtracted, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9FastImage_12FastImage32f_20get_subtracted},
  {"get_subtracted_put", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_23get_subtracted_put, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9FastImage_12FastImage32f_22get_subtracted_put},
  {"toself_subtract", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_25toself_subtract, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9FastImage_12FastImage32f_24toself_subtract},
  {"get_8u_copy", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_27get_8u_copy, METH_O, __pyx_doc_9FastImage_12FastImage32f_26get_8u_copy},
  {"get_8u_copy_put", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_29get_8u_copy_put, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9FastImage_12FastImage32f_28get_8u_copy_put},
  {"toself_multiply", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_31toself_multiply, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9FastImage_12FastImage32f_30toself_multiply},
  {"get_multiply_put", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_33get_multiply_put, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9FastImage_12FastImage32f_32get_multiply_put},
  {"pow", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_35pow, METH_O, 0},
  {"max_index", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_37max_index, METH_O, 0},
  {"dot", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_39dot, METH_VARARGS|METH_KEYWORDS, 0},
  {"sobel_horiz", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_41sobel_horiz, METH_VARARGS|METH_KEYWORDS, 0},
  {"sobel_vert", (PyCFunction)__pyx_pw_9FastImage_12FastImage32f_43sobel_vert, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9FastImage_FastImage32f = {
  PyVarObject_HEAD_INIT(0, 0)
  "FastImage.FastImage32f", /*tp_name*/
  sizeof(struct __pyx_obj_9FastImage_FastImage32f), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9FastImage_FastImageBase, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_9FastImage_FastImageBase, /*tp_traverse*/
  __pyx_tp_clear_9FastImage_FastImageBase, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9FastImage_FastImage32f, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9FastImage_12FastImage32f_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9FastImage_FastImage32f, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};
static struct __pyx_vtabstruct_9FastImage_FastImage8u __pyx_vtable_9FastImage_FastImage8u;

static PyObject *__pyx_tp_new_9FastImage_FastImage8u(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_9FastImage_FastImage8u *p;
  PyObject *o = __pyx_tp_new_9FastImage_FastImageBase(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_9FastImage_FastImage8u *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_9FastImage_FastImageBase*)__pyx_vtabptr_9FastImage_FastImage8u;
  if (unlikely(__pyx_pw_9FastImage_11FastImage8u_1__cinit__(o, a, k) < 0)) goto bad;
  return o;
  bad:
  Py_DECREF(o); o = 0;
  return NULL;
}

static PyMethodDef __pyx_methods_9FastImage_FastImage8u[] = {
  {"set_val", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_5set_val, METH_VARARGS|METH_KEYWORDS, 0},
  {"set_val_masked", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_7set_val_masked, METH_VARARGS|METH_KEYWORDS, 0},
  {"get_32f_copy", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_9get_32f_copy, METH_O, 0},
  {"get_32f_copy_put", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_11get_32f_copy_put, METH_VARARGS|METH_KEYWORDS, 0},
  {"get_8u_copy", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_13get_8u_copy, METH_O, 0},
  {"get_8u_copy_put", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_15get_8u_copy_put, METH_VARARGS|METH_KEYWORDS, 0},
  {"get_absdiff_put", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_17get_absdiff_put, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9FastImage_11FastImage8u_16get_absdiff_put},
  {"get_sub_put", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_19get_sub_put, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9FastImage_11FastImage8u_18get_sub_put},
  {"toself_threshold", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_21toself_threshold, METH_VARARGS|METH_KEYWORDS, __pyx_doc_9FastImage_11FastImage8u_20toself_threshold},
  {"get_compare", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_23get_compare, METH_VARARGS|METH_KEYWORDS, 0},
  {"get_compare_put", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_25get_compare_put, METH_VARARGS|METH_KEYWORDS, 0},
  {"min_index", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_27min_index, METH_O, 0},
  {"max_index", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_29max_index, METH_O, 0},
  {"to_yuv422", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_31to_yuv422, METH_NOARGS, 0},
  {"dot", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_33dot, METH_VARARGS|METH_KEYWORDS, 0},
  {"sobel_horiz", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_35sobel_horiz, METH_VARARGS|METH_KEYWORDS, 0},
  {"sobel_vert", (PyCFunction)__pyx_pw_9FastImage_11FastImage8u_37sobel_vert, METH_VARARGS|METH_KEYWORDS, 0},
  {0, 0, 0, 0}
};

static PyTypeObject __pyx_type_9FastImage_FastImage8u = {
  PyVarObject_HEAD_INIT(0, 0)
  "FastImage.FastImage8u", /*tp_name*/
  sizeof(struct __pyx_obj_9FastImage_FastImage8u), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_9FastImage_FastImageBase, /*tp_dealloc*/
  0, /*tp_print*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "a fast 2D array of 8u", /*tp_doc*/
  __pyx_tp_traverse_9FastImage_FastImageBase, /*tp_traverse*/
  __pyx_tp_clear_9FastImage_FastImageBase, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_9FastImage_FastImage8u, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_9FastImage_11FastImage8u_3__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_9FastImage_FastImage8u, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
static struct PyModuleDef __pyx_moduledef = {
  #if PY_VERSION_HEX < 0x03020000
    { PyObject_HEAD_INIT(NULL) NULL, 0, NULL },
  #else
    PyModuleDef_HEAD_INIT,
  #endif
    "FastImage",
    0, /* m_doc */
    -1, /* m_size */
    __pyx_methods /* m_methods */,
    NULL, /* m_reload */
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_32f, __pyx_k_32f, sizeof(__pyx_k_32f), 0, 0, 1, 0},
  {&__pyx_kp_s_8u, __pyx_k_8u, sizeof(__pyx_k_8u), 0, 0, 1, 0},
  {&__pyx_n_s_CmpEq, __pyx_k_CmpEq, sizeof(__pyx_k_CmpEq), 0, 0, 1, 1},
  {&__pyx_n_s_CmpGreater, __pyx_k_CmpGreater, sizeof(__pyx_k_CmpGreater), 0, 0, 1, 1},
  {&__pyx_n_s_CmpGreaterEq, __pyx_k_CmpGreaterEq, sizeof(__pyx_k_CmpGreaterEq), 0, 0, 1, 1},
  {&__pyx_n_s_CmpLess, __pyx_k_CmpLess, sizeof(__pyx_k_CmpLess), 0, 0, 1, 1},
  {&__pyx_n_s_CmpLessEq, __pyx_k_CmpLessEq, sizeof(__pyx_k_CmpLessEq), 0, 0, 1, 1},
  {&__pyx_kp_s_Error_allocating_memory, __pyx_k_Error_allocating_memory, sizeof(__pyx_k_Error_allocating_memory), 0, 0, 1, 0},
  {&__pyx_n_s_FastImage, __pyx_k_FastImage, sizeof(__pyx_k_FastImage), 0, 0, 1, 1},
  {&__pyx_kp_s_FastImage_Point_d_d, __pyx_k_FastImage_Point_d_d, sizeof(__pyx_k_FastImage_Point_d_d), 0, 0, 1, 0},
  {&__pyx_kp_s_FastImage_Size_d_d, __pyx_k_FastImage_Size_d_d, sizeof(__pyx_k_FastImage_Size_d_d), 0, 0, 1, 0},
  {&__pyx_n_s_FicError, __pyx_k_FicError, sizeof(__pyx_k_FicError), 0, 0, 1, 1},
  {&__pyx_n_s_IppError, __pyx_k_IppError, sizeof(__pyx_k_IppError), 0, 0, 1, 1},
  {&__pyx_n_s_IppError___init, __pyx_k_IppError___init, sizeof(__pyx_k_IppError___init), 0, 0, 1, 1},
  {&__pyx_n_s_MemoryError, __pyx_k_MemoryError, sizeof(__pyx_k_MemoryError), 0, 0, 1, 1},
  {&__pyx_n_s_NotImplementedError, __pyx_k_NotImplementedError, sizeof(__pyx_k_NotImplementedError), 0, 0, 1, 1},
  {&__pyx_n_s_TypeError, __pyx_k_TypeError, sizeof(__pyx_k_TypeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Users_straw_Documents_src_motmo, __pyx_k_Users_straw_Documents_src_motmo, sizeof(__pyx_k_Users_straw_Documents_src_motmo), 0, 0, 1, 0},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_kp_s__10, __pyx_k__10, sizeof(__pyx_k__10), 0, 0, 1, 0},
  {&__pyx_kp_s__12, __pyx_k__12, sizeof(__pyx_k__12), 0, 0, 1, 0},
  {&__pyx_kp_s__13, __pyx_k__13, sizeof(__pyx_k__13), 0, 0, 1, 0},
  {&__pyx_kp_s__7, __pyx_k__7, sizeof(__pyx_k__7), 0, 0, 1, 0},
  {&__pyx_kp_s__8, __pyx_k__8, sizeof(__pyx_k__8), 0, 0, 1, 0},
  {&__pyx_n_s_alpha, __pyx_k_alpha, sizeof(__pyx_k_alpha), 0, 0, 1, 1},
  {&__pyx_n_s_append, __pyx_k_append, sizeof(__pyx_k_append), 0, 0, 1, 1},
  {&__pyx_n_s_arch_str, __pyx_k_arch_str, sizeof(__pyx_k_arch_str), 0, 0, 1, 1},
  {&__pyx_n_s_arr, __pyx_k_arr, sizeof(__pyx_k_arr), 0, 0, 1, 1},
  {&__pyx_n_s_array_struct, __pyx_k_array_struct, sizeof(__pyx_k_array_struct), 0, 0, 1, 1},
  {&__pyx_n_s_asfastimage, __pyx_k_asfastimage, sizeof(__pyx_k_asfastimage), 0, 0, 1, 1},
  {&__pyx_kp_s_asfastimage_calling___new, __pyx_k_asfastimage_calling___new, sizeof(__pyx_k_asfastimage_calling___new), 0, 0, 1, 0},
  {&__pyx_kp_s_assigning_ROI_from_self_im_x_to, __pyx_k_assigning_ROI_from_self_im_x_to, sizeof(__pyx_k_assigning_ROI_from_self_im_x_to), 0, 0, 1, 0},
  {&__pyx_n_s_attr, __pyx_k_attr, sizeof(__pyx_k_attr), 0, 0, 1, 1},
  {&__pyx_n_s_bottom, __pyx_k_bottom, sizeof(__pyx_k_bottom), 0, 0, 1, 1},
  {&__pyx_kp_s_can_only_compare_against_ints_an, __pyx_k_can_only_compare_against_ints_an, sizeof(__pyx_k_can_only_compare_against_ints_an), 0, 0, 1, 0},
  {&__pyx_kp_s_cannot_handle_arrays_with_descr, __pyx_k_cannot_handle_arrays_with_descr, sizeof(__pyx_k_cannot_handle_arrays_with_descr), 0, 0, 1, 0},
  {&__pyx_kp_s_cannot_handle_misaligned_arrays, __pyx_k_cannot_handle_misaligned_arrays, sizeof(__pyx_k_cannot_handle_misaligned_arrays), 0, 0, 1, 0},
  {&__pyx_kp_s_cannot_handle_misaligned_or_not, __pyx_k_cannot_handle_misaligned_or_not, sizeof(__pyx_k_cannot_handle_misaligned_or_not), 0, 0, 1, 0},
  {&__pyx_kp_s_cannot_handle_such_large_data, __pyx_k_cannot_handle_such_large_data, sizeof(__pyx_k_cannot_handle_such_large_data), 0, 0, 1, 0},
  {&__pyx_kp_s_cannot_perform_comparison_equali, __pyx_k_cannot_perform_comparison_equali, sizeof(__pyx_k_cannot_perform_comparison_equali), 0, 0, 1, 0},
  {&__pyx_n_s_chr, __pyx_k_chr, sizeof(__pyx_k_chr), 0, 0, 1, 1},
  {&__pyx_n_s_cmsg, __pyx_k_cmsg, sizeof(__pyx_k_cmsg), 0, 0, 1, 1},
  {&__pyx_n_s_compare_value, __pyx_k_compare_value, sizeof(__pyx_k_compare_value), 0, 0, 1, 1},
  {&__pyx_kp_s_comparison_between_Point_objects, __pyx_k_comparison_between_Point_objects, sizeof(__pyx_k_comparison_between_Point_objects), 0, 0, 1, 0},
  {&__pyx_kp_s_comparison_between_Size_objects, __pyx_k_comparison_between_Size_objects, sizeof(__pyx_k_comparison_between_Size_objects), 0, 0, 1, 0},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_kp_s_creating_ROI_viewer, __pyx_k_creating_ROI_viewer, sizeof(__pyx_k_creating_ROI_viewer), 0, 0, 1, 0},
  {&__pyx_n_s_data, __pyx_k_data, sizeof(__pyx_k_data), 0, 0, 1, 1},
  {&__pyx_n_s_dest, __pyx_k_dest, sizeof(__pyx_k_dest), 0, 0, 1, 1},
  {&__pyx_n_s_doc, __pyx_k_doc, sizeof(__pyx_k_doc), 0, 0, 1, 1},
  {&__pyx_n_s_dtype, __pyx_k_dtype, sizeof(__pyx_k_dtype), 0, 0, 1, 1},
  {&__pyx_n_s_empty, __pyx_k_empty, sizeof(__pyx_k_empty), 0, 0, 1, 1},
  {&__pyx_n_s_end, __pyx_k_end, sizeof(__pyx_k_end), 0, 0, 1, 1},
  {&__pyx_kp_s_error_d_s, __pyx_k_error_d_s, sizeof(__pyx_k_error_d_s), 0, 0, 1, 0},
  {&__pyx_n_s_errval, __pyx_k_errval, sizeof(__pyx_k_errval), 0, 0, 1, 1},
  {&__pyx_n_s_file, __pyx_k_file, sizeof(__pyx_k_file), 0, 0, 1, 1},
  {&__pyx_n_s_float32, __pyx_k_float32, sizeof(__pyx_k_float32), 0, 0, 1, 1},
  {&__pyx_n_s_flush, __pyx_k_flush, sizeof(__pyx_k_flush), 0, 0, 1, 1},
  {&__pyx_kp_s_freeing_memory_x, __pyx_k_freeing_memory_x, sizeof(__pyx_k_freeing_memory_x), 0, 0, 1, 0},
  {&__pyx_n_s_get_8u_copy, __pyx_k_get_8u_copy, sizeof(__pyx_k_get_8u_copy), 0, 0, 1, 1},
  {&__pyx_n_s_get_IPP_arch, __pyx_k_get_IPP_arch, sizeof(__pyx_k_get_IPP_arch), 0, 0, 1, 1},
  {&__pyx_n_s_get_IPP_static, __pyx_k_get_IPP_static, sizeof(__pyx_k_get_IPP_static), 0, 0, 1, 1},
  {&__pyx_n_s_get_IPP_version, __pyx_k_get_IPP_version, sizeof(__pyx_k_get_IPP_version), 0, 0, 1, 1},
  {&__pyx_n_s_get_compare_put, __pyx_k_get_compare_put, sizeof(__pyx_k_get_compare_put), 0, 0, 1, 1},
  {&__pyx_kp_s_got_mem_x_step_d, __pyx_k_got_mem_x_step_d, sizeof(__pyx_k_got_mem_x_step_d), 0, 0, 1, 0},
  {&__pyx_n_s_height, __pyx_k_height, sizeof(__pyx_k_height), 0, 0, 1, 1},
  {&__pyx_n_s_i, __pyx_k_i, sizeof(__pyx_k_i), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_inter, __pyx_k_inter, sizeof(__pyx_k_inter), 0, 0, 1, 1},
  {&__pyx_kp_s_invalid___array_struct, __pyx_k_invalid___array_struct, sizeof(__pyx_k_invalid___array_struct), 0, 0, 1, 0},
  {&__pyx_n_s_join, __pyx_k_join, sizeof(__pyx_k_join), 0, 0, 1, 1},
  {&__pyx_n_s_left, __pyx_k_left, sizeof(__pyx_k_left), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_map, __pyx_k_map, sizeof(__pyx_k_map), 0, 0, 1, 1},
  {&__pyx_n_s_mask, __pyx_k_mask, sizeof(__pyx_k_mask), 0, 0, 1, 1},
  {&__pyx_n_s_metaclass, __pyx_k_metaclass, sizeof(__pyx_k_metaclass), 0, 0, 1, 1},
  {&__pyx_n_s_module, __pyx_k_module, sizeof(__pyx_k_module), 0, 0, 1, 1},
  {&__pyx_kp_s_need_file_object, __pyx_k_need_file_object, sizeof(__pyx_k_need_file_object), 0, 0, 1, 0},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_new_value, __pyx_k_new_value, sizeof(__pyx_k_new_value), 0, 0, 1, 1},
  {&__pyx_n_s_numpy, __pyx_k_numpy, sizeof(__pyx_k_numpy), 0, 0, 1, 1},
  {&__pyx_n_s_ones_like, __pyx_k_ones_like, sizeof(__pyx_k_ones_like), 0, 0, 1, 1},
  {&__pyx_kp_s_only_2D_arrays_may_be_copied_to, __pyx_k_only_2D_arrays_may_be_copied_to, sizeof(__pyx_k_only_2D_arrays_may_be_copied_to), 0, 0, 1, 0},
  {&__pyx_kp_s_only_2D_arrays_may_be_viewed_as, __pyx_k_only_2D_arrays_may_be_viewed_as, sizeof(__pyx_k_only_2D_arrays_may_be_viewed_as), 0, 0, 1, 0},
  {&__pyx_kp_s_only_2_and_0_5_powers_supported, __pyx_k_only_2_and_0_5_powers_supported, sizeof(__pyx_k_only_2_and_0_5_powers_supported), 0, 0, 1, 0},
  {&__pyx_n_s_op, __pyx_k_op, sizeof(__pyx_k_op), 0, 0, 1, 1},
  {&__pyx_n_s_other, __pyx_k_other, sizeof(__pyx_k_other), 0, 0, 1, 1},
  {&__pyx_n_s_prepare, __pyx_k_prepare, sizeof(__pyx_k_prepare), 0, 0, 1, 1},
  {&__pyx_n_s_print, __pyx_k_print, sizeof(__pyx_k_print), 0, 0, 1, 1},
  {&__pyx_n_s_print_stack, __pyx_k_print_stack, sizeof(__pyx_k_print_stack), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_qualname, __pyx_k_qualname, sizeof(__pyx_k_qualname), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_kp_s_requesting_32f_memory_size_W_d_H, __pyx_k_requesting_32f_memory_size_W_d_H, sizeof(__pyx_k_requesting_32f_memory_size_W_d_H), 0, 0, 1, 0},
  {&__pyx_kp_s_requesting_memory_size_W_d_H_d, __pyx_k_requesting_memory_size_W_d_H_d, sizeof(__pyx_k_requesting_memory_size_W_d_H_d), 0, 0, 1, 0},
  {&__pyx_n_s_result, __pyx_k_result, sizeof(__pyx_k_result), 0, 0, 1, 1},
  {&__pyx_kp_s_result_2, __pyx_k_result_2, sizeof(__pyx_k_result_2), 0, 0, 1, 0},
  {&__pyx_kp_s_result_view, __pyx_k_result_view, sizeof(__pyx_k_result_view), 0, 0, 1, 0},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_kp_s_self_step_is_d_width_d_height_d, __pyx_k_self_step_is_d_width_d_height_d, sizeof(__pyx_k_self_step_is_d_width_d_height_d), 0, 0, 1, 0},
  {&__pyx_n_s_set_debug, __pyx_k_set_debug, sizeof(__pyx_k_set_debug), 0, 0, 1, 1},
  {&__pyx_n_s_shape, __pyx_k_shape, sizeof(__pyx_k_shape), 0, 0, 1, 1},
  {&__pyx_n_s_show_mem_and_step, __pyx_k_show_mem_and_step, sizeof(__pyx_k_show_mem_and_step), 0, 0, 1, 1},
  {&__pyx_n_s_size, __pyx_k_size, sizeof(__pyx_k_size), 0, 0, 1, 1},
  {&__pyx_n_s_stderr, __pyx_k_stderr, sizeof(__pyx_k_stderr), 0, 0, 1, 1},
  {&__pyx_n_s_strides, __pyx_k_strides, sizeof(__pyx_k_strides), 0, 0, 1, 1},
  {&__pyx_n_s_sys, __pyx_k_sys, sizeof(__pyx_k_sys), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_toself_sqrt, __pyx_k_toself_sqrt, sizeof(__pyx_k_toself_sqrt), 0, 0, 1, 1},
  {&__pyx_n_s_toself_square, __pyx_k_toself_square, sizeof(__pyx_k_toself_square), 0, 0, 1, 1},
  {&__pyx_n_s_traceback, __pyx_k_traceback, sizeof(__pyx_k_traceback), 0, 0, 1, 1},
  {&__pyx_kp_s_type_not_supported, __pyx_k_type_not_supported, sizeof(__pyx_k_type_not_supported), 0, 0, 1, 0},
  {&__pyx_kp_s_typekind_s_itemsize_d_not_suppor, __pyx_k_typekind_s_itemsize_d_not_suppor, sizeof(__pyx_k_typekind_s_itemsize_d_not_suppor), 0, 0, 1, 0},
  {&__pyx_n_s_typestr, __pyx_k_typestr, sizeof(__pyx_k_typestr), 0, 0, 1, 1},
  {&__pyx_kp_s_u1, __pyx_k_u1, sizeof(__pyx_k_u1), 0, 0, 1, 0},
  {&__pyx_kp_s_unknown_basetype_r, __pyx_k_unknown_basetype_r, sizeof(__pyx_k_unknown_basetype_r), 0, 0, 1, 0},
  {&__pyx_n_s_val, __pyx_k_val, sizeof(__pyx_k_val), 0, 0, 1, 1},
  {&__pyx_n_s_width, __pyx_k_width, sizeof(__pyx_k_width), 0, 0, 1, 1},
  {&__pyx_n_s_write, __pyx_k_write, sizeof(__pyx_k_write), 0, 0, 1, 1},
  {&__pyx_n_s_x, __pyx_k_x, sizeof(__pyx_k_x), 0, 0, 1, 1},
  {&__pyx_n_s_y, __pyx_k_y, sizeof(__pyx_k_y), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 22, __pyx_L1_error)
  __pyx_builtin_TypeError = __Pyx_GetBuiltinName(__pyx_n_s_TypeError); if (!__pyx_builtin_TypeError) __PYX_ERR(0, 134, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 144, __pyx_L1_error)
  __pyx_builtin_map = __Pyx_GetBuiltinName(__pyx_n_s_map); if (!__pyx_builtin_map) __PYX_ERR(0, 221, __pyx_L1_error)
  __pyx_builtin_NotImplementedError = __Pyx_GetBuiltinName(__pyx_n_s_NotImplementedError); if (!__pyx_builtin_NotImplementedError) __PYX_ERR(0, 280, __pyx_L1_error)
  __pyx_builtin_MemoryError = __Pyx_GetBuiltinName(__pyx_n_s_MemoryError); if (!__pyx_builtin_MemoryError) __PYX_ERR(0, 360, __pyx_L1_error)
  __pyx_builtin_chr = __Pyx_GetBuiltinName(__pyx_n_s_chr); if (!__pyx_builtin_chr) __PYX_ERR(0, 818, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "FastImage.pyx":134
 *         cdef Size ys
 *         if type(x) != Size:
 *             raise TypeError("comparison between Size objects only")             # <<<<<<<<<<<<<<
 *         if type(y) != Size:
 *             raise TypeError("comparison between Size objects only")
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_kp_s_comparison_between_Size_objects); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 134, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "FastImage.pyx":136
 *             raise TypeError("comparison between Size objects only")
 *         if type(y) != Size:
 *             raise TypeError("comparison between Size objects only")             # <<<<<<<<<<<<<<
 *         xs = x
 *         ys = y
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_comparison_between_Size_objects); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "FastImage.pyx":144
 *             return not (xs.sz.width == ys.sz.width and xs.sz.height == ys.sz.height)
 *         else:
 *             raise ValueError("cannot perform comparison (equality testing only)")             # <<<<<<<<<<<<<<
 * 
 *     # leave as short names to make life easy when looking for .sz.width (fast C) vs. .w (slow Python)
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_cannot_perform_comparison_equali); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "FastImage.pyx":165
 *         cdef Point bp
 *         if type(a) != Point:
 *             raise TypeError("comparison between Point objects only")             # <<<<<<<<<<<<<<
 *         if type(b) != Point:
 *             raise TypeError("comparison between Point objects only")
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_kp_s_comparison_between_Point_objects); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 165, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "FastImage.pyx":167
 *             raise TypeError("comparison between Point objects only")
 *         if type(b) != Point:
 *             raise TypeError("comparison between Point objects only")             # <<<<<<<<<<<<<<
 *         ap = a
 *         bp = b
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_kp_s_comparison_between_Point_objects); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 167, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "FastImage.pyx":175
 *             return not (ap.pt.x == bp.pt.x and ap.pt.y == bp.pt.y)
 *         else:
 *             raise ValueError("cannot perform comparison (equality testing only)")             # <<<<<<<<<<<<<<
 * 
 *     # leave as short names to make life easy when looking for .sz.width (fast C) vs. .w (slow Python)
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_kp_s_cannot_perform_comparison_equali); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 175, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "FastImage.pyx":237
 *         bytes_per_pixel = self.strides[1]
 *         if not c_python.PyFile_Check(fobj):
 *             raise ValueError('need file object')             # <<<<<<<<<<<<<<
 *         fd = c_python.PyFile_AsFile(fobj)
 *         with nogil:
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_kp_s_need_file_object); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 237, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "FastImage.pyx":280
 *                 inter['typestr'] = '|u1'
 *             else:
 *                 raise NotImplementedError('')             # <<<<<<<<<<<<<<
 *             readonly = True
 *             data_ptr_int = <intptr_t>self.im
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_kp_s__10); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 280, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "FastImage.pyx":362
 *             raise MemoryError('self.step is %d (width=%d, height=%d)'%(
 *                 self.step, size.sz.width, size.sz.height))
 *         if self.im==NULL: raise MemoryError("Error allocating memory")             # <<<<<<<<<<<<<<
 *         self.strides[0] = self.step
 *         if FASTIMAGEDEBUG>=1:
 */
  __pyx_tuple__14 = PyTuple_Pack(1, __pyx_kp_s_Error_allocating_memory); if (unlikely(!__pyx_tuple__14)) __PYX_ERR(0, 362, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__14);
  __Pyx_GIVEREF(__pyx_tuple__14);

  /* "FastImage.pyx":465
 *                                                  size.sz,<ipp.IppCmpOp>op))
 *         else:
 *             raise TypeError("can only compare against ints and FastImage8u types")             # <<<<<<<<<<<<<<
 * 
 *     cdef void fast_get_compare_int_put_greater( self, int other_int, FastImage8u dest, Size size):
 */
  __pyx_tuple__15 = PyTuple_Pack(1, __pyx_kp_s_can_only_compare_against_ints_an); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(0, 465, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);

  /* "FastImage.pyx":559
 *                                                              self.imsiz.sz.height)
 *         self.im=ipp.ippiMalloc_32f_C1( size.sz.width, size.sz.height, &self.step )
 *         if self.im==NULL: raise MemoryError("Error allocating memory")             # <<<<<<<<<<<<<<
 *         self.strides[0] = self.step
 *         if FASTIMAGEDEBUG>=1:
 */
  __pyx_tuple__16 = PyTuple_Pack(1, __pyx_kp_s_Error_allocating_memory); if (unlikely(!__pyx_tuple__16)) __PYX_ERR(0, 559, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__16);
  __Pyx_GIVEREF(__pyx_tuple__16);

  /* "FastImage.pyx":579
 *                                                size.sz))
 *         else:
 *             raise ValueError("type not supported")             # <<<<<<<<<<<<<<
 * 
 *     def toself_add_weighted(self, FastImageBase other, Size size, float alpha):
 */
  __pyx_tuple__17 = PyTuple_Pack(1, __pyx_kp_s_type_not_supported); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(0, 579, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);

  /* "FastImage.pyx":592
 *                                                        size.sz, alpha))
 *         else:
 *             raise ValueError("type not supported")             # <<<<<<<<<<<<<<
 * 
 *     cdef void fast_toself_add_weighted_8u( self, FastImage8u other, Size size, float alpha):
 */
  __pyx_tuple__18 = PyTuple_Pack(1, __pyx_kp_s_type_not_supported); if (unlikely(!__pyx_tuple__18)) __PYX_ERR(0, 592, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__18);
  __Pyx_GIVEREF(__pyx_tuple__18);

  /* "FastImage.pyx":715
 *             xself.toself_sqrt(xself.imsiz)
 *         else:
 *             raise ValueError("only 2 and 0.5 powers supported")             # <<<<<<<<<<<<<<
 *         return xself
 * 
 */
  __pyx_tuple__19 = PyTuple_Pack(1, __pyx_kp_s_only_2_and_0_5_powers_supported); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(0, 715, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);

  /* "FastImage.pyx":752
 *             dtype = numpy.float32
 *             tmp = numpy.empty( shape, dtype=dtype)
 *             tmp = tmp[:,:size.sz.width]             # <<<<<<<<<<<<<<
 *             dest = asfastimage(tmp)
 *         return dest
 */
  __pyx_slice__20 = PySlice_New(Py_None, Py_None, Py_None); if (unlikely(!__pyx_slice__20)) __PYX_ERR(0, 752, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_slice__20);
  __Pyx_GIVEREF(__pyx_slice__20);

  /* "FastImage.pyx":792
 *     attr = arr.__array_struct__
 *     if not c_python.PyCObject_Check(attr):
 *         raise ValueError("invalid __array_struct__")             # <<<<<<<<<<<<<<
 * 
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)
 */
  __pyx_tuple__21 = PyTuple_Pack(1, __pyx_kp_s_invalid___array_struct); if (unlikely(!__pyx_tuple__21)) __PYX_ERR(0, 792, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__21);
  __Pyx_GIVEREF(__pyx_tuple__21);

  /* "FastImage.pyx":796
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)
 *     if inter.two != 2:
 *         raise ValueError("invalid __array_struct__")             # <<<<<<<<<<<<<<
 * 
 *     if (inter.flags & ARR_HAS_DESCR) == (ARR_HAS_DESCR):
 */
  __pyx_tuple__22 = PyTuple_Pack(1, __pyx_kp_s_invalid___array_struct); if (unlikely(!__pyx_tuple__22)) __PYX_ERR(0, 796, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__22);
  __Pyx_GIVEREF(__pyx_tuple__22);

  /* "FastImage.pyx":799
 * 
 *     if (inter.flags & ARR_HAS_DESCR) == (ARR_HAS_DESCR):
 *         raise ValueError("cannot handle arrays with descr")             # <<<<<<<<<<<<<<
 * 
 *     # TODO: don't really know what these flags all mean, figure out if
 */
  __pyx_tuple__23 = PyTuple_Pack(1, __pyx_kp_s_cannot_handle_arrays_with_descr); if (unlikely(!__pyx_tuple__23)) __PYX_ERR(0, 799, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__23);
  __Pyx_GIVEREF(__pyx_tuple__23);

  /* "FastImage.pyx":804
 *     # this is OK:
 *     if (inter.flags & (ALIGNED | WRITEABLE)) != (ALIGNED | WRITEABLE):
 *         raise ValueError("cannot handle misaligned or not writeable arrays.")             # <<<<<<<<<<<<<<
 * 
 *     if inter.nd != 2:
 */
  __pyx_tuple__24 = PyTuple_Pack(1, __pyx_kp_s_cannot_handle_misaligned_or_not); if (unlikely(!__pyx_tuple__24)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__24);
  __Pyx_GIVEREF(__pyx_tuple__24);

  /* "FastImage.pyx":807
 * 
 *     if inter.nd != 2:
 *         raise ValueError("only 2D arrays may be viewed as a FastImage")             # <<<<<<<<<<<<<<
 * 
 *     if FASTIMAGEDEBUG>=2:
 */
  __pyx_tuple__25 = PyTuple_Pack(1, __pyx_kp_s_only_2D_arrays_may_be_viewed_as); if (unlikely(!__pyx_tuple__25)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__25);
  __Pyx_GIVEREF(__pyx_tuple__25);

  /* "FastImage.pyx":820
 *         raise ValueError('typekind "%s", itemsize %d not supported'%(chr(inter.typekind),inter.itemsize))
 *     if (inter.shape[0] > INT_MAX) or (inter.shape[1] > INT_MAX) or (inter.strides[0] > INT_MAX):
 *         raise ValueError("cannot handle such large data")             # <<<<<<<<<<<<<<
 *     FastImageBase.__init__(result, Size(inter.shape[1],inter.shape[0])) # run common __init__
 *     result.strides[0] = inter.strides[0]
 */
  __pyx_tuple__26 = PyTuple_Pack(1, __pyx_kp_s_cannot_handle_such_large_data); if (unlikely(!__pyx_tuple__26)) __PYX_ERR(0, 820, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__26);
  __Pyx_GIVEREF(__pyx_tuple__26);

  /* "FastImage.pyx":842
 *     attr = arr.__array_struct__
 *     if not c_python.PyCObject_Check(attr):
 *         raise ValueError("invalid __array_struct__")             # <<<<<<<<<<<<<<
 * 
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)
 */
  __pyx_tuple__27 = PyTuple_Pack(1, __pyx_kp_s_invalid___array_struct); if (unlikely(!__pyx_tuple__27)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__27);
  __Pyx_GIVEREF(__pyx_tuple__27);

  /* "FastImage.pyx":846
 *     inter = <PyArrayInterface*>c_python.PyCObject_AsVoidPtr(attr)
 *     if inter.two != 2:
 *         raise ValueError("invalid __array_struct__")             # <<<<<<<<<<<<<<
 * 
 *     if (inter.flags & ARR_HAS_DESCR) == (ARR_HAS_DESCR):
 */
  __pyx_tuple__28 = PyTuple_Pack(1, __pyx_kp_s_invalid___array_struct); if (unlikely(!__pyx_tuple__28)) __PYX_ERR(0, 846, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__28);
  __Pyx_GIVEREF(__pyx_tuple__28);

  /* "FastImage.pyx":849
 * 
 *     if (inter.flags & ARR_HAS_DESCR) == (ARR_HAS_DESCR):
 *         raise ValueError("cannot handle arrays with descr")             # <<<<<<<<<<<<<<
 * 
 *     # TODO: don't really know what these flags all mean, figure out if
 */
  __pyx_tuple__29 = PyTuple_Pack(1, __pyx_kp_s_cannot_handle_arrays_with_descr); if (unlikely(!__pyx_tuple__29)) __PYX_ERR(0, 849, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__29);
  __Pyx_GIVEREF(__pyx_tuple__29);

  /* "FastImage.pyx":854
 *     # this is OK:
 *     if (inter.flags & (ALIGNED)) != (ALIGNED):
 *         raise ValueError("cannot handle misaligned arrays.")             # <<<<<<<<<<<<<<
 * 
 *     if inter.nd != 2:
 */
  __pyx_tuple__30 = PyTuple_Pack(1, __pyx_kp_s_cannot_handle_misaligned_arrays); if (unlikely(!__pyx_tuple__30)) __PYX_ERR(0, 854, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__30);
  __Pyx_GIVEREF(__pyx_tuple__30);

  /* "FastImage.pyx":857
 * 
 *     if inter.nd != 2:
 *         raise ValueError("only 2D arrays may be copied to a FastImage")             # <<<<<<<<<<<<<<
 * 
 *     if (inter.shape[0] > INT_MAX) or (inter.shape[1] > INT_MAX) or (inter.strides[0] > INT_MAX):
 */
  __pyx_tuple__31 = PyTuple_Pack(1, __pyx_kp_s_only_2D_arrays_may_be_copied_to); if (unlikely(!__pyx_tuple__31)) __PYX_ERR(0, 857, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__31);
  __Pyx_GIVEREF(__pyx_tuple__31);

  /* "FastImage.pyx":860
 * 
 *     if (inter.shape[0] > INT_MAX) or (inter.shape[1] > INT_MAX) or (inter.strides[0] > INT_MAX):
 *         raise ValueError("cannot handle such large data")             # <<<<<<<<<<<<<<
 * 
 *     if inter.typekind == CHAR_u and inter.itemsize==1:
 */
  __pyx_tuple__32 = PyTuple_Pack(1, __pyx_kp_s_cannot_handle_such_large_data); if (unlikely(!__pyx_tuple__32)) __PYX_ERR(0, 860, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__32);
  __Pyx_GIVEREF(__pyx_tuple__32);

  /* "FastImage.pyx":22
 * n_hist_levels = 256
 * cdef ipp.Ipp32s hist_levels[257]
 * for i in range(0,257):             # <<<<<<<<<<<<<<
 *     hist_levels[i]=i
 * 
 */
  __pyx_tuple__33 = PyTuple_Pack(2, __pyx_int_0, __pyx_int_257); if (unlikely(!__pyx_tuple__33)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__33);
  __Pyx_GIVEREF(__pyx_tuple__33);

  /* "FastImage.pyx":25
 *     hist_levels[i]=i
 * 
 * def set_debug(int val):             # <<<<<<<<<<<<<<
 *     global FASTIMAGEDEBUG
 *     FASTIMAGEDEBUG = val
 */
  __pyx_tuple__34 = PyTuple_Pack(2, __pyx_n_s_val, __pyx_n_s_val); if (unlikely(!__pyx_tuple__34)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__34);
  __Pyx_GIVEREF(__pyx_tuple__34);
  __pyx_codeobj__35 = (PyObject*)__Pyx_PyCode_New(1, 0, 2, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__34, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_Users_straw_Documents_src_motmo, __pyx_n_s_set_debug, 25, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__35)) __PYX_ERR(0, 25, __pyx_L1_error)

  /* "FastImage.pyx":65
 * InitIPP()
 * 
 * def get_IPP_version():             # <<<<<<<<<<<<<<
 *     cdef ipp_version_struct_t val
 *     val = GetIPPVersion()
 */
  __pyx_tuple__36 = PyTuple_Pack(1, __pyx_n_s_val); if (unlikely(!__pyx_tuple__36)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__36);
  __Pyx_GIVEREF(__pyx_tuple__36);
  __pyx_codeobj__37 = (PyObject*)__Pyx_PyCode_New(0, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__36, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_Users_straw_Documents_src_motmo, __pyx_n_s_get_IPP_version, 65, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__37)) __PYX_ERR(0, 65, __pyx_L1_error)

  /* "FastImage.pyx":70
 *     return val.major, val.minor, val.build
 * 
 * def get_IPP_arch():             # <<<<<<<<<<<<<<
 *     cdef const char* arch_str
 *     arch_str = GetIPPArch()
 */
  __pyx_tuple__38 = PyTuple_Pack(1, __pyx_n_s_arch_str); if (unlikely(!__pyx_tuple__38)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__38);
  __Pyx_GIVEREF(__pyx_tuple__38);
  __pyx_codeobj__39 = (PyObject*)__Pyx_PyCode_New(0, 0, 1, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__38, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_Users_straw_Documents_src_motmo, __pyx_n_s_get_IPP_arch, 70, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__39)) __PYX_ERR(0, 70, __pyx_L1_error)

  /* "FastImage.pyx":75
 *     return arch_str
 * 
 * def get_IPP_static():             # <<<<<<<<<<<<<<
 *     return IsIPPStatic()
 * 
 */
  __pyx_codeobj__40 = (PyObject*)__Pyx_PyCode_New(0, 0, 0, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_Users_straw_Documents_src_motmo, __pyx_n_s_get_IPP_static, 75, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__40)) __PYX_ERR(0, 75, __pyx_L1_error)

  /* "FastImage.pyx":79
 * 
 * class IppError(Exception):
 *     def __init__(self, int errval):             # <<<<<<<<<<<<<<
 *         cdef const char* cmsg
 *         cmsg = ippGetStatusString(<ipp.IppStatus>errval)
 */
  __pyx_tuple__41 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_errval, __pyx_n_s_cmsg); if (unlikely(!__pyx_tuple__41)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__41);
  __Pyx_GIVEREF(__pyx_tuple__41);
  __pyx_codeobj__42 = (PyObject*)__Pyx_PyCode_New(2, 0, 3, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__41, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_Users_straw_Documents_src_motmo, __pyx_n_s_init, 79, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__42)) __PYX_ERR(0, 79, __pyx_L1_error)

  /* "FastImage.pyx":783
 *         return dest
 * 
 * def asfastimage( object arr ):             # <<<<<<<<<<<<<<
 *     """return a FastImage view of an array"""
 *     cdef PyArrayInterface* inter
 */
  __pyx_tuple__43 = PyTuple_Pack(4, __pyx_n_s_arr, __pyx_n_s_inter, __pyx_n_s_result, __pyx_n_s_attr); if (unlikely(!__pyx_tuple__43)) __PYX_ERR(0, 783, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__43);
  __Pyx_GIVEREF(__pyx_tuple__43);
  __pyx_codeobj__44 = (PyObject*)__Pyx_PyCode_New(1, 0, 4, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__43, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_Users_straw_Documents_src_motmo, __pyx_n_s_asfastimage, 783, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__44)) __PYX_ERR(0, 783, __pyx_L1_error)

  /* "FastImage.pyx":834
 *     return result
 * 
 * def copy( object arr ):             # <<<<<<<<<<<<<<
 *     """return a FastImage copy of an array"""
 *     cdef PyArrayInterface* inter
 */
  __pyx_tuple__45 = PyTuple_Pack(5, __pyx_n_s_arr, __pyx_n_s_inter, __pyx_n_s_result, __pyx_n_s_i, __pyx_n_s_attr); if (unlikely(!__pyx_tuple__45)) __PYX_ERR(0, 834, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__45);
  __Pyx_GIVEREF(__pyx_tuple__45);
  __pyx_codeobj__46 = (PyObject*)__Pyx_PyCode_New(1, 0, 5, 0, 0, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__45, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_Users_straw_Documents_src_motmo, __pyx_n_s_copy, 834, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__46)) __PYX_ERR(0, 834, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_float_0_5 = PyFloat_FromDouble(0.5); if (unlikely(!__pyx_float_0_5)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_20 = PyInt_FromLong(20); if (unlikely(!__pyx_int_20)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_128 = PyInt_FromLong(128); if (unlikely(!__pyx_int_128)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_257 = PyInt_FromLong(257); if (unlikely(!__pyx_int_257)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

#if PY_MAJOR_VERSION < 3
PyMODINIT_FUNC initFastImage(void); /*proto*/
PyMODINIT_FUNC initFastImage(void)
#else
PyMODINIT_FUNC PyInit_FastImage(void); /*proto*/
PyMODINIT_FUNC PyInit_FastImage(void)
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *(*__pyx_t_4)(PyObject *);
  Ipp32s __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  __Pyx_RefNannyDeclarations
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
      PyErr_Clear();
      __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
      if (!__Pyx_RefNanny)
          Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  __Pyx_RefNannySetupContext("PyMODINIT_FUNC PyInit_FastImage(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  #ifdef WITH_THREAD /* Python build with threading support? */
  PyEval_InitThreads();
  #endif
  #endif
  /*--- Module creation code ---*/
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("FastImage", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  #if CYTHON_COMPILING_IN_PYPY
  Py_INCREF(__pyx_b);
  #endif
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_FastImage) {
    if (PyObject_SetAttrString(__pyx_m, "__name__", __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "FastImage")) {
      if (unlikely(PyDict_SetItemString(modules, "FastImage", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global init code ---*/
  /*--- Variable export code ---*/
  /*--- Function export code ---*/
  /*--- Type init code ---*/
  if (PyType_Ready(&__pyx_type_9FastImage_Size) < 0) __PYX_ERR(0, 119, __pyx_L1_error)
  __pyx_type_9FastImage_Size.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "Size", (PyObject *)&__pyx_type_9FastImage_Size) < 0) __PYX_ERR(0, 119, __pyx_L1_error)
  __pyx_ptype_9FastImage_Size = &__pyx_type_9FastImage_Size;
  if (PyType_Ready(&__pyx_type_9FastImage_Point) < 0) __PYX_ERR(0, 154, __pyx_L1_error)
  __pyx_type_9FastImage_Point.tp_print = 0;
  if (PyObject_SetAttrString(__pyx_m, "Point", (PyObject *)&__pyx_type_9FastImage_Point) < 0) __PYX_ERR(0, 154, __pyx_L1_error)
  __pyx_ptype_9FastImage_Point = &__pyx_type_9FastImage_Point;
  __pyx_vtabptr_9FastImage_FastImageBase = &__pyx_vtable_9FastImage_FastImageBase;
  __pyx_vtable_9FastImage_FastImageBase.c_roi = (struct __pyx_obj_9FastImage_FastImageBase *(*)(struct __pyx_obj_9FastImage_FastImageBase *, int, int, struct __pyx_obj_9FastImage_Size *))__pyx_f_9FastImage_13FastImageBase_c_roi;
  if (PyType_Ready(&__pyx_type_9FastImage_FastImageBase) < 0) __PYX_ERR(0, 185, __pyx_L1_error)
  __pyx_type_9FastImage_FastImageBase.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_9FastImage_FastImageBase.tp_dict, __pyx_vtabptr_9FastImage_FastImageBase) < 0) __PYX_ERR(0, 185, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "FastImageBase", (PyObject *)&__pyx_type_9FastImage_FastImageBase) < 0) __PYX_ERR(0, 185, __pyx_L1_error)
  __pyx_ptype_9FastImage_FastImageBase = &__pyx_type_9FastImage_FastImageBase;
  __pyx_vtabptr_9FastImage_FastImage32f = &__pyx_vtable_9FastImage_FastImage32f;
  __pyx_vtable_9FastImage_FastImage32f.__pyx_base = *__pyx_vtabptr_9FastImage_FastImageBase;
  __pyx_vtable_9FastImage_FastImage32f.fast_toself_add_weighted_8u = (void (*)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_Size *, float))__pyx_f_9FastImage_12FastImage32f_fast_toself_add_weighted_8u;
  __pyx_vtable_9FastImage_FastImage32f.fast_toself_add_weighted_32f = (void (*)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_Size *, float))__pyx_f_9FastImage_12FastImage32f_fast_toself_add_weighted_32f;
  __pyx_vtable_9FastImage_FastImage32f.fast_get_8u_copy_put = (void (*)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_Size *))__pyx_f_9FastImage_12FastImage32f_fast_get_8u_copy_put;
  __pyx_vtable_9FastImage_FastImage32f.fast_toself_square = (void (*)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_Size *))__pyx_f_9FastImage_12FastImage32f_fast_toself_square;
  __pyx_vtable_9FastImage_FastImage32f.fast_get_square_put = (void (*)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_Size *))__pyx_f_9FastImage_12FastImage32f_fast_get_square_put;
  __pyx_vtable_9FastImage_FastImage32f.fast_get_subtracted_put = (void (*)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_Size *))__pyx_f_9FastImage_12FastImage32f_fast_get_subtracted_put;
  __pyx_vtable_9FastImage_FastImage32f.fast_toself_multiply = (void (*)(struct __pyx_obj_9FastImage_FastImage32f *, float, struct __pyx_obj_9FastImage_Size *))__pyx_f_9FastImage_12FastImage32f_fast_toself_multiply;
  __pyx_vtable_9FastImage_FastImage32f.create_equal_shape_empty = (struct __pyx_obj_9FastImage_FastImage32f *(*)(struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_Size *))__pyx_f_9FastImage_12FastImage32f_create_equal_shape_empty;
  __pyx_type_9FastImage_FastImage32f.tp_base = __pyx_ptype_9FastImage_FastImageBase;
  if (PyType_Ready(&__pyx_type_9FastImage_FastImage32f) < 0) __PYX_ERR(0, 545, __pyx_L1_error)
  __pyx_type_9FastImage_FastImage32f.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_9FastImage_FastImage32f.tp_dict, __pyx_vtabptr_9FastImage_FastImage32f) < 0) __PYX_ERR(0, 545, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "FastImage32f", (PyObject *)&__pyx_type_9FastImage_FastImage32f) < 0) __PYX_ERR(0, 545, __pyx_L1_error)
  __pyx_ptype_9FastImage_FastImage32f = &__pyx_type_9FastImage_FastImage32f;
  __pyx_vtabptr_9FastImage_FastImage8u = &__pyx_vtable_9FastImage_FastImage8u;
  __pyx_vtable_9FastImage_FastImage8u.__pyx_base = *__pyx_vtabptr_9FastImage_FastImageBase;
  __pyx_vtable_9FastImage_FastImage8u.fast_get_absdiff_put = (void (*)(struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_Size *))__pyx_f_9FastImage_11FastImage8u_fast_get_absdiff_put;
  __pyx_vtable_9FastImage_FastImage8u.fast_get_sub_put = (void (*)(struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_Size *))__pyx_f_9FastImage_11FastImage8u_fast_get_sub_put;
  __pyx_vtable_9FastImage_FastImage8u.fast_set_val_masked = (void (*)(struct __pyx_obj_9FastImage_FastImage8u *, Ipp8u, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_Size *))__pyx_f_9FastImage_11FastImage8u_fast_set_val_masked;
  __pyx_vtable_9FastImage_FastImage8u.fast_get_32f_copy_put = (void (*)(struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_FastImage32f *, struct __pyx_obj_9FastImage_Size *))__pyx_f_9FastImage_11FastImage8u_fast_get_32f_copy_put;
  __pyx_vtable_9FastImage_FastImage8u.fast_get_compare_int_put_greater = (void (*)(struct __pyx_obj_9FastImage_FastImage8u *, int, struct __pyx_obj_9FastImage_FastImage8u *, struct __pyx_obj_9FastImage_Size *))__pyx_f_9FastImage_11FastImage8u_fast_get_compare_int_put_greater;
  __pyx_type_9FastImage_FastImage8u.tp_base = __pyx_ptype_9FastImage_FastImageBase;
  if (PyType_Ready(&__pyx_type_9FastImage_FastImage8u) < 0) __PYX_ERR(0, 338, __pyx_L1_error)
  __pyx_type_9FastImage_FastImage8u.tp_print = 0;
  if (__Pyx_SetVtable(__pyx_type_9FastImage_FastImage8u.tp_dict, __pyx_vtabptr_9FastImage_FastImage8u) < 0) __PYX_ERR(0, 338, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "FastImage8u", (PyObject *)&__pyx_type_9FastImage_FastImage8u) < 0) __PYX_ERR(0, 338, __pyx_L1_error)
  __pyx_ptype_9FastImage_FastImage8u = &__pyx_type_9FastImage_FastImage8u;
  /*--- Type import code ---*/
  /*--- Variable import code ---*/
  /*--- Function import code ---*/
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "FastImage.pyx":6
 * 
 * cimport c_lib
 * import traceback, sys             # <<<<<<<<<<<<<<
 * import numpy
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_traceback, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_traceback, __pyx_t_1) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_sys, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sys, __pyx_t_1) < 0) __PYX_ERR(0, 6, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "FastImage.pyx":7
 * cimport c_lib
 * import traceback, sys
 * import numpy             # <<<<<<<<<<<<<<
 * 
 * cdef int FASTIMAGEDEBUG
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_numpy, 0, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_numpy, __pyx_t_1) < 0) __PYX_ERR(0, 7, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "FastImage.pyx":10
 * 
 * cdef int FASTIMAGEDEBUG
 * FASTIMAGEDEBUG = 0             # <<<<<<<<<<<<<<
 * 
 * # create char types that work in Pyrex and Cython
 */
  __pyx_v_9FastImage_FASTIMAGEDEBUG = 0;

  /* "FastImage.pyx":14
 * # create char types that work in Pyrex and Cython
 * cdef char CHAR_u
 * CHAR_u = ord('u')             # <<<<<<<<<<<<<<
 * cdef char CHAR_f
 * CHAR_f = ord('f')
 */
  __pyx_v_9FastImage_CHAR_u = 0x75;

  /* "FastImage.pyx":16
 * CHAR_u = ord('u')
 * cdef char CHAR_f
 * CHAR_f = ord('f')             # <<<<<<<<<<<<<<
 * 
 * # for computing histograms
 */
  __pyx_v_9FastImage_CHAR_f = 0x66;

  /* "FastImage.pyx":20
 * # for computing histograms
 * cdef int n_hist_levels
 * n_hist_levels = 256             # <<<<<<<<<<<<<<
 * cdef ipp.Ipp32s hist_levels[257]
 * for i in range(0,257):
 */
  __pyx_v_9FastImage_n_hist_levels = 0x100;

  /* "FastImage.pyx":22
 * n_hist_levels = 256
 * cdef ipp.Ipp32s hist_levels[257]
 * for i in range(0,257):             # <<<<<<<<<<<<<<
 *     hist_levels[i]=i
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_builtin_range, __pyx_tuple__33, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (likely(PyList_CheckExact(__pyx_t_1)) || PyTuple_CheckExact(__pyx_t_1)) {
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_3 = 0;
    __pyx_t_4 = NULL;
  } else {
    __pyx_t_3 = -1; __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 22, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = Py_TYPE(__pyx_t_2)->tp_iternext; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 22, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  for (;;) {
    if (likely(!__pyx_t_4)) {
      if (likely(PyList_CheckExact(__pyx_t_2))) {
        if (__pyx_t_3 >= PyList_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 22, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      } else {
        if (__pyx_t_3 >= PyTuple_GET_SIZE(__pyx_t_2)) break;
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        __pyx_t_1 = PyTuple_GET_ITEM(__pyx_t_2, __pyx_t_3); __Pyx_INCREF(__pyx_t_1); __pyx_t_3++; if (unlikely(0 < 0)) __PYX_ERR(0, 22, __pyx_L1_error)
        #else
        __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_3); __pyx_t_3++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        #endif
      }
    } else {
      __pyx_t_1 = __pyx_t_4(__pyx_t_2);
      if (unlikely(!__pyx_t_1)) {
        PyObject* exc_type = PyErr_Occurred();
        if (exc_type) {
          if (likely(exc_type == PyExc_StopIteration || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) PyErr_Clear();
          else __PYX_ERR(0, 22, __pyx_L1_error)
        }
        break;
      }
      __Pyx_GOTREF(__pyx_t_1);
    }
    if (PyDict_SetItem(__pyx_d, __pyx_n_s_i, __pyx_t_1) < 0) __PYX_ERR(0, 22, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "FastImage.pyx":23
 * cdef ipp.Ipp32s hist_levels[257]
 * for i in range(0,257):
 *     hist_levels[i]=i             # <<<<<<<<<<<<<<
 * 
 * def set_debug(int val):
 */
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyInt_As_Ipp32s(__pyx_t_1); if (unlikely((__pyx_t_5 == ((Ipp32s)-1)) && PyErr_Occurred())) __PYX_ERR(0, 23, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = __Pyx_GetModuleGlobalName(__pyx_n_s_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 23, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = __Pyx_PyIndex_AsSsize_t(__pyx_t_1); if (unlikely((__pyx_t_6 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 23, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    (__pyx_v_9FastImage_hist_levels[__pyx_t_6]) = __pyx_t_5;

    /* "FastImage.pyx":22
 * n_hist_levels = 256
 * cdef ipp.Ipp32s hist_levels[257]
 * for i in range(0,257):             # <<<<<<<<<<<<<<
 *     hist_levels[i]=i
 * 
 */
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":25
 *     hist_levels[i]=i
 * 
 * def set_debug(int val):             # <<<<<<<<<<<<<<
 *     global FASTIMAGEDEBUG
 *     FASTIMAGEDEBUG = val
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9FastImage_1set_debug, NULL, __pyx_n_s_FastImage); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_set_debug, __pyx_t_2) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":63
 *     cdef int IsIPPStatic()
 * 
 * InitIPP()             # <<<<<<<<<<<<<<
 * 
 * def get_IPP_version():
 */
  InitIPP();

  /* "FastImage.pyx":65
 * InitIPP()
 * 
 * def get_IPP_version():             # <<<<<<<<<<<<<<
 *     cdef ipp_version_struct_t val
 *     val = GetIPPVersion()
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9FastImage_3get_IPP_version, NULL, __pyx_n_s_FastImage); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_IPP_version, __pyx_t_2) < 0) __PYX_ERR(0, 65, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":70
 *     return val.major, val.minor, val.build
 * 
 * def get_IPP_arch():             # <<<<<<<<<<<<<<
 *     cdef const char* arch_str
 *     arch_str = GetIPPArch()
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9FastImage_5get_IPP_arch, NULL, __pyx_n_s_FastImage); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_IPP_arch, __pyx_t_2) < 0) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":75
 *     return arch_str
 * 
 * def get_IPP_static():             # <<<<<<<<<<<<<<
 *     return IsIPPStatic()
 * 
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9FastImage_7get_IPP_static, NULL, __pyx_n_s_FastImage); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_get_IPP_static, __pyx_t_2) < 0) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":78
 *     return IsIPPStatic()
 * 
 * class IppError(Exception):             # <<<<<<<<<<<<<<
 *     def __init__(self, int errval):
 *         cdef const char* cmsg
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __Pyx_GIVEREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __pyx_t_1 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_Py3MetaclassPrepare(__pyx_t_1, __pyx_t_2, __pyx_n_s_IppError, __pyx_n_s_IppError, (PyObject *) NULL, __pyx_n_s_FastImage, (PyObject *) NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "FastImage.pyx":79
 * 
 * class IppError(Exception):
 *     def __init__(self, int errval):             # <<<<<<<<<<<<<<
 *         cdef const char* cmsg
 *         cmsg = ippGetStatusString(<ipp.IppStatus>errval)
 */
  __pyx_t_8 = __Pyx_CyFunction_NewEx(&__pyx_mdef_9FastImage_8IppError_1__init__, 0, __pyx_n_s_IppError___init, NULL, __pyx_n_s_FastImage, __pyx_d, ((PyObject *)__pyx_codeobj__42)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyObject_SetItem(__pyx_t_7, __pyx_n_s_init, __pyx_t_8) < 0) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;

  /* "FastImage.pyx":78
 *     return IsIPPStatic()
 * 
 * class IppError(Exception):             # <<<<<<<<<<<<<<
 *     def __init__(self, int errval):
 *         cdef const char* cmsg
 */
  __pyx_t_8 = __Pyx_Py3ClassCreate(__pyx_t_1, __pyx_n_s_IppError, __pyx_t_2, __pyx_t_7, NULL, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_IppError, __pyx_t_8) < 0) __PYX_ERR(0, 78, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":85
 *         Exception.__init__(self,cmsg)
 * 
 * class FicError(Exception):             # <<<<<<<<<<<<<<
 *     pass
 * 
 */
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __Pyx_GIVEREF(((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)(&((PyTypeObject*)PyExc_Exception)[0])));
  __pyx_t_1 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_7 = __Pyx_Py3MetaclassPrepare(__pyx_t_1, __pyx_t_2, __pyx_n_s_FicError, __pyx_n_s_FicError, (PyObject *) NULL, __pyx_n_s_FastImage, (PyObject *) NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_Py3ClassCreate(__pyx_t_1, __pyx_n_s_FicError, __pyx_t_2, __pyx_t_7, NULL, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_FicError, __pyx_t_8) < 0) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":113
 * 
 * # comparision operators
 * CmpLess = ipp.ippCmpLess             # <<<<<<<<<<<<<<
 * CmpLessEq = ipp.ippCmpLessEq
 * CmpEq = ipp.ippCmpEq
 */
  __pyx_t_2 = __Pyx_PyInt_From_IppCmpOp(ippCmpLess); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CmpLess, __pyx_t_2) < 0) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":114
 * # comparision operators
 * CmpLess = ipp.ippCmpLess
 * CmpLessEq = ipp.ippCmpLessEq             # <<<<<<<<<<<<<<
 * CmpEq = ipp.ippCmpEq
 * CmpGreaterEq = ipp.ippCmpGreaterEq
 */
  __pyx_t_2 = __Pyx_PyInt_From_IppCmpOp(ippCmpLessEq); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CmpLessEq, __pyx_t_2) < 0) __PYX_ERR(0, 114, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":115
 * CmpLess = ipp.ippCmpLess
 * CmpLessEq = ipp.ippCmpLessEq
 * CmpEq = ipp.ippCmpEq             # <<<<<<<<<<<<<<
 * CmpGreaterEq = ipp.ippCmpGreaterEq
 * CmpGreater = ipp.ippCmpGreater
 */
  __pyx_t_2 = __Pyx_PyInt_From_IppCmpOp(ippCmpEq); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CmpEq, __pyx_t_2) < 0) __PYX_ERR(0, 115, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":116
 * CmpLessEq = ipp.ippCmpLessEq
 * CmpEq = ipp.ippCmpEq
 * CmpGreaterEq = ipp.ippCmpGreaterEq             # <<<<<<<<<<<<<<
 * CmpGreater = ipp.ippCmpGreater
 * 
 */
  __pyx_t_2 = __Pyx_PyInt_From_IppCmpOp(ippCmpGreaterEq); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CmpGreaterEq, __pyx_t_2) < 0) __PYX_ERR(0, 116, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":117
 * CmpEq = ipp.ippCmpEq
 * CmpGreaterEq = ipp.ippCmpGreaterEq
 * CmpGreater = ipp.ippCmpGreater             # <<<<<<<<<<<<<<
 * 
 * cdef class Size:
 */
  __pyx_t_2 = __Pyx_PyInt_From_IppCmpOp(ippCmpGreater); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_CmpGreater, __pyx_t_2) < 0) __PYX_ERR(0, 117, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":783
 *         return dest
 * 
 * def asfastimage( object arr ):             # <<<<<<<<<<<<<<
 *     """return a FastImage view of an array"""
 *     cdef PyArrayInterface* inter
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9FastImage_9asfastimage, NULL, __pyx_n_s_FastImage); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 783, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_asfastimage, __pyx_t_2) < 0) __PYX_ERR(0, 783, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":834
 *     return result
 * 
 * def copy( object arr ):             # <<<<<<<<<<<<<<
 *     """return a FastImage copy of an array"""
 *     cdef PyArrayInterface* inter
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_9FastImage_11copy, NULL, __pyx_n_s_FastImage); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 834, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_copy, __pyx_t_2) < 0) __PYX_ERR(0, 834, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "FastImage.pyx":1
 * #emacs, this is -*-Python-*- mode             # <<<<<<<<<<<<<<
 * 
 * # indexing starts from left, bottom
 */
  __pyx_t_2 = PyDict_New(); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init FastImage", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_DECREF(__pyx_m); __pyx_m = 0;
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init FastImage");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if PY_MAJOR_VERSION < 3
  return;
  #else
  return __pyx_m;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule((char *)modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, (char *)"RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_CheckExact(key)) || likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (PyUnicode_GET_SIZE(**name) != PyUnicode_GET_SIZE(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (PyUnicode_GET_SIZE(**argname) != PyUnicode_GET_SIZE(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
#include "frameobject.h"
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = PyThreadState_GET();
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = f->f_localsplus;
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, int nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif  // CPython < 3.6
#endif  // CYTHON_FAST_PYCALL

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    PyObject *result;
    int flags;
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    return (*((__Pyx_PyCFunctionFast)meth)) (self, args, nargs, NULL);
}
#endif  // CYTHON_FAST_PYCCALL

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = func->ob_type->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* GetModuleGlobalName */
static CYTHON_INLINE PyObject *__Pyx_GetModuleGlobalName(PyObject *name) {
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
    result = PyDict_GetItem(__pyx_d, name);
    if (likely(result)) {
        Py_INCREF(result);
    } else {
#else
    result = PyObject_GetItem(__pyx_d, name);
    if (!result) {
        PyErr_Clear();
#endif
        result = __Pyx_GetBuiltinName(name);
    }
    return result;
}

/* PyObjectCallMethO */
  #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
  #if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (PyCFunction_GET_FLAGS(func) & METH_FASTCALL) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyErrFetchRestore */
    #if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* RaiseException */
    #if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
#if PY_VERSION_HEX >= 0x03030000
    if (cause) {
#else
    if (cause && cause != Py_None) {
#endif
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = PyThreadState_GET();
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* ExtTypeTest */
      static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(PyObject_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* KeywordStringCheck */
      static CYTHON_INLINE int __Pyx_CheckKeywordStrings(
    PyObject *kwdict,
    const char* function_name,
    int kw_allowed)
{
    PyObject* key = 0;
    Py_ssize_t pos = 0;
#if CYTHON_COMPILING_IN_PYPY
    if (!kw_allowed && PyDict_Next(kwdict, &pos, &key, 0))
        goto invalid_keyword;
    return 1;
#else
    while (PyDict_Next(kwdict, &pos, &key, 0)) {
        #if PY_MAJOR_VERSION < 3
        if (unlikely(!PyString_CheckExact(key)) && unlikely(!PyString_Check(key)))
        #endif
            if (unlikely(!PyUnicode_Check(key)))
                goto invalid_keyword_type;
    }
    if ((!kw_allowed) && unlikely(key))
        goto invalid_keyword;
    return 1;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    return 0;
#endif
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
    return 0;
}

/* ArgTypeTest */
      static void __Pyx_RaiseArgumentTypeInvalid(const char* name, PyObject *obj, PyTypeObject *type) {
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
}
static CYTHON_INLINE int __Pyx_ArgTypeTest(PyObject *obj, PyTypeObject *type, int none_allowed,
    const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (none_allowed && obj == Py_None) return 1;
    else if (exact) {
        if (likely(Py_TYPE(obj) == type)) return 1;
        #if PY_MAJOR_VERSION == 2
        else if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(PyObject_TypeCheck(obj, type))) return 1;
    }
    __Pyx_RaiseArgumentTypeInvalid(name, obj, type);
    return 0;
}

/* WriteUnraisableException */
      static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* BytesEquals */
      static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
      static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* PyObjectCallMethod1 */
      static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method, *result = NULL;
    method = __Pyx_PyObject_GetAttrStr(obj, method_name);
    if (unlikely(!method)) goto done;
#if CYTHON_UNPACK_METHODS
    if (likely(PyMethod_Check(method))) {
        PyObject *self = PyMethod_GET_SELF(method);
        if (likely(self)) {
            PyObject *args;
            PyObject *function = PyMethod_GET_FUNCTION(method);
            #if CYTHON_FAST_PYCALL
            if (PyFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            #if CYTHON_FAST_PYCCALL
            if (__Pyx_PyFastCFunction_Check(function)) {
                PyObject *args[2] = {self, arg};
                result = __Pyx_PyCFunction_FastCall(function, args, 2);
                goto done;
            }
            #endif
            args = PyTuple_New(2);
            if (unlikely(!args)) goto done;
            Py_INCREF(self);
            PyTuple_SET_ITEM(args, 0, self);
            Py_INCREF(arg);
            PyTuple_SET_ITEM(args, 1, arg);
            Py_INCREF(function);
            Py_DECREF(method); method = NULL;
            result = __Pyx_PyObject_Call(function, args, NULL);
            Py_DECREF(args);
            Py_DECREF(function);
            return result;
        }
    }
#endif
    result = __Pyx_PyObject_CallOneArg(method, arg);
done:
    Py_XDECREF(method);
    return result;
}

/* append */
      static CYTHON_INLINE int __Pyx_PyObject_Append(PyObject* L, PyObject* x) {
    if (likely(PyList_CheckExact(L))) {
        if (unlikely(__Pyx_PyList_Append(L, x) < 0)) return -1;
    } else {
        PyObject* retval = __Pyx_PyObject_CallMethod1(L, __pyx_n_s_append, x);
        if (unlikely(!retval))
            return -1;
        Py_DECREF(retval);
    }
    return 0;
}

/* StringJoin */
      #if !CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values) {
    return PyObject_CallMethodObjArgs(sep, __pyx_n_s_join, values, NULL);
}
#endif

/* RaiseTooManyValuesToUnpack */
      static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
      static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
      static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = PyThreadState_GET();
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(exc_type == PyExc_StopIteration) || PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
      static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* PyObjectCallNoArg */
      #if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || PyObject_TypeCheck(func, __pyx_CyFunctionType))) {
#else
    if (likely(PyCFunction_Check(func))) {
#endif
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyIntBinop */
        #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED int inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) {
            Py_RETURN_TRUE;
        } else {
            Py_RETURN_FALSE;
        }
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        const long b = intval;
        long a;
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        if (likely(__Pyx_sst_abs(size) <= 1)) {
            a = likely(size) ? digits[0] : 0;
            if (size == -1) a = -a;
        } else {
            switch (size) {
                case -2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 2:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        a = (long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case -3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 3:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        a = (long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case -4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                case 4:
                    if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                        a = (long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                        break;
                    }
                #if PyLong_SHIFT < 30 && PyLong_SHIFT != 15
                default: return PyLong_Type.tp_richcompare(op1, op2, Py_EQ);
                #else
                default: Py_RETURN_FALSE;
                #endif
            }
        }
            if (a == b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
            if ((double)a == (double)b) {
                Py_RETURN_TRUE;
            } else {
                Py_RETURN_FALSE;
            }
    }
    return PyObject_RichCompare(op1, op2, Py_EQ);
}
#endif

/* PyFloatBinop */
        #if !CYTHON_COMPILING_IN_PYPY
static PyObject* __Pyx_PyFloat_EqObjC(PyObject *op1, PyObject *op2, double floatval, CYTHON_UNUSED int inplace) {
    const double b = floatval;
    double a;
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    if (likely(PyFloat_CheckExact(op1))) {
        a = PyFloat_AS_DOUBLE(op1);
    } else
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        a = (double) PyInt_AS_LONG(op1);
    } else
    #endif
    if (likely(PyLong_CheckExact(op1))) {
        #if CYTHON_USE_PYLONG_INTERNALS
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        const Py_ssize_t size = Py_SIZE(op1);
        switch (size) {
            case  0: a = 0.0; break;
            case -1: a = -(double) digits[0]; break;
            case  1: a = (double) digits[0]; break;
            case -2:
            case 2:
                if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (1 * PyLong_SHIFT < 53))) {
                    a = (double) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53) || (a < (double) (1L<<53))) {
                        if (size == -2)
                            a = -a;
                        break;
                    }
                }
            case -3:
            case 3:
                if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (2 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53) || (a < (double) (1L<<53))) {
                        if (size == -3)
                            a = -a;
                        break;
                    }
                }
            case -4:
            case 4:
                if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT && ((8 * sizeof(unsigned long) < 53) || (3 * PyLong_SHIFT < 53))) {
                    a = (double) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0]));
                    if ((8 * sizeof(unsigned long) < 53) || (4 * PyLong_SHIFT < 53) || (a < (double) (1L<<53))) {
                        if (size == -4)
                            a = -a;
                        break;
                    }
                }
            default:
        #else
        {
        #endif
            return PyFloat_Type.tp_richcompare(op2, op1, Py_EQ);
        }
    } else {
        return PyObject_RichCompare(op1, op2, Py_EQ);
    }
        if (a == b) {
            Py_RETURN_TRUE;
        } else {
            Py_RETURN_FALSE;
        }
}
#endif

/* None */
          static CYTHON_INLINE long __Pyx_div_long(long a, long b) {
    long q = a / b;
    long r = a - q*b;
    q -= ((r != 0) & ((r ^ b) < 0));
    return q;
}

/* SetVTable */
          static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* Import */
          static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_VERSION_HEX < 0x03030000
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if (strchr(__Pyx_MODULE_NAME, '.')) {
                #if PY_VERSION_HEX < 0x03030000
                PyObject *py_level = PyInt_FromLong(1);
                if (!py_level)
                    goto bad;
                module = PyObject_CallFunctionObjArgs(py_import,
                    name, global_dict, empty_dict, list, py_level, NULL);
                Py_DECREF(py_level);
                #else
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                #endif
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_VERSION_HEX < 0x03030000
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_VERSION_HEX < 0x03030000
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* CalculateMetaclass */
          static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases = PyTuple_GET_SIZE(bases);
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
        tmptype = Py_TYPE(tmp);
#if PY_MAJOR_VERSION < 3
        if (tmptype == &PyClass_Type)
            continue;
#endif
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
#if PY_MAJOR_VERSION < 3
        metaclass = &PyClass_Type;
#else
        metaclass = &PyType_Type;
#endif
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* FetchCommonType */
          static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunction */
          static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
#else
    if (unlikely(value == NULL || !PyString_Check(value))) {
#endif
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(__pyx_CyFunctionObject, func.m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromString(m->func.m_ml->ml_name);
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_New(PyTypeObject *type, PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    __pyx_CyFunctionObject *op = PyObject_GC_New(__pyx_CyFunctionObject, type);
    if (op == NULL)
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    PyObject_GC_Track(op);
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0 = PySequence_ITEM(arg, 0);
                if (unlikely(!arg0)) return NULL;
                result = (*meth)(self, arg0);
                Py_DECREF(arg0);
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (__pyx_CyFunctionType == NULL) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (!m->defaults)
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* Py3ClassCreate */
              static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStr(metaclass, __pyx_n_s_prepare);
        if (prep) {
            PyObject *pargs = PyTuple_Pack(2, name, bases);
            if (unlikely(!pargs)) {
                Py_DECREF(prep);
                return NULL;
            }
            ns = PyObject_Call(prep, pargs, mkw);
            Py_DECREF(prep);
            Py_DECREF(pargs);
        } else {
            if (unlikely(!PyErr_ExceptionMatches(PyExc_AttributeError)))
                return NULL;
            PyErr_Clear();
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_n_s_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_n_s_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result, *margs;
    PyObject *owned_metaclass = NULL;
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_n_s_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    margs = PyTuple_Pack(3, name, bases, dict);
    if (unlikely(!margs)) {
        result = NULL;
    } else {
        result = PyObject_Call(metaclass, margs, mkw);
        Py_DECREF(margs);
    }
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CodeObjectCache */
              static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, (size_t)new_max*sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
              #include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    py_code = __pyx_find_code_object(c_line ? c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        PyThreadState_GET(), /*PyThreadState *tstate,*/
        py_code,             /*PyCodeObject *code,*/
        __pyx_d,      /*PyObject *globals,*/
        0                    /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
              #define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_IppCmpOp(IppCmpOp value) {
    const IppCmpOp neg_one = (IppCmpOp) -1, const_zero = (IppCmpOp) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(IppCmpOp) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(IppCmpOp) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(IppCmpOp) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(IppCmpOp) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(IppCmpOp) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(IppCmpOp),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_IppStatus(IppStatus value) {
    const IppStatus neg_one = (IppStatus) -1, const_zero = (IppStatus) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(IppStatus) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(IppStatus) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(IppStatus) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(IppStatus) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(IppStatus) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(IppStatus),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_FicStatus(FicStatus value) {
    const FicStatus neg_one = (FicStatus) -1, const_zero = (FicStatus) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(FicStatus) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(FicStatus) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(FicStatus) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(FicStatus) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(FicStatus) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(FicStatus),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_intptr_t(intptr_t value) {
    const intptr_t neg_one = (intptr_t) -1, const_zero = (intptr_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(intptr_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(intptr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(intptr_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(intptr_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(intptr_t),
                                     little, !is_unsigned);
    }
}

/* Print */
              #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static PyObject *__Pyx_GetStdout(void) {
    PyObject *f = PySys_GetObject((char *)"stdout");
    if (!f) {
        PyErr_SetString(PyExc_RuntimeError, "lost sys.stdout");
    }
    return f;
}
static int __Pyx_Print(PyObject* f, PyObject *arg_tuple, int newline) {
    int i;
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    for (i=0; i < PyTuple_GET_SIZE(arg_tuple); i++) {
        PyObject* v;
        if (PyFile_SoftSpace(f, 1)) {
            if (PyFile_WriteString(" ", f) < 0)
                goto error;
        }
        v = PyTuple_GET_ITEM(arg_tuple, i);
        if (PyFile_WriteObject(v, f, Py_PRINT_RAW) < 0)
            goto error;
        if (PyString_Check(v)) {
            char *s = PyString_AsString(v);
            Py_ssize_t len = PyString_Size(v);
            if (len > 0) {
                switch (s[len-1]) {
                    case ' ': break;
                    case '\f': case '\r': case '\n': case '\t': case '\v':
                        PyFile_SoftSpace(f, 0);
                        break;
                    default:  break;
                }
            }
        }
    }
    if (newline) {
        if (PyFile_WriteString("\n", f) < 0)
            goto error;
        PyFile_SoftSpace(f, 0);
    }
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
}
#else
static int __Pyx_Print(PyObject* stream, PyObject *arg_tuple, int newline) {
    PyObject* kwargs = 0;
    PyObject* result = 0;
    PyObject* end_string;
    if (unlikely(!__pyx_print)) {
        __pyx_print = PyObject_GetAttr(__pyx_b, __pyx_n_s_print);
        if (!__pyx_print)
            return -1;
    }
    if (stream) {
        kwargs = PyDict_New();
        if (unlikely(!kwargs))
            return -1;
        if (unlikely(PyDict_SetItem(kwargs, __pyx_n_s_file, stream) < 0))
            goto bad;
        if (!newline) {
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                goto bad;
            if (PyDict_SetItem(kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                goto bad;
            }
            Py_DECREF(end_string);
        }
    } else if (!newline) {
        if (unlikely(!__pyx_print_kwargs)) {
            __pyx_print_kwargs = PyDict_New();
            if (unlikely(!__pyx_print_kwargs))
                return -1;
            end_string = PyUnicode_FromStringAndSize(" ", 1);
            if (unlikely(!end_string))
                return -1;
            if (PyDict_SetItem(__pyx_print_kwargs, __pyx_n_s_end, end_string) < 0) {
                Py_DECREF(end_string);
                return -1;
            }
            Py_DECREF(end_string);
        }
        kwargs = __pyx_print_kwargs;
    }
    result = PyObject_Call(__pyx_print, arg_tuple, kwargs);
    if (unlikely(kwargs) && (kwargs != __pyx_print_kwargs))
        Py_DECREF(kwargs);
    if (!result)
        return -1;
    Py_DECREF(result);
    return 0;
bad:
    if (kwargs != __pyx_print_kwargs)
        Py_XDECREF(kwargs);
    return -1;
}
#endif

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Ipp8u(Ipp8u value) {
    const Ipp8u neg_one = (Ipp8u) -1, const_zero = (Ipp8u) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(Ipp8u) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Ipp8u) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Ipp8u) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(Ipp8u) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Ipp8u) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(Ipp8u),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_Py_intptr_t(Py_intptr_t value) {
    const Py_intptr_t neg_one = (Py_intptr_t) -1, const_zero = (Py_intptr_t) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(Py_intptr_t) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Py_intptr_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(Py_intptr_t) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(Py_intptr_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(Py_intptr_t),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
              static CYTHON_INLINE PyObject* __Pyx_PyInt_From_char(char value) {
    const char neg_one = (char) -1, const_zero = (char) 0;
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(char) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(char) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(char) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(char) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(char) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(char),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
              static CYTHON_INLINE Ipp32s __Pyx_PyInt_As_Ipp32s(PyObject *x) {
    const Ipp32s neg_one = (Ipp32s) -1, const_zero = (Ipp32s) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(Ipp32s) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(Ipp32s, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (Ipp32s) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Ipp32s) 0;
                case  1: __PYX_VERIFY_RETURN_INT(Ipp32s, digit, digits[0])
                case 2:
                    if (8 * sizeof(Ipp32s) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Ipp32s, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Ipp32s) >= 2 * PyLong_SHIFT) {
                            return (Ipp32s) (((((Ipp32s)digits[1]) << PyLong_SHIFT) | (Ipp32s)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Ipp32s) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Ipp32s, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Ipp32s) >= 3 * PyLong_SHIFT) {
                            return (Ipp32s) (((((((Ipp32s)digits[2]) << PyLong_SHIFT) | (Ipp32s)digits[1]) << PyLong_SHIFT) | (Ipp32s)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Ipp32s) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Ipp32s, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Ipp32s) >= 4 * PyLong_SHIFT) {
                            return (Ipp32s) (((((((((Ipp32s)digits[3]) << PyLong_SHIFT) | (Ipp32s)digits[2]) << PyLong_SHIFT) | (Ipp32s)digits[1]) << PyLong_SHIFT) | (Ipp32s)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (Ipp32s) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(Ipp32s) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Ipp32s, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(Ipp32s) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Ipp32s, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (Ipp32s) 0;
                case -1: __PYX_VERIFY_RETURN_INT(Ipp32s, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(Ipp32s,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(Ipp32s) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Ipp32s, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Ipp32s) - 1 > 2 * PyLong_SHIFT) {
                            return (Ipp32s) (((Ipp32s)-1)*(((((Ipp32s)digits[1]) << PyLong_SHIFT) | (Ipp32s)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(Ipp32s) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Ipp32s, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Ipp32s) - 1 > 2 * PyLong_SHIFT) {
                            return (Ipp32s) ((((((Ipp32s)digits[1]) << PyLong_SHIFT) | (Ipp32s)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(Ipp32s) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Ipp32s, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Ipp32s) - 1 > 3 * PyLong_SHIFT) {
                            return (Ipp32s) (((Ipp32s)-1)*(((((((Ipp32s)digits[2]) << PyLong_SHIFT) | (Ipp32s)digits[1]) << PyLong_SHIFT) | (Ipp32s)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(Ipp32s) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Ipp32s, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Ipp32s) - 1 > 3 * PyLong_SHIFT) {
                            return (Ipp32s) ((((((((Ipp32s)digits[2]) << PyLong_SHIFT) | (Ipp32s)digits[1]) << PyLong_SHIFT) | (Ipp32s)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(Ipp32s) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Ipp32s, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Ipp32s) - 1 > 4 * PyLong_SHIFT) {
                            return (Ipp32s) (((Ipp32s)-1)*(((((((((Ipp32s)digits[3]) << PyLong_SHIFT) | (Ipp32s)digits[2]) << PyLong_SHIFT) | (Ipp32s)digits[1]) << PyLong_SHIFT) | (Ipp32s)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(Ipp32s) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(Ipp32s, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(Ipp32s) - 1 > 4 * PyLong_SHIFT) {
                            return (Ipp32s) ((((((((((Ipp32s)digits[3]) << PyLong_SHIFT) | (Ipp32s)digits[2]) << PyLong_SHIFT) | (Ipp32s)digits[1]) << PyLong_SHIFT) | (Ipp32s)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(Ipp32s) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(Ipp32s, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(Ipp32s) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(Ipp32s, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            Ipp32s val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (Ipp32s) -1;
        }
    } else {
        Ipp32s val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (Ipp32s) -1;
        val = __Pyx_PyInt_As_Ipp32s(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to Ipp32s");
    return (Ipp32s) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to Ipp32s");
    return (Ipp32s) -1;
}

/* CIntFromPy */
              static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
    const int neg_one = (int) -1, const_zero = (int) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* PrintOne */
              #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION < 3
static int __Pyx_PrintOne(PyObject* f, PyObject *o) {
    if (!f) {
        if (!(f = __Pyx_GetStdout()))
            return -1;
    }
    Py_INCREF(f);
    if (PyFile_SoftSpace(f, 0)) {
        if (PyFile_WriteString(" ", f) < 0)
            goto error;
    }
    if (PyFile_WriteObject(o, f, Py_PRINT_RAW) < 0)
        goto error;
    if (PyFile_WriteString("\n", f) < 0)
        goto error;
    Py_DECREF(f);
    return 0;
error:
    Py_DECREF(f);
    return -1;
    /* the line below is just to avoid C compiler
     * warnings about unused functions */
    return __Pyx_Print(f, NULL, 0);
}
#else
static int __Pyx_PrintOne(PyObject* stream, PyObject *o) {
    int res;
    PyObject* arg_tuple = PyTuple_Pack(1, o);
    if (unlikely(!arg_tuple))
        return -1;
    res = __Pyx_Print(stream, arg_tuple, 1);
    Py_DECREF(arg_tuple);
    return res;
}
#endif

/* CIntFromPy */
              static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
    const long neg_one = (long) -1, const_zero = (long) 0;
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CheckBinaryVersion */
              static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* InitStrings */
              static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
static CYTHON_INLINE char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if CYTHON_COMPILING_IN_CPYTHON && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
#if PY_VERSION_HEX < 0x03030000
        char* defenc_c;
        PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
        if (!defenc) return NULL;
        defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        {
            char* end = defenc_c + PyBytes_GET_SIZE(defenc);
            char* c;
            for (c = defenc_c; c < end; c++) {
                if ((unsigned char) (*c) >= 128) {
                    PyUnicode_AsASCIIString(o);
                    return NULL;
                }
            }
        }
#endif
        *length = PyBytes_GET_SIZE(defenc);
        return defenc_c;
#else
        if (__Pyx_PyUnicode_READY(o) == -1) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        if (PyUnicode_IS_ASCII(o)) {
            *length = PyUnicode_GET_LENGTH(o);
            return PyUnicode_AsUTF8(o);
        } else {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
#else
        return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (PyInt_Check(x) || PyLong_Check(x))
#else
  if (PyLong_Check(x))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = PyNumber_Long(x);
  }
  #else
  if (m && m->nb_int) {
    name = "int";
    res = PyNumber_Long(x);
  }
  #endif
#else
  res = PyNumber_Int(x);
#endif
  if (res) {
#if PY_MAJOR_VERSION < 3
    if (!PyInt_Check(res) && !PyLong_Check(res)) {
#else
    if (!PyLong_Check(res)) {
#endif
      PyErr_Format(PyExc_TypeError,
                   "__%.4s__ returned non-%.4s (type %.200s)",
                   name, name, Py_TYPE(res)->tp_name);
      Py_DECREF(res);
      return NULL;
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(x);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
